/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    apiSNMP.h
 *
 * Date:        June 1994
 *
 * Purpose:     Include for UT_SNMP application.
 *
 * Author(s):   Vincent Berkhout, Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */

#ifndef APISNMP_H
#define APISNMP_H

#include <snmpTypes.h>
#include <apiTYPES.h>

/* -----------------------------------------------------------------------
 *
 *
 *	TYPES, FUNCTIONS & MACROS
 *
 *
 * -----------------------------------------------------------------------
 */
#define INTEGER AsnInt
#define VARLIST VarBindList
typedef char NAME[256];
typedef NAME DESCRIPTION;


/* 
 *	the HOW definition.
 */
typedef enum {
	GRACEFULL = 0,
	DISCARD = 1
} HOW;

/* 
 *	Possible roles an entity may perform.
 */
typedef enum {
	MANAGER	= 1, 
	AGENT = 2,
	MANAGER_AGENT = 3
} ROLE;


/* 
 *	Possible SNMP (API) Errors.
 */
typedef enum {
 	SnmpErrUNKNOWN = 0,
	SnmpErrTIMEOUT = 1,
	SnmpErrNOSUCHCONTEXT = 2,
	SnmpErrNOSUCHHOST = 3,
	SnmpErrQOSFAILED = 4,
	SnmpErrNODOMAIN = 5,
	SnmpErrNOPACKET = 6,
	SnmpErrTOOMANYYVBS = 7,
	SnmpErrQOSUNKNOWN = 8
} ERRTYPE;

/*
 *	Definition of Service Primitive types/management communication
 *	type.
 */
typedef enum {
	GET = 0, 
	GETNEXT = 1, 
	GETBULK	= 5,
	SET = 3, 
	INFORM = 6, 
	TRAP = 7, 
	COMMIT=8, 
	UNDO=9
} TYPE;

#define REQUEST		0xA0
#define INDICATION	0xB0
#define	RESPONSE	0xC0
#define CONFIRM		0xD0
#define NEG_CONFIRM	0xE0
#define ADMIN		0x90
#define STATUS_		0x70
#define META		0x50

typedef enum{
	GET_REQ		= GET + REQUEST,
	GET_IND		= GET + INDICATION,
	GET_RSP	 	= GET + RESPONSE,
	GET_CNF		= GET + CONFIRM,

	META_GET_REQ	= META + GET + REQUEST,
	META_GET_CNF	= META + GET + CONFIRM,

	GETNEXT_REQ 	= GETNEXT + REQUEST,
	GETNEXT_IND	= GETNEXT + INDICATION,
	GETNEXT_RSP	= GETNEXT + RESPONSE,
	GETNEXT_CNF	= GETNEXT + CONFIRM,

	META_GETNEXT_REQ= META + GETNEXT + REQUEST,
	META_GETNEXT_CNF= META + GETNEXT + CONFIRM,

	GETBULK_REQ	= GETBULK + REQUEST,
	GETBULK_IND	= GETBULK + INDICATION,
	GETBULK_RSP	= GETBULK + RESPONSE,
	GETBULK_CNF	= GETBULK + CONFIRM,

	SET_REQ		= SET + REQUEST,
	SET_IND		= SET + INDICATION,
	SET_RSP		= SET + RESPONSE,
	SET_CNF		= SET + CONFIRM,

	META_SET_REQ	= META + SET + REQUEST,
	META_SET_CNF	= META + SET + CONFIRM,

	META_UNDO_REQ	= META + UNDO + REQUEST,
	META_UNDO_CNF	= META + UNDO + CONFIRM,

	META_COMMIT_REQ	= META + COMMIT + REQUEST,
	META_COMMIT_CNF	= META + COMMIT + CONFIRM,

	INFORM_REQ	= INFORM + REQUEST,
	INFORM_IND	= INFORM + INDICATION,
	INFORM_RSP	= INFORM + RESPONSE,
	INFORM_CNF	= INFORM + CONFIRM,
	TRAP_REQ	= TRAP + REQUEST,
	TRAP_IND	= TRAP + INDICATION,

	GET_NEGCNF	= GET + NEG_CONFIRM,
	GETNEXT_NEGCNF	= GETNEXT + NEG_CONFIRM,
	GETBULK_NEGCNF	= GETBULK + NEG_CONFIRM,
	SET_NEGCNF	= SET + NEG_CONFIRM,
	INFORM_NEGCNF	= INFORM + NEG_CONFIRM,

	TIMEOUT		= STATUS_ + 0,
	NONSNMP		= STATUS_ + 1,
	ERR_IND		= STATUS_ + 2,

	AGENT_ADM 	= ADMIN + 0,
	MANAGER_ADM	= ADMIN + 1,
	AGENTANDMAN_ADM = ADMIN + 2
	
} SPTYPE;
	
/*
 *	Quality Of Service definition
 */
typedef enum {
	qosNONE = 0,
	qosAUTH = 1,
	qosPRIV = 2
} QOS;

/*
 *	External declared api functions.
 */
EXTERN STATUS snmpOpen( NAME application, char **arge );
EXTERN STATUS snmpLook( SPTYPE *eventType );
EXTERN STATUS snmpClose( HOW how );
EXTERN STATUS snmpErrorDescription( INTEGER *requestId, DESCRIPTION error );
EXTERN STATUS snmpReceiveIndication( SPTYPE mType, INTEGER *mReqId, 
			INTEGER *mErrInd, INTEGER *mErrStat, 
			char *mConName, VARLIST **mVarBinds );
EXTERN STATUS snmpReceiveNotification( SPTYPE mType, char *mSysName, 
			char *mConName, VARLIST **mVarBinds );
EXTERN STATUS snmpReceiveConfirm( SPTYPE mType, INTEGER *mReqId, 
			INTEGER *mErrStat, INTEGER *mErrInd, 
			VARLIST **mVarBinds );
EXTERN STATUS snmpReceiveNegConfirm( SPTYPE mType, INTEGER *mReqId );
/* EXTERN STATUS snmpSend( EXTERN IADDRESS remoteAddress, MSG *mMsg ); */
EXTERN STATUS snmpSendRequest( SPTYPE mType, char *mSysName, INTEGER mReqId,
			QOS mQos, INTEGER mErrInd, INTEGER mErrStat,
			char *mConName, VARLIST *mVarBinds );
EXTERN STATUS snmpSendResponse( SPTYPE mType, INTEGER mReqId, INTEGER mErrStat,
			INTEGER mErrInd, VARLIST *mVarBinds );
EXTERN STATUS snmpGetRequest( NAME host, NAME application,
			INTEGER requestId, QOS qos, VARLIST *variableNames );
EXTERN STATUS snmpGetIndication( INTEGER *replyId, VARLIST **variableNames );
EXTERN STATUS snmpGetResponse( INTEGER replyId, INTEGER errorStatus,
			INTEGER errorIndex, VARLIST *variableValues );
EXTERN STATUS snmpGetConfirm( INTEGER *requestId, INTEGER *errorStatus, 
			INTEGER *errorIndex, VARLIST **variableValues );
EXTERN STATUS snmpGetNextRequest( NAME host, NAME application, 
			INTEGER requestId, QOS qos, VARLIST *variableNames );
EXTERN STATUS snmpGetNextIndication( INTEGER *replyId, VARLIST **variableNames);
EXTERN STATUS snmpGetNextResponse( INTEGER replyId, INTEGER errorStatus,
			INTEGER errorIndex, VARLIST *variableValues );
EXTERN STATUS snmpGetNextConfirm( INTEGER *requestId, INTEGER *errorStatus,
			INTEGER *errorIndex, VARLIST **variableValues );
EXTERN STATUS snmpGetBulkRequest( NAME host, NAME application, INTEGER requestId,
			QOS qos, INTEGER maxRepetitions,
			INTEGER nonRepeaters, VARLIST *variableNames );
EXTERN STATUS snmpGetBulkIndication( INTEGER *replyId, INTEGER *maxRepetitions, 
			INTEGER *nonRepeaters, VARLIST **variableNames );
EXTERN STATUS snmpGetBulkResponse( INTEGER replyId, INTEGER errorStatus, 
			INTEGER errorIndex, VARLIST *variableValues );
EXTERN STATUS snmpGetBulkConfirm( INTEGER *requestId, INTEGER *errorStatus,
			INTEGER *errorIndex, VARLIST **variableValues );
EXTERN STATUS snmpSetRequest( NAME host, NAME application, INTEGER requestId,
			QOS qos, VARLIST *variableValues );
EXTERN STATUS snmpSetIndication( INTEGER *replyId, VARLIST **variableValues );
EXTERN STATUS snmpSetResponse( INTEGER replyId, INTEGER errorStatus,
			INTEGER errorIndex );
EXTERN STATUS snmpSetConfirm( INTEGER *requestId, INTEGER *errorStatus, 
			INTEGER *errorIndex );
EXTERN STATUS snmpInformRequest( NAME host, NAME application, 
			INTEGER requestId, QOS qos, VARLIST *variableValues );
EXTERN STATUS snmpInformIndication( NAME host, NAME application,
			VARLIST **variableValues );
EXTERN STATUS snmpInformConfirm( INTEGER *requestId, INTEGER *errorStatus,
			INTEGER *errorIndex, VARLIST **variableValues );
EXTERN STATUS snmpTrapRequest( VARLIST *variableValues );
EXTERN STATUS snmpTrapIndication( NAME host, NAME application,
			VARLIST **variableValues );

EXTERN STATUS snmpMetaGetRequest( INTEGER requestId, VARLIST *variableNames );
EXTERN STATUS snmpMetaGetConfirm( INTEGER *requestId, INTEGER *errorStatus, 
			INTEGER *errorIndex, VARLIST **variableValues );
EXTERN STATUS snmpMetaGetNextRequest( INTEGER requestId, VARLIST *variableNames );
EXTERN STATUS snmpMetaGetNextConfirm( INTEGER *requestId, INTEGER *errorStatus, 
			INTEGER *errorIndex, VARLIST **variableValues );
EXTERN STATUS snmpMetaSetRequest( INTEGER requestId, VARLIST *variableValues );
EXTERN STATUS snmpMetaSetConfirm( INTEGER *requestId, INTEGER *errorStatus, 
			INTEGER *errorIndex );

static int SNMP_DEBUG_API = 100;

#endif /* APISNMP_H */

