/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:    snmpTypes.h
 *
 * Date:        May 1994
 *
 * Purpose:     Type definitions and mappings.
 *
 * Author(s):   H.C. Scholtes, J.W.Togtema, 
 *		Vincent Berkhout, Eric van Hengstum
 *
 * -----------------------------------------------------------------------
 */

#ifndef SNMPTYPES_H
#define SNMPTYPES_H

#include <snmpConst.h>
#include <snacc/asn_incl.h>
#include <snmpv2_mess.h>


/* ^L
 * -----------------------------------------------------------------------
 *
 *	RFC 1443, PAGE 5: DisplayString
 *
 * -----------------------------------------------------------------------
 */
typedef AsnOcts DisplayString;


/* ^L
 * -----------------------------------------------------------------------
 *
 *	RFC 1443, PAGE 5: PhysAddress
 *
 * -----------------------------------------------------------------------
 */
typedef AsnOcts PhysAddress;


/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1443, PAGE 5: MacAddress
 *
 * -----------------------------------------------------------------------
 */
typedef AsnOcts MacAddress;


/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1443, PAGE 5: TruthValue
 *
 * -----------------------------------------------------------------------
 */
typedef enum {
	true     = 1,
	false    = 2
} TruthValue;


/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1443, PAGE 6: TestAndIncr
 *
 * -----------------------------------------------------------------------
 */
typedef AsnInt TestAndIncr; 


/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1443, PAGE 7: Autonomous Type
 *
 * -----------------------------------------------------------------------
 */
typedef AsnOid AutonomousType;


/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1443, PAGE 7: InstancePointer
 */
typedef AsnOid InstancePointer;

/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1443, PAGE 8: RowStatus
 *
 * -----------------------------------------------------------------------
 */
typedef enum {
	active          = 1,
	notInService    = 2,
	notReady        = 3,
	createAndGo     = 4,
	createAndWait   = 5,
	destroy         = 6
} RowStatus;


/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1443, PAGE 20: TimeStamp
 *
 * -----------------------------------------------------------------------
 */
typedef TimeTicks TimeStamp;



/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1443, PAGE 20: TimeInterval
 *
 * -----------------------------------------------------------------------
 */
typedef AsnInt TimeInterval;


/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1443, PAGE 21: DateAndTime
 *
 * -----------------------------------------------------------------------
 */
typedef AsnOcts dateAndTime;


/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1447, PAGE 4: Party
 *
 * -----------------------------------------------------------------------
 */
typedef AsnOid Party;


/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1447, PAGE 4, TAddress.
 *
 * -----------------------------------------------------------------------
 */
typedef AsnOcts TAddress;


/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1447, PAGE 5, Clock.
 *
 * -----------------------------------------------------------------------
 */
typedef UInteger32 Clock;


/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1447, PAGE 5, Context.
 *
 * -----------------------------------------------------------------------
 */
typedef AsnOid Context;


/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1447, PAGE 6, StorageType.
 *
 * -----------------------------------------------------------------------
 */
typedef enum {
	other           = 1,
	Volatile	= 2,
	nonVolatile     = 3,
	permanent       = 4
} StorageType;

EXTERN AsnInt max_bindings ;
EXTERN AsnOid iso ;
EXTERN AsnOid org ;
EXTERN AsnOid dod ;
EXTERN AsnOid internet ;
EXTERN AsnOid directory ;
EXTERN AsnOid mgmt ;
EXTERN AsnOid mib2 ;
EXTERN AsnOid asnsystem ;
EXTERN AsnOid sysDescr ;
EXTERN AsnOid sysObjectID ;
EXTERN AsnOid sysUpTime ;
EXTERN AsnOid sysContact ;
EXTERN AsnOid sysName ;
EXTERN AsnOid sysLocation ;
EXTERN AsnOid sysServices ;
EXTERN AsnOid interfaces ;
EXTERN AsnOid at ;
EXTERN AsnOid ip ;
EXTERN AsnOid icmp ;
EXTERN AsnOid tcp ;
EXTERN AsnOid udp ;
EXTERN AsnOid egp ; 
EXTERN AsnOid snmp ;
EXTERN AsnOid experimental ;
EXTERN AsnOid private ;
EXTERN AsnOid enterprises ;
EXTERN AsnOid utV1domain ;
EXTERN AsnOid snmpV2 ;
EXTERN AsnOid snmpDomains ;
EXTERN AsnOid snmpUDPDomain ;
EXTERN AsnOid snmpCLNSDomain ;
EXTERN AsnOid snmpCONSDomain ;
EXTERN AsnOid snmpDDPDomain ;
EXTERN AsnOid snmpIPXDomain ;
EXTERN AsnOid snmpTCPDomain ;
EXTERN AsnOid snmpProxys ;
EXTERN AsnOid rfc1157Proxy;
EXTERN AsnOid rfc1157Domain;
EXTERN AsnOid rfc1157noAuth;
EXTERN AsnOid snmpModules ;
EXTERN AsnOid snmpMIB    ;

/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1450, PAGE 3, snmpMIBObjects.
 *
 * -----------------------------------------------------------------------
 */
EXTERN AsnOid snmpMIBObjects ;

EXTERN AsnOid snmpStats ;
EXTERN AsnOid snmpV1 ;
EXTERN AsnOid snmpOR ;
EXTERN AsnOid snmpTrap ;
EXTERN AsnOid snmpTrapOID ;
EXTERN AsnOid snmpTrapTable ;
EXTERN AsnOid snmpTrapEntry ;
EXTERN AsnOid snmpTrapNumbers ;
EXTERN AsnOid snmpTrapEnterprise ;
EXTERN AsnOid snmpTraps ;
EXTERN AsnOid snmpSet ;
EXTERN AsnOid coldStart ;
EXTERN AsnOid warmStart ;
EXTERN AsnOid linkDown ;
EXTERN AsnOid linkUp  ;
EXTERN AsnOid authenticationFailure ;
EXTERN AsnOid egpNeighborLoss ;

/* 
 * -----------------------------------------------------------------------
 *
 *	RFC 1447, PAGE 3, partyMIB.
 *	RFC 1447, PAGE 7, partyAdmin, partyProtocol, noAuth, noPriv,
 *			  desPrivProtocol, v2md5AuthProtocol, 
 *			  currentTime, cacheTime, restartTime.
 *	RFC 1447, PAGE 8, initialPartyId, initialContextId.
 *
 * -----------------------------------------------------------------------
 */
EXTERN AsnOid partyMIB ;
EXTERN AsnOid partyAdmin ;
EXTERN AsnOid partyProtocols ;
EXTERN AsnOid noAuth ;
EXTERN AsnOid noPriv ;
EXTERN AsnOid v2md5AuthProtocol ;
EXTERN AsnOid desPrivProtocol ;
EXTERN AsnOid currentTime ;
EXTERN AsnOid restartTime ;
EXTERN AsnOid cacheTime ;
EXTERN AsnOid initialPartyId ;
EXTERN AsnOid initialContextId ;

EXTERN AsnOid partyMIBObjects ;
EXTERN AsnOid snmpParties ;
EXTERN AsnOid partyTable ;
EXTERN AsnOid snmpContexts ;
EXTERN AsnOid ContextTable ;
EXTERN AsnOid snmpAccess ;
EXTERN AsnOid aclTable   ;
EXTERN AsnOid snmpViews ;
EXTERN AsnOid viewTable ;

EXTERN AsnOid snmpM2M;
EXTERN AsnOid snmpM2MObjects;
EXTERN AsnOid snmpAlarm;
EXTERN AsnOid snmpEvent;
EXTERN AsnOid snmpEventNextIndex;
EXTERN AsnOid snmpEventTable;
EXTERN AsnOid snmpEventEntry;
EXTERN AsnOid snmpEventIndex;
EXTERN AsnOid snmpEventID;
EXTERN AsnOid snmpEventDescription;
EXTERN AsnOid snmpEventEvents;
EXTERN AsnOid snmpEventLastTimeSent;
EXTERN AsnOid snmpEventStatus;
EXTERN AsnOid snmpEventNotifyMinInterval;
EXTERN AsnOid snmpEventNotifyMaxRetransmissions;
EXTERN AsnOid snmpEventNotifyTable;
EXTERN AsnOid snmpEventNotifyEntry;
EXTERN AsnOid snmpEventNotifyIntervalRequested;
EXTERN AsnOid snmpEventNotifyRetransmissionsRequested;
EXTERN AsnOid snmpEventNotifyLifetime;
EXTERN AsnOid snmpEventNotifyStatus	;

/*
 * 	VOLATILE memory routines 
 */
EXTERN AsnOcts*	NewOct( UAsnInt size );
EXTERN void FreeOct( AsnOcts *o );

#define	NewOid	NewOct
#define FreeOid	FreeOct

#define	NewVbl() AsnListNew(sizeof(char *))

/*
 * 	PERMANENT memory routines 
 */
EXTERN AsnOcts*	pNewOct( UAsnInt size );
EXTERN void pFreeOct( AsnOcts *o );

#define	pNewOid	pNewOct
#define	pFreeOid pFreeOct

#define pNewVbl() varBindListNew(sizeof(char *))

EXTERN void pFreeVbl( VarBindList * v );

EXTERN VarBind* NewVb(VarBindList *vbl, UAsnInt objectNameSize,
		    VarBindChoiceChoiceId varbindChoice,
		    ObjectSyntaxChoiceId syntaxChoice,
		    UAsnInt valueSize);
EXTERN VarBind* NewVbp(VarBindList *vbl, UAsnInt objectNameSize,
		     VarBindChoiceChoiceId varbindChoice,
		     ObjectSyntaxChoiceId syntaxChoice,
		     UAsnInt valueSize);
EXTERN SnmpPrivMsg* AllocPrivMsg(UAsnInt privDstSize,
				  UAsnInt privDataSize);
EXTERN SnmpAuthMsg* AllocAuthMsg(UAsnInt authDataSize,
				  SnmpAuthMsgChoiceChoiceId choice);
EXTERN SnmpMgmtCom* AllocMgmtCom(PDUsChoiceId tag,UAsnInt operationSize,
				  UAsnInt srcPartySize, UAsnInt dstPartySize,
				  UAsnInt contextSize);

/* 
 * -----------------------------------------------------------------------
 * Function:    CmpOID
 * Arguments:   o1, object identifier 1 (IN).
 *              od2, object identifier 2 (IN).
 * Return:      STATUS, TRUE on success, and FALSE otherwise.
 * Purpose:     This function compare both identifiers.
 * Remarks:     This routine _just_ compares, and _not_ orders, both oid's.
 * -----------------------------------------------------------------------
 */
EXTERN STATUS	CmpOID( AsnOid *o1, AsnOid *o2 );

/*
 * -----------------------------------------------------------------------
 *
 * Function:    CMPOID
 *
 * Arguments:   AsnOid  *o1     (in)
 *              AsnOid  *o2     (in)
 *
 * Return:      COMPARETYPE
 *
 * Purpose:     Compare two object identifiers lexicographically.
 *              LESS     <->     o1 < o2
 *              EQUAL    <->     o1 = o2
 *              GREATER  <->     o1 > o2
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
EXTERN	COMPARETYPE
CMPOID( AsnOid *o1, AsnOid *o2);

/* 
 * -----------------------------------------------------------------------
 * Function:    OIDtoASCII
 * Arguments:   oid, object identifier (IN).
 *              str, converted object identifier (OUT).
 * Return:      STATUS, TRUE on success, and FALSE otherwise.
 * Purpose:     This function converts object identifier from internal -
 *              to external format.
 * Remarks:
 * -----------------------------------------------------------------------
 */
EXTERN STATUS	ASCIItoOID( char *str, AsnOid *oid );

/* 
 * -----------------------------------------------------------------------
 * Function:    OIDtoASCII
 * Arguments:   oid, object identifier (IN).
 *              str, converted object identifier (OUT).
 * Return:      STATUS, TRUE on success, and FALSE otherwise.
 * Purpose:     This function converts object identifiers from internal -
 *              to external format.
 * Remarks:
 * -----------------------------------------------------------------------
 */
EXTERN STATUS	OIDtoASCII( AsnOid *eoid, char *str );

/* 
 * -----------------------------------------------------------------------
 * Function:    printArgumentList
 * Arguments:   set of variable arguments
 *		( format, value, value, value, etc ) 
 * Return:      STATUS, TRUE on success, and FALSE otherwise.
 * Purpose:     This function converts object identifiers from internal -
 *              to external format.
 * Remarks:
 * -----------------------------------------------------------------------
 */
EXTERN void printArgumentList( ) ;

EXTERN STATUS DynToStatVbl ( VarBindList *vbl, char *staticvbl ) ;

EXTERN STATUS StatToDynVbl ( char *staticvbl, VarBindList *vbl ) ;

EXTERN STATUS VarbindlistToV1Trap ( VarBindList **varbindlist,  
	AsnOid *enterprise, AsnInt *generic_trap, 
	AsnInt *specific_trap, UAsnInt *time_stamp) ;

EXTERN STATUS V1TrapToVarbindlist ( VarBindList **varbindlist,  
	AsnOid *enterprise, AsnInt *generic_trap, 
	AsnInt *specific_trap, UAsnInt *time_stamp ) ;

#endif	/* SNMPTYPES_H */
