/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename: 	spmSocket.h
 *
 * Date: 	May 1994
 *
 * Purpose:	headerfile for spmSocket.c
 *
 * Author(s):	Vincent Berkhout, Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */

#ifndef	SPMSOCKET_H
#define	SPMSOCKET_H

#include <snmpTypes.h>
#include <snmpConst.h> 
#include <sys/socket.h>
#include <sys/resource.h>
#include <fcntl.h>
#include <netinet/in.h>

#define	PACKETSIZE	6550	/* 64 K minus 36 bytes */

/* ---------------------------------------------------------------------
 * Function: */	EXTERN STATUS SocketCheckInputs  PROTO(()) ;
/* 
 * Arguments:	int	*allHandles	(in)
 * 		int 	*activeHandles	(out)
 *		int	STATUS 		(out)
 * Purpose:	reads an integer array of filedescriptors, converts them to a filedescriptor set,
 *		calls "select" to look for active connections and stores the active "handles" in 
 * 		an integer array activeHandles.
 *		returns STATUS which additionally contains the number of active connections
 */

/* ---------------------------------------------------------------------
 * Function: */	EXTERN STATUS SocketClose PROTO(()) ;
/* 
 * Arguments:	int	filedescriptor	(in)
 *		int	STATUS 		(out)
 * Purpose:	Close the connection identified by the handle and on succes 
 *		return STATUS OK
 */


/* ---------------------------------------------------------------------
 * Function: */ EXTERN STATUS SocketStreamPassiveOpen PROTO(());
/* 
 *  Arguments:	char	*localTaddress 	(in) 
 *		int	*connectHandle 	(out) 
 *		int	STATUS		(out)
 * Purpose:	This function creates a new socket and opens it. Then it binds the 
 * 		local transport address to the socket and goes into the listen state. 
 */

/* ---------------------------------------------------------------------
 * Function: */ EXTERN STATUS SocketStreamActiveOpen PROTO(());
/* 
 *  Arguments:	char	*remoteTAddress (in) 
 *		int	*dataHandle 	(out) 
 *		int	STATUS 		(out)
 * Purpose:	This function creates a new socket and opens it. Then it binds the 
 * 		local transport address to the socket. It then has to wait for the
 *		server to accept the connection. 
 */

/* ---------------------------------------------------------------------
 * Function: */	EXTERN STATUS SocketStreamAcceptConnection PROTO(()) ;
/* 
 *  Arguments:	int	*connectHandle 	(in)
 *		int	*dataHandle 	(out)
 *		char	*remoteTaddress	(out)
 *		int	STATUS		(out)
 * Purpose:	This function accepts a data request. When the server is in the 
 * 		listen state and a client performs an active open, this function is 
 * 		called to accept the new connection (at the server side!)
 *		returns STATUS OK if everything is correct
 */

/* ---------------------------------------------------------------------
 * Function: */	EXTERN STATUS SocketStreamSend PROTO(());
/* 
 *  Arguments:	int	*dataHandle 	(in) 
 * 		char	*message 	(in)
 *		int	STATUS 		(out)
 * Purpose:	Write the userdata to a specified filedescriptor (tcid)
 * 		and on succes return STATUS OK
 */

/* ---------------------------------------------------------------------
 * Function: */	EXTERN STATUS SocketStreamReceive PROTO(()) ;
/* 
 *  Arguments:	int	*dataHandle	(in)
 * 		char 	*message	(out)
 *		int	STATUS 		(out)
 * Purpose:	This is the function where the data is send, using the syscall "send"
 */


/* ---------------------------------------------------------------------
 * Function: */	EXTERN STATUS SocketPacketOpen  PROTO(());
/* 
 *  Arguments:	char	*localTAddress	(in)
 *		int 	*dataHandle	(out)
 *		int	STATUS		(out)
 * Purpose:	This call performs an open for udp at the local side . It derives the
 * 		portnumber from the localTAddress parameter. It subsequently creates
 *		a socket, binds it to the portnumber and goes into "listen" mode.
 */

/* ---------------------------------------------------------------------
 * Function: */	EXTERN STATUS SocketPacketSend  PROTO(()) ;
/* 
 *  Arguments:	int	*dataHandle	(in)
 * 		char 	*remoteTAddress	(in)
 * 		char 	*message	(in)
 * Purpose:	This function sends the data to the remote address. 
 * 		After conversion of the address, the messagesize is determined and the
 *		message is send.
 */

/* ---------------------------------------------------------------------
 * Function: */	EXTERN STATUS SocketPacketReceive  PROTO(()) ;
/* 
 *  Arguments:	int 	*dataHandle	(in)
 * 		char 	*remoteTAddress	(out)
 *		char	*message	(out)
 * Purpose:	Reads the data from the given handle and returns the message and peer address
 */


#endif 	/* ! SPMSOCKET_H */
