/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:    snmpMib2.h
 *
 * Date:        May 1994
 *
 * Purpose:     definitions for the MIB II for SNMP version 2 according 
 *		to RFC 1450. (snmpMibObjects) 
 *
 * Author(s):   Koert Wapper,Vincent Berkhout
 *
 * -----------------------------------------------------------------------
 */

#ifndef SNMPV2MIB_H
#define SNMPV2MIB_H

#include <snmpTypes.h>
#include <spmMIB_INT.h>


/*
 *	TYPE DEFINITIONS.
 */
typedef enum {
	SNMPSTATS			= 1,
	SNMPV1				= 2
} SNMPSTATES;

typedef enum {
	snmpStatsPackets		= 1,
	snmpStats30Something		= 2,
	snmpStatsEncodingErrors		= 3,
	snmpStatsUnknownDstParties	= 4,
	snmpStatsDstPartyMismatches	= 5,
	snmpStatsUnknownSrcParties	= 6,
	snmpStatsBadAuths		= 7,
	snmpStatsNotInLifetimes		= 8,
	snmpStatsWrongDigestValues	= 9,
	snmpStatsUnknownContexts	= 10,
	snmpStatsBadOperations		= 11,
	snmpStatsSilentDrops		= 12
} snmpStatsIndex;

typedef enum {
	snmpV1BadCommunityNames		= 1,
	snmpV1BadCommunityUses		= 2
} snmpV1Index;

typedef struct SNMPTRAPENTRY {
    UAsnInt partyIndex;
    Counter32 trapNumber;
    struct SNMPTRAPENTRY *next;
} SNMPTRAPENTRY;

/*
 *	CONSTANTS.
 */
#define	ALLSTATS		0 		/* for printing all the snmpStats */
#define TOTALSNMPSTATS		snmpStatsSilentDrops	 /* number of snmpStats errors */
#define TOTALSNMPV1		snmpV1BadCommunityUses	 /* number of snmpV1 errors */
#define BUFSIZE			256		/* size of input buffer */

/*
 *	TABLE DECLARATIONS
 * 	SNMPv2 statistics group: describes the behaviour of the SNMPv2 Entity 
 */
AsnInt	SnmpStats[TOTALSNMPSTATS + 1];
AsnInt	SnmpV1[ TOTALSNMPV1+1];


extern struct MibRecord	snmpv2mibtab[];
extern struct TabRecord	snmpv2tabtab[];
extern int	snmpv2msize, snmpv2tsize;

/*
 *-------------------------------------------------------------------
 * Function:	SpmMibInitSnmpV2
 * Arguments:	none	
 * Returns:	STATUS	which is OK on succes
 * Purpose:	initialize snmpMIB
 *-------------------------------------------------------------------
 */
EXTERN STATUS SpmInitSnmpV2Mib  PROTO(( )) ;

/*
 *-------------------------------------------------------------------
 * Function:	MibInitSnmpV2
 * Arguments:	none	
 * Returns:	STATUS	which is OK on succes
 * Purpose:	initialize all relevant statistics and mib objects
 *-------------------------------------------------------------------
 */
EXTERN STATUS MibInitSnmpV2 PROTO(());

/*
 *-------------------------------------------------------------------
 * Function:	ReadSnmpStats
 * Arguments:	filename	*char	(in)	
 * Returns:	STATUS	which is OK on succes
 * Purpose:	Reads snmpV2 statistics from file referenced by filename
 *		and stores them in the SNMPV2MIB
 *-------------------------------------------------------------------
 */
EXTERN STATUS ReadSnmpStats PROTO(()) ;
EXTERN STATUS SaveSnmpStats PROTO(());
EXTERN STATUS MibIncrementStat PROTO(( snmpStatsIndex index));
EXTERN STATUS MibPutStat PROTO(( snmpStatsIndex index, AsnInt value ));
EXTERN STATUS MibGetStat PROTO(( snmpStatsIndex index, AsnInt *value ));
EXTERN STATUS MibPrintStats PROTO(( int index ));

/*
 *-------------------------------------------------------------------
 * Function:	MibInitTrapGroup
 * Arguments:	none	
 * Returns:	STATUS	which is OK on succes
 * Purpose:	initialize the Trap Group 
 *-------------------------------------------------------------------
 */
EXTERN STATUS MibInitTrapGroup PROTO(());

/*
 *-------------------------------------------------------------------
 * Function:	MibGetTrapNumber
 * Arguments:	UAsnInt  pi, 		(in) partyIndex 
 *              AsnInt *trapNumber	(out) retrieved trap number	
 * Returns:     STATUS	which is OK on succes
 * Purpose:     Get the trap number of the given partyindex, if not
 *              available create one and initialize to 1 else return 
 *              the number of traps already generated 
 *-------------------------------------------------------------------
 */
EXTERN STATUS MibGetTrapNumber PROTO((UAsnInt pi, AsnInt *trapNumber));


#endif /* ! SNMPV2MIB_H */
