/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:    spmTIM.h
 *
 * Date:        May 1994
 *
 * Purpose:	headerfile for spmTransport.c
 *
 * Author(s):	Vincent Berkhout, Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */

#ifndef SPMTRANSPORT_H
#define SPMTRANSPORT_H

#include <snmpTypes.h>
#include <fcntl.h>
#include <sys/param.h>	/* imports NOFILE */
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <tiuser.h>


#define MAXFILES	64	/* should be system configured */

typedef	int		THANDLE ;	/* the abstract transport handle */
typedef	struct {
	int length;
	char buf[1024*64];
} TMESSAGE ; 
typedef	unsigned char	TADDRESS ;

typedef enum {
	UDPDOMAIN	= 0,	
	CLNSDOMAIN	= 1,
	CONSDOMAIN	= 2,
	DDPDOMAIN	= 3,
	IPXDOMAIN	= 4,
	TCPDOMAIN	= 5
} TDOMAIN ;

#define	DOMAINTOTAL	6	/* last domain plus one */    

typedef struct listenEntry {
	char	*transportAddress ;	/* pointer to transport address */
	int 	handle ;		/* the identity of the handle */
	int	numberOfOpens ; 
	struct	listenEntry	*next ;	/* pointer to next entry */
} listenTable ;

typedef struct activeConnectionEntry {
	char	*localTAddress,		/* pointer to transport address */
		*remoteTAddress;	/* pointer to transport address */
	int	handle;
	struct	timeval	*lastUpdate; 	/* ppointer to the time since last update */
	struct 	activeConnectionEntry	*next;	/* pointer to next entry */
} activeConnectionsTable ;
	
static int SNMP_DEBUG_TIM = 30;

/*
 *      The transport layer interface related errno ...
 */
EXTERN int	errno, t_errno;
EXTERN char	*sys_errlist[], *t_errlist[];	

/*
 *-------------------------------------------------------------------
 * Function:	SpmInitTim
 * Arguments:	none
 * Returns:	STATUS	which is OK on succes
 * Purpose:	Initialize the Transportlayer 
 * Remarks:	
 *-------------------------------------------------------------------
 */
EXTERN	STATUS	SpmInitTim PROTO(());

/*
 *-------------------------------------------------------------------
 * Function:	TransportOpen
 * Arguments:	transportDomain		(in)
 *		transportAddress	(in)
 * Returns:	STATUS	which is OK on succes
 * Purpose:	open a transport connection on a given handle
 * Remarks:	
 *-------------------------------------------------------------------
 */
EXTERN	STATUS	TransportOpen PROTO(( TDOMAIN transportDomain,
			TADDRESS *transportAddress ));


/*
 *-------------------------------------------------------------------
 * Function:	TransportSend
 * Arguments:	transportDomain,	(in)
 * 		localTAddress, 		(in)
 *		remoteTAddress, 	(in)
 * 		message			(in)
 * Returns:	STATUS	which is OK on succes
 * Purpose:	write message to datahandle on specified domain and address	
 * Remarks:	-
 *-------------------------------------------------------------------
 */
EXTERN	STATUS	TransportSend PROTO(( TDOMAIN transportDomain, 
			TADDRESS *localTAddress, TADDRESS *remoteTAddress, 
			AsnOcts *tmessage ));


/*
 *-------------------------------------------------------------------
 * Function:	TransportRecieve
 * Arguments:	transportDomain,	(out)
 * 		localTAddress,		(out)
 * 		remoteTAddress,		(out)
 * 		message			(out) 
 * Returns:	STATUS	which is OK on succes
 * Purpose:	receive a message from the different domains 
 * Remarks:	
 *-------------------------------------------------------------------
 */
EXTERN	STATUS	TransportReceive PROTO(( 
			TDOMAIN *transportDomain, TADDRESS *localTAddress,
			TADDRESS *remoteTAddress, TMESSAGE *message));

/*
 *-------------------------------------------------------------------
 * Function:	TransportClose
 * Arguments:	transportDomain,	(in)
 * 		localTAddress		(in) 
 * Returns:	STATUS	which is OK on succes
 * Purpose:	close the handle (and associated handles!) on the given domain
 * Remarks:	
 *-------------------------------------------------------------------
 */
EXTERN	STATUS	TransportClose PROTO(( 
			TDOMAIN transportDomain, TADDRESS *localTAddress));

/* 
 * ---------------------------------------------------------------------
 * Function:	TransCheckInputs
 * Arguments:	TDOMAIN 	transportDomain	(in)
 *		THANDLE		*allHandles	(in)
 * 		THANDLE	 	*activeHandles	(out)
 * returns: 	STATUS 
 * Purpose:	Directly mapped on SocketCheckInputs () 	
 */
EXTERN STATUS TransCheckInputs PROTO(( TDOMAIN transportDomain, THANDLE *allHandles,
				THANDLE *activeHandles ));

/* 
 * ---------------------------------------------------------------------
 * Function:	TransClose
 * Arguments:	TDOMAIN 	transportDomain	(in)
 *		THANDLE		handle		(in)
 * Returns: 	STATUS
 * Purpose:	Directly mapped on SocketClose () 	
 */
EXTERN STATUS TransClose PROTO(( TDOMAIN transportDomain, THANDLE handle ));   


/* 
 * ---------------------------------------------------------------------
 * Function:	TransPassiveOpen ()
 * Arguments:	TDOMAIN 	transportDomain	(in)
 *		TADDRESS	*localTAddress	(in)
 * 		THANDLE	 	*connectHandle	(out)
 * Returns: 	STATUS
 * Purpose:	Directly mapped on SocketStreamPassiveOpen () 	
 */
EXTERN STATUS TransPassiveOpen PROTO(( TDOMAIN transportDomain, 
			TADDRESS *localTAddress, THANDLE *connectHandle ));

/* 
 * ---------------------------------------------------------------------
 * Function:	TransActiveOpen ()
 * Arguments:	TDOMAIN 	transportDomain	(in)
 *		TADDRESS	*remoteTAddress	(in)
 * 		THANDLE	 	*dataHandle	(out)
 * Returns: 	STATUS
 * Purpose:	Directly mapped on SocketStreamActiveOpen () 	
 */
EXTERN STATUS TransActiveOpen PROTO(( TDOMAIN transportDomain, 
			TADDRESS *remoteTAddress, THANDLE *dataHandle ));


/* 
 * ---------------------------------------------------------------------
 * Function:	TransAcceptConnection ()
 * Arguments:	TDOMAIN 	transportDomain	(in)
 * 		THANDLE	 	*connectHandle	(in)
 * 		THANDLE		*dataHandle	(out)
 *		TADDRESS	*remoteTAddress	(out)
 * Returns: 	STATUS
 * Purpose:	Directly mapped on SocketStreamAcceptConnection () 	
 */
EXTERN STATUS TransAcceptConnection  PROTO(( 
			TDOMAIN transportDomain, THANDLE *connectHandle, 
			TADDRESS *remoteTAddress, THANDLE *dataHandle ));


/*
 * ---------------------------------------------------------------------
 * Function:	TransStreamSend ()
 * Arguments:	TDOMAIN 	transportDomain	(in)
 * 		THANDLE 	*dataHandle	(in)
 *		TMESSAGE	*message	(in)
 * Returns: 	STATUS
 * Purpose:	Directly mapped on SocketStreamSend () 	
 */
EXTERN STATUS TransStreamSend PROTO(( TDOMAIN transportDomain, THANDLE *dataHandle,
				TMESSAGE *message ));  


/*
 * ---------------------------------------------------------------------
 * Function:	TransStreamRecieve ()
 * Arguments:	TDOMAIN 	transportDomain	(in)
 * 		THANDLE 	*dataHandle	(in)
 *		TMESSAGE	*message	(out)
 * Returns: 	STATUS
 * Purpose:	Directly mapped on SocketStreamReceive () 	
 */
EXTERN STATUS TransStreamRecieve  PROTO(( TDOMAIN transportDomain, 
			THANDLE *dataHandle, TMESSAGE *message ));


/* 
 * ---------------------------------------------------------------------
 * Function	TransPacketOpen ()
 * Arguments:	TDOMAIN 	transportDomain	(in)
 *		TADDRESS	*localTAddress	(in)
 * 		THANDLE 	*dataHandle	(out)
 *		int	STATUS 		(out)
 * Purpose:	Directly mapped on SocketPacketOpen () 	
 */
EXTERN STATUS TransPacketOpen PROTO(( TDOMAIN transportDomain, 
			TADDRESS *localTAddress, THANDLE *dataHandle )); 

/*
 * ---------------------------------------------------------------------
 * Function:	TransPacketSend ()
 * Arguments:	TDOMAIN 	transportDomain	(in)
 * 		THANDLE 	*dataHandle	(in)
 *		TADDRESS	*remoteTAddress	(in) 
 *		TMESSAGE	*message	(in)
 * Returns: 	STATUS
 * Purpose:	Directly mapped on SocketPacketSend () 	
 */
EXTERN STATUS TransPacketSend PROTO(( 
			TDOMAIN transportDomain, THANDLE *dataHandle,
			TADDRESS *remoteTAddress, TMESSAGE *message ));      


/*
 * ---------------------------------------------------------------------
 * Function:	TransPacketReceive ()
 * Arguments:	TDOMAIN 	transportDomain	(in)
 * 		THANDLE 	*dataHandle	(in)
 *		TADDRESS	*remoteTAddress	(in) 
 *		TMESSAGE	*message	(out)
 * Returns: 	STATUS
 * Purpose:	Directly mapped on SocketPacketReceive () 	
 */
EXTERN STATUS TransPacketReceive PROTO(( 
			TDOMAIN transportDomain, THANDLE *dataHandle,
			TADDRESS *remoteTAddress, TMESSAGE *message ));     


#endif /* ! SPMTRANSPORT_H */

/*--------------------------- end of file spmTrans.h ------------------------------*/
