/* -----------------------------------------------------------------------
 *
 * Project:		UT-SNMP
 *
 * Filename:    v2Agent.c
 *
 * Date:        June 1994
 *
 * Purpose:     Agent program to test/demonstrate the useability of the 
 *				SNMPv2 API-functions. Agnt acts like a native-proxy Agnt.
 *
 * Author(s):   J.W.Togtema, Eric van Hengstum, Vincent Berkhout.
 *				Antoinne A(pi) `olle'man. 
 *
 * -----------------------------------------------------------------------
 */

#include <apiSNMP.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/uio.h>
#include <unistd.h>

#define AgntMain main
/*
 *      Definition of linked list for mapping corresponding requests.
 */
typedef struct outreq {
	int     responseid;
	int     requestid;
	struct outreq *next;
} OUTREQ;
OUTREQ  *outstreqlist = NULL;

/*
 *      Request sequence number.
 */
INTEGER REQ_SEQNR = 5000;

/*
 *	for support of SNMP version 1 
 */ 
#define	V1AGENT	( !strcmp("v1Agent", argv[0]) )
#define	V2AGENT	( strcmp("v1Agent", argv[0]) )

/*
 *      Must this application produce debug output? Default is no.
 */
int AgntDEBUG = 0;


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntErrorTypes
 *
 * Arguments:	
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntErrorTypes( INTEGER errorstatus, char **argv )
{
	if V1AGENT {
		switch( (int) errorstatus ) {
		case 0 : (void) fprintf( stdout, "noError"); break;
		case TOOBIG : (void) fprintf( stdout, "tooBig"); break;
		case NOSUCHNAME : (void) fprintf( stdout, "noSuchName"); break;
		case BADVALUE : (void) fprintf( stdout, "badValue"); break;
		case READONLY : (void) fprintf( stdout, "readOnly"); break;	
		case GENERR : (void) fprintf( stdout, "genErr"); break;
		default : (void) fprintf( stdout, "noError %d", errorstatus ); break;
		}
	} else {
		switch( (int) errorstatus ) {
		case 0 : (void) fprintf( stdout, "noError"); break;
		case TOOBIG : (void) fprintf( stdout, "tooBig"); break;
		case NOSUCHNAME : (void) fprintf( stdout, "noSuchName"); break;
		case BADVALUE : (void) fprintf( stdout, "badValue"); break;
		case READONLY : (void) fprintf( stdout, "readOnly"); break;
		case GENERR : (void) fprintf( stdout, "genErr"); break;
		case NOACCESS : (void) fprintf( stdout, "noAccess"); break;
		case WRONGTYPE : (void) fprintf( stdout, "wrongType"); break;
		case WRONGLENGTH : (void) fprintf( stdout, "wrongLength"); break;
		case WRONGENCODING : (void) fprintf( stdout, "wrongEncoding"); break;
		case WRONGVALUE : (void) fprintf( stdout, "wrongValue"); break;
		case NOCREATION : (void) fprintf( stdout, "noCreation"); break;
		case INCONSISTENTVALUE : (void) fprintf( stdout, "inconsistentValue"); break;
		case RESOURCEUNAVAILABLE : (void) fprintf( stdout, "resourceUnavailable"); break;
		case COMMITFAILED : (void) fprintf( stdout, "commitFailed"); break;
		case UNDOFAILED : (void) fprintf( stdout, "undoFailed"); break;
		case AUTHORIZATIONERROR : (void) fprintf( stdout, "authorizationError"); break;
		case NOTWRITABLE : (void) fprintf( stdout, "notWritable"); break;
		case INCONSISTENTNAME : (void) fprintf( stdout, "inconsistentname"); break;
		default : (void) fprintf( stdout, "noError %d", errorstatus ); break;
		}
	}
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntIdIncrement
 *
 * Arguments:	id, request id (IN/OUT).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Produce a new request identifier and assign it to the 
 *				parameter 'id'.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntIdIncrement( INTEGER *id )
{
	*id = REQ_SEQNR;
	REQ_SEQNR++;
	if ( REQ_SEQNR == 10000 )
		REQ_SEQNR = 0;
	*id = REQ_SEQNR ;
	return( 0 );
}	


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntDeleteEntryList
 *
 * Arguments:	entry, pointer to entry to be deleted/removed (IN).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     unlink entry 'entry' from the global linked list 
 *				'outstreqlist' and free the alloced memory of 'entry'.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntDeleteEntryList( OUTREQ *entry )
{
	OUTREQ *curr;

	curr = outstreqlist; 
	if( curr == (OUTREQ *) NULL )
		return( -1 );
	if( entry == curr )
	{
		outstreqlist = curr->next;
		(void) free( curr );
		return( 0 );
	}
	while( curr->next != (OUTREQ *) NULL )
	{
	       	if( curr->next == entry)
		{
		       	curr->next = entry->next;
		       	(void) free( entry );
			return( 0 );
		} else
			curr = curr->next;
	}
	return( -1 );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntCreateEntryList
 *
 * Arguments:	list, newly allocated entry pointed by list (IN/OUT).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Allocate memory for new entry 'list' and prepend entry
 *				on top of global linked-list 'outstreqlist'.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntCreateEntryList( OUTREQ **list )
{
	*list = (OUTREQ *) malloc( sizeof( OUTREQ ));
	if( *list == (OUTREQ *) NULL )
		return( -1 );
	(*list)->next = outstreqlist; 
	outstreqlist = *list;
	return( 0 );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntgetEntryOutList
 *
 * Arguments:	requestid, identifier of incoming communication (IN).
 * 				reponseid, identifier of outgoing communication (OUT).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Search for 'requestid' and it's corresponsing 'responseid'
 *				in global linked list 'outstreqlist'. Remove entry, iff
 *				found.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntgetEntryOutList( INTEGER requestid, INTEGER *responseid )
{
	OUTREQ	*curr = NULL;

	curr = outstreqlist;
	while( curr != NULL )
	{
		if( curr->requestid == requestid )
		{
			*responseid = curr->responseid;			
			(void) AgntDeleteEntryList( curr );
			return( 0 );
		} else 
			curr = curr->next;
	}
	return( -1 );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntPutEntryInList
 *
 * Arguments:	responsid, identifier for outgoing communication (IN).
 *				requestid, identifier for incoming communication (IN).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Create an entry, fill entry with both values 'responseid' and 
 *				'requestid', and link the entry in the global linked-list 
 *				'outstreqlist'.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntPutEntryInList( INTEGER responseid, INTEGER requestid )
{
	OUTREQ *list;

	if( AgntCreateEntryList( &list ) )
		return( -1 );
	list->requestid = requestid; 
	list->responseid = responseid; 
	return( 0 );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntWriteIndication
 *
 * Arguments:	
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Display indication info .
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
w( char *w, char *host, char *app, int qos, int from, int to, 
		int stat, int ind, int max, int non , VARLIST *vbl, char **argv)
{
	fprintf( stdout, "\tsnmp%sIndication(\n", w );
	fprintf( stdout, "\t\t\tINTEGER replyId = %d,\n", from );
	if V2AGENT {
		if( strcmp( w, "GetBulk" ) == 0 ) {
			fprintf( stdout, "\t\t\tINTEGER maxRepetitions = %d,\n", max );
			fprintf( stdout, "\t\t\tINTEGER maxRepeaters = %d,\n", non );
		}
	}
	fprintf( stdout, "\t\t\tVARLIST variableNames =\n" );
	PrintVarBindList( stdout, vbl, 32 );
	fprintf( stdout, "\t);\n\n" );
	fprintf( stdout, "\tsnmp%sRequest(\tNAME host = %s,\n", w, host );
	fprintf( stdout, "\t\t\tNAME application  = %s,\n", app );
	fprintf( stdout, "\t\t\tINTEGER requestId = %d,\n", to );
	if V2AGENT {
		if( strcmp( w, "GetBulk" ) == 0 ) {
			fprintf( stdout, "\t\t\tINTEGER maxRepetitions = %d,\n", max );
			fprintf( stdout, "\t\t\tINTEGER maxRepeaters = %d,\n", non );
		}
		fprintf( stdout, "\t\t\tQOS qos = %d,\n", qos );
	}
	fprintf( stdout, "\t\t\tVARLIST variableNames =\n" );
	fprintf( stdout, "\t);\n\n" );
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntWriteIndication
 *
 * Arguments:	
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Display indication info .
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
p( char *w, char *host, char *app, int qos, int from, int to, 
	int stat, int ind, int max, int non , VARLIST *vbl, char **argv )
{
	fprintf( stdout, "\tsnmp%sConfirm(\tINTEGER replyId = %d,\n", w, to );
	fprintf( stdout, "\t\t\tINTEGER errorStatus = " );
	AgntErrorTypes( stat, argv ); fprintf( stdout, ",\n" );
	fprintf( stdout, "\t\t\tINTEGER errorIndex = %d,\n", ind );
	fprintf( stdout, "\t\t\tVARLIST variableValues =\n" );
	fprintf( stdout, "\t\t\t" );
	(void) PrintVarBindList( stdout, vbl, 24 ) ;
	fprintf( stdout, "\n" );
	fprintf( stdout, "\t);\n\n" );
	fprintf( stdout, "\tsnmp%sResponse(\n", w );
	fprintf( stdout, "\t\t\tINTEGER replyId = %d,\n", from );
	fprintf( stdout, "\t\t\t\tINTEGER errorStatus = " );
	AgntErrorTypes( stat, argv );
	fprintf( stdout, ",\n" );
	fprintf( stdout, "\t\t\t\tINTEGER errorIndex = %d,\n", ind );
	fprintf( stdout, "\t\t\t\tVARLIST variableValues =\n" );
	fprintf( stdout, "\t);\n\n" );
	return OK;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntHandleNet
 *
 * Arguments:	type, management communication type(IN).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Routine handle all communication related to provider.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntHandleNet( SPTYPE type, char **argv )
{
	INTEGER		toId = 0, fromId = 0, ErrStat =0, ErrInd = 0, MaxRep, NonRep;
	NAME		fromAgnt="", toAgnt="utis94", toCnt="";
	QOS		toQoS=qosNONE;
	VarBindList	*Binds=mVlNew();
	DESCRIPTION	error;


	system( "clear" ); StarLine; NewLine;

	switch( type ) {

	case GET_IND:
		snmpGetIndication( &fromId, &Binds );
		AgntIdIncrement( &toId );
		AgntPutEntryInList( fromId, toId );
		snmpGetRequest( toAgnt, toCnt, toId, toQoS, Binds );
		w("Get",toAgnt,toCnt,toQoS,fromId,toId,ErrStat,ErrInd,MaxRep,NonRep,Binds, argv);
		break;

	case GET_CNF:
		snmpGetConfirm( &toId, &ErrStat, &ErrInd, &Binds );	
		AgntgetEntryOutList( toId, &fromId );
		snmpGetResponse( fromId, ErrStat, ErrInd, Binds );
		p("Get",toAgnt,toCnt,toQoS,fromId,toId,ErrStat,ErrInd,MaxRep,NonRep,Binds, argv);
		break;

	case GETNEXT_IND:
		snmpGetNextIndication( &fromId, &Binds );
		AgntIdIncrement( &toId );
		AgntPutEntryInList( fromId, toId );
		snmpGetNextRequest( toAgnt, toCnt, toId, toQoS, Binds );
		w("GetNext",toAgnt,toCnt,toQoS,fromId,toId,ErrStat,ErrInd,MaxRep,NonRep,Binds, argv);
		break;

	case GETNEXT_CNF:
		(void) snmpGetNextConfirm( &toId, &ErrStat, &ErrInd, &Binds );
(void) fprintf(stdout, "before: toId = %d, fromId = %d", toId, fromId ) ;
		(void) AgntgetEntryOutList( toId, &fromId );
(void) fprintf(stdout, "aftter: toId = %d, fromId = %d", toId, fromId ) ;
		(void) snmpGetNextResponse( fromId, ErrStat, ErrInd, Binds );
		p("GetNext",toAgnt,toCnt,toQoS,fromId,toId,ErrStat,ErrInd,MaxRep,NonRep,Binds, argv);
		break;

	case GETBULK_IND:
		(void) snmpGetBulkIndication( &fromId, &MaxRep, &NonRep, &Binds );
		(void) AgntIdIncrement( &toId );
		(void) AgntPutEntryInList( fromId, toId );
		snmpGetBulkRequest( toAgnt, toCnt, toId , toQoS, MaxRep, NonRep, Binds );
		w("GetBulk",toAgnt,toCnt,toQoS,fromId,toId,ErrStat,ErrInd,MaxRep,NonRep,Binds, argv);
		break;

	case GETBULK_CNF:
		(void) snmpGetBulkConfirm( &toId, &ErrStat, &ErrInd, &Binds );
		(void) AgntgetEntryOutList( toId, &fromId );
		(void) snmpGetBulkResponse( fromId, ErrStat, ErrInd, Binds );
		fprintf( stdout, "\t\t\tINTEGER errorIndex = %d );\n\n", ErrInd );
		p("GetBulk",toAgnt,toCnt,toQoS,fromId,toId,ErrStat,ErrInd,MaxRep,NonRep,Binds, argv);
		break;

	case SET_IND:
		(void) snmpSetIndication( &fromId, &Binds );
		(void) AgntIdIncrement( &toId );
		(void) AgntPutEntryInList( fromId, toId );
		(void) snmpSetRequest( toAgnt, toCnt, toId, toQoS, Binds );
		w("Set",toAgnt,toCnt,toQoS,fromId,toId,ErrStat,ErrInd,MaxRep,NonRep,Binds, argv);
		break;
	
	case SET_CNF:
		snmpSetConfirm( &toId, &ErrStat, &ErrInd );
		AgntgetEntryOutList( toId, &fromId );
		snmpSetResponse( fromId, ErrStat, ErrInd );
		p("Set",toAgnt,toCnt,toQoS,fromId,toId,ErrStat,ErrInd,MaxRep,NonRep,Binds, argv);
		break;

	case ERR_IND:
		(void) snmpErrorDescription( &toId, error );
		(void) AgntgetEntryOutList( toId, &fromId );
		fprintf( stderr, "\tsnmpErrorDescription(\tINTEGER requestId = %d,\n", toId );
		fprintf( stderr, "\t\t\t\terror = %s\n", error );
		fprintf( stderr, "\t);\n" );
		break;

	case TRAP_IND:
		(void) snmpTrapIndication( fromAgnt, toCnt, &Binds );

		fprintf( stdout, "\tsnmpTrapIndication(\n" );
		fprintf( stdout, "\t\t\tNAME host = %s,\n", fromAgnt );
		fprintf( stdout, "\t\t\tNAME application = %s,\n", toCnt );
		fprintf( stdout, "\t\t\tVARLIST variableValues =\n" );
		fprintf( stdout, "\t\t\t" );
			(void) PrintVarBindList( stdout, Binds, 24) ;
		fprintf( stdout, "\n" );
		fprintf( stdout, "\t);\n\n" );
		break;

	case INFORM_IND:
		(void) snmpInformIndication( fromAgnt, toCnt, &Binds );
	
		fprintf( stdout, "\tsnmpInformIndication(\n" );
		fprintf( stdout, "\t\t\tNAME host = %s,\n", fromAgnt );
		fprintf( stdout, "\t\t\tNAME application = %s,\n", toCnt );
		fprintf( stdout, "\t\t\tVARLIST variableValues =\n" );
		fprintf( stdout, "\t\t\t" );
		(void) PrintVarBindList( stdout, Binds, 24) ;
		fprintf( stdout, "\n" );
		fprintf( stdout, "\t);\n\n" );
		break;

	default: 
		return OK ;
	}
	NewLine; StarLine; fprintf( stdout, "\tHit <enter> key to exit: " ); Flush; 
	return FALSE;
}


/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    AgntUserMonitor
 *
 * Arguments:
 *
 * Return:      STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Prints the results.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntUserMonitor( )
{
	fd_set fds;
	struct timeval tv;

	tv.tv_sec = 0;
	tv.tv_usec = 10;

	FD_ZERO( &fds );
	FD_SET( 0, &fds );
	 
	if( select( getdtablesize(), &fds, 0, 0, &tv ) > 0 )
		return( OK );
	return( FAILED );
}



/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    AgntHandleUser
 *
 * Arguments:
 *
 * Return:      STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function fetches the user input.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntHandleUser( )
{
	return TRUE;
}


/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    AgntUserIntro
 *
 * Arguments:   argv, argument vector (IN)
 *
 * Return:      STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function prints the intro screen.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntUserIntro( char** argv )
{
    system( "clear" ); StarLine;

	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout, " * %s (version %s)\t\t\t\t\t\t*\n", argv[0], VERSION );
	fprintf( stdout, " * -------\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout, " * This proxy agent demonstrates/tests the UT-SNMP-V3_0" );
	fprintf( stdout, " package.\t*\n");
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout, " *\t\t\t\t\tSuccess, \t\t\t*\n" );
	fprintf( stdout, " *\t\t\t\t\t\tsnmp@cs.utwente.nl\t*\n" );
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	if V2AGENT {
		fprintf( stdout, " * Additional remarks:\t\t\t\t\t\t\t*\n");
		fprintf( stdout, " *\t1) %% setenv SNMP_PARTYMIB_PATH ./ ");
		fprintf( stdout, "\t# path for partyfiles\t*\n");
	}
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	StarLine;  NewLine; 
	fprintf( stdout,"\tHit <enter> key to exit: " ); 
	Flush;
	return OK ;
}


/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    AgntNetMonitor
 *
 * Arguments:   type, the arrived management communication type (OUT).
 *
 * Return:      STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function monitors the snmp layer .
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AgntNetMonitor( SPTYPE *type )
{
	return( snmpLook( type ) );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	AgntMain
 *
 * Arguments:	argc, argument octetLen (IN).
 *				argv, argument vector (IN).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     main driver for dispatching the tasks depending on the 
 *				occurred events.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	void
main( int argc, char **argv, char **arge ) 
{
	SPTYPE type = 0;
	BOOLEAN	dying = FALSE;


	if V2AGENT {
		if( snmpOpen( argv[0], arge ) ) {
			Error( "couldn't open snmp" );
			exit(-1);
		}
	} else {
		if( snmpOpen( "v1Application", arge ) ) {
			Error( "couldn't open snmp" );
			exit(-1);
		}
	}

	sleep(1) ;	/* see if everything went alright */ 
	(void) AgntUserIntro( argv );
		
	while( dying != TRUE ) {
		if( AgntUserMonitor() == OK )
			dying = AgntHandleUser();
		else if( AgntNetMonitor( &type ) == OK )
		  	dying = AgntHandleNet( type, argv );
												  }

	if( snmpClose( GRACEFULL ) ) {
		Error( "couldn't close snmp" );
		exit( -1 );
	}
	exit( 0 );
}

/* ----------------------- end of file snmpAgent.c -------------------------- */
