/* -----------------------------------------------------------------------
 *
 * Project:		UT-SNMP
 *
 * Filename:    v2Manager.c
 *
 * Date:        May 1994
 *
 * Purpose:     SNMP V2 Manager program for demonstration/testing purposes.
 *
 * Author(s):   J.W.Togtema, Frederic P. H. van Hengstum, Vincent Berkhout
 *
 * -----------------------------------------------------------------------
 */

#include <apiSNMP.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/uio.h>
#include <unistd.h>

#define MngrMain main
#define BUFSIZE 512

typedef char ASCII[BUFSIZE];
typedef int* INT;

EXTERN STATUS MngrUserMonitor( );
EXTERN STATUS MngrUserRead( );

#define mVb(p,n,c,v) \
	mVbPut(p,n,mVcPut(mVcNew(),VARBINDCHOICE_VALUE,mObPut(mObNew(),c,v)))

#define	V1MANAGER	(!strcmp("v1Manager", argv[0]))
#define	V2MANAGER	( (!strcmp("v2Manager", argv[0])) || \
			  (!strcmp("manager", argv[0])) )

static int requestIdentifier = 100 ;
static struct timeval	startTime;

static AsnOid sysUpTime	= { 7, "\53\6\1\2\1\1\3" };
static AsnOid snmpEventID = { 11, "\53\6\1\6\3\2\1\2\2\1\2" };
static AsnOid snmpTrapOID = { 9, "\53\6\1\6\3\1\1\4\1" };
static AsnOid linkUp = { 9, "\53\6\1\6\3\1\1\5\4" };

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrReadValue
 *
 * Arguments:	
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrReadValue( VarBind *vb, ObjectName *n )
{
	ASCII type;
	int value = 0, s;
	AsnOid *oid;

	fprintf( stdout, "\t\t\t\t(I)NTEGER, (O)CTETSTR, O(B)JECTID, \n" );
	fprintf( stdout, "\t\t\t\tBIT(S)TR, IP(A)DDR, (C)OUNTER32, \n" );
	fprintf( stdout, "\t\t\t\t(G)AUGE32, TIMETIC(K)S, OPA(Q)UE, \n" );
	fprintf( stdout, "\t\t\t\t(N)SAPADDR, CO(U)NTER64, UIN(T)EGER32: " );
	Flush;
	while( MngrUserMonitor( ) );
	(void) MngrUserRead( type, &value );

	switch( (int) *type ) {
	case 'I' :
		fprintf( stdout, "\t\t\t\tINTEGER " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value ); 
		mVb( vb, n, OBJECTSYNTAX_INTEGER_VALUE, value );
		break;

	case 'O' :
		(void) fprintf( stdout, "\t\t\t\tOCTETSTR " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value ); s=strlen(type);
		mVb( vb, n, OBJECTSYNTAX_STRING_VALUE, mOcPut( mOcNew(s), type, s) );
		break;

	case 'B':
		(void) fprintf( stdout, "\t\t\t\tOBJECTID " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value ); s=strlen(type);
		ASCIItoOID( type, oid=mOiNew( 256 ) );
		mVb( vb, n, OBJECTSYNTAX_OBJECTID_VALUE, oid );
		break;	

	case 'S':
		(void) fprintf( stdout, "\t\t\t\tBITSTR " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value ); s = strlen(type);
		mVb( vb, n, OBJECTSYNTAX_BIT_VALUE, mBiPut( mBiNew( s ), type, s ) );
		break;

	case 'A':
		(void) fprintf( stdout, "\t\t\t\tIPADDRESS " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value ); s = strlen(type);
		mVb( vb, n, OBJECTSYNTAX_IPADDRESS_VALUE, mIpPut( mIpNew( s ), type, s ) );
		break;

	case 'C':
		(void) fprintf( stdout, "\t\t\t\tCOUNTER32 " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value );
		mVb( vb, n, OBJECTSYNTAX_COUNTER_VALUE, value );
		break;

	case 'G':
		(void) fprintf( stdout, "\t\t\t\tGAUGE32 " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value );
		mVb( vb, n, OBJECTSYNTAX_GAUGE_VALUE, value );
		break;

	case 'K':
		(void) fprintf( stdout, "\t\t\t\tTIMETICKS " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value );
		mVb( vb, n, OBJECTSYNTAX_TIMETICKS_VALUE, value );
		break;

	case 'Q':
		/* choiceId = OBJECTSYNTAX_ARBITRARY_VALUE ; */
		(void) fprintf( stdout, "\t\t\t\tNOT IMPLEMENTED, YET! " );
		break;

	case 'N':
		/* choiceId = OBJECTSYNTAX_NSAPADDRESS_VALUE ; */
		(void) fprintf( stdout, "\t\t\t\tNOT IMPLEMENTED, YET! " );
		break;

	case 'U':
		(void) fprintf( stdout, "\t\t\t\tCOUNTER64 " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value );
		mVb( vb, n, OBJECTSYNTAX_BIG_COUNTER_VALUE, value );
		break;

	case 'T':
		(void) fprintf( stdout, "\t\t\t\tUINTEGER32 " ); Flush;
		while( MngrUserMonitor( ) );
		MngrUserRead( type, &value );
		mVb( vb, n, OBJECTSYNTAX_UNSIGNED_INTEGER_VALUE, value );
		break;

	default:
		return FAILED;
	}
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrErrorTypes
 *
 * Arguments:	
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrErrorTypes( PDUInt mErrStat, char **argv  )
{
	if V1MANAGER {
		switch( (int) mErrStat ) {
		case 0 : (void) fprintf( stdout, "noError"); break;
		case TOOBIG : (void) fprintf( stdout, "tooBig"); break;
		case NOSUCHNAME : (void) fprintf( stdout, "noSuchName"); break;
		case BADVALUE : (void) fprintf( stdout, "badValue"); break;
		case READONLY : (void) fprintf( stdout, "readOnly"); break;	
		case GENERR : (void) fprintf( stdout, "genErr"); break;
		default : (void) fprintf( stdout, "noError %d", mErrStat ); break;
		}
	} else {
		switch( (int) mErrStat ) {
		case 0 : (void) fprintf( stdout, "noError"); break;
		case TOOBIG : (void) fprintf( stdout, "tooBig"); break;
		case NOSUCHNAME : (void) fprintf( stdout, "noSuchName"); break;
		case BADVALUE : (void) fprintf( stdout, "badValue"); break;
		case READONLY : (void) fprintf( stdout, "readOnly"); break;
		case GENERR : (void) fprintf( stdout, "genErr"); break;
		case NOACCESS : (void) fprintf( stdout, "noAccess"); break;
		case WRONGTYPE : (void) fprintf( stdout, "wrongType"); break;
		case WRONGLENGTH : (void) fprintf( stdout, "wrongLength"); break;
		case WRONGENCODING : (void) fprintf( stdout, "wrongEncoding"); break;
		case WRONGVALUE : (void) fprintf( stdout, "wrongValue"); break;
		case NOCREATION : (void) fprintf( stdout, "noCreation"); break;
		case INCONSISTENTVALUE : (void) fprintf( stdout, "inconsistentValue"); break;
		case RESOURCEUNAVAILABLE : (void) fprintf( stdout, "resourceUnavailable"); break;
		case COMMITFAILED : (void) fprintf( stdout, "commitFailed"); break;
		case UNDOFAILED : (void) fprintf( stdout, "undoFailed"); break;
		case AUTHORIZATIONERROR : (void) fprintf( stdout, "authorizationError"); break;
		case NOTWRITABLE : (void) fprintf( stdout, "notWritable"); break;
		case INCONSISTENTNAME : (void) fprintf( stdout, "inconsistentname"); break;
		default : (void) fprintf( stdout, "noError %d", mErrStat ); break;
		}
	}
	return OK ;
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserWriteConfirm
 *
 * Arguments:	which, type of confirm
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserWriteConfirm( char *which, INTEGER mReqId, INTEGER mErrInd, 
			INTEGER mErrTyp, VarBindList *mVarBinds, char **argv )
{
 	fprintf( stdout, "\tsnmp%sConfirm(\n", which );
	fprintf( stdout, "\t\t\tINTEGER requestId = %d,\n", mReqId );
	fprintf( stdout, "\t\t\tINTEGER errorStatus = " );
	MngrErrorTypes( mErrTyp, argv ); (void) fprintf( stdout, ",\n" );
	fprintf( stdout, "\t\t\tINTEGER errorIndex = %d,\n", 0, mErrInd );
	fprintf( stdout, "\t\t\tVARLIST variableValues =\n" );
	fprintf( stdout, "\t\t\t" );
	PrintVarBindList( stdout, mVarBinds, 24 ) ;
	fprintf( stdout, "\n" );
	fprintf( stdout, "\t );\n" ); Flush ;
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrHandleNet
 *
 * Arguments:	type, service type (IN).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrHandleNet( SPTYPE mType, char **argv )
{
	INTEGER 	mReqId, mErrInd, mErrTyp;
	DESCRIPTION	error = "";
	VarBindList	*mVarBinds = mVlNew( );

	mReqId = mErrInd = mErrTyp = 0;

	(void) fprintf( stdout, "\tUT-SNMP C API snmpConfirmation primitive(s):\n" );
	(void) fprintf( stdout, "\t--------------------------------------------\n\n" );
	Flush ;
	switch( mType )
	{

	case GET_CNF:
	if( snmpGetConfirm( &mReqId, &mErrTyp, &mErrInd, &mVarBinds ) == OK ) 
		MngrUserWriteConfirm( "Get", mReqId, mErrInd, mErrTyp, mVarBinds, argv );
	break;

	case GETNEXT_CNF:
	if( snmpGetNextConfirm( &mReqId, &mErrTyp, &mErrInd, &mVarBinds ) == OK )
		MngrUserWriteConfirm( "GetNext", mReqId, mErrInd, mErrTyp, mVarBinds, argv );
	break;

	case GETBULK_CNF:
	if( snmpGetBulkConfirm( &mReqId, &mErrTyp, &mErrInd, &mVarBinds ) == OK )
		MngrUserWriteConfirm( "GetBulk", mReqId, mErrInd, mErrTyp, mVarBinds, argv );
	break;

	case SET_CNF:
	if( snmpSetConfirm( &mReqId, &mErrTyp, &mErrInd ) == OK )
		MngrUserWriteConfirm( "Set", mReqId, mErrInd, mErrTyp, NULL, argv );
	break;

	case INFORM_CNF:
	if( snmpInformConfirm( &mReqId, &mErrTyp, &mErrInd, &mVarBinds ) == OK )
		MngrUserWriteConfirm( "Inform", mReqId, mErrInd, mErrTyp, mVarBinds, argv );
	break;

	case ERR_IND:
 	if( snmpErrorDescription( &mReqId, error ) == OK ) {
		fprintf( stdout, "\tsnmpErrorDescription(\tINTEGER requestId = %d,\n", mReqId );
		fprintf( stdout, "\t\t\t\tDESCRIPTION error = %s );\n", error );
		Flush;
	}
	break;

	default: 
	break;
	}
	NewLine; StarLine ; NewLine ; HitKey ; Flush ;
	return FALSE;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrNetMonitor
 *
 * Arguments:	type, the arrived management communication type (OUT).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function monitors the snmp layer .
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrNetMonitor( SPTYPE *type )
{
	return( snmpLook( type ) );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserWriteIntro
 *
 * Arguments:	
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function prints the intro screen.
 *
 * Remarks:	argv, argument vector (IN) is now obsolete!
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserWriteIntro(char **argv)
{
	system( "clear" );
	StarLine ;
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout, " * %s (version %s)\t\t\t\t\t*\n", argv[0], VERSION );
	fprintf( stdout, " * -----------\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout, 
	" * This interactive snmpmanager demonstrates the Ut-SNMP-V3 package.\t*\n");
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	fprintf( stdout, " *\t\t\t\t\tSuccess, \t\t\t*\n" );
	fprintf( stdout, " *\t\t\t\t\t\tsnmp@cs.utwente.nl\t*\n" );
	if V2MANAGER {
		fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
		fprintf( stdout, " * Additional remarks:\t\t\t\t\t\t\t*\n");
		fprintf( stdout, " *\t1) For CMU interaction is applicationname superfluous !\t\t*\n" );
	} 
	fprintf( stdout, " *\t\t\t\t\t\t\t\t\t*\n" );
	StarLine ; NewLine; Flush; HitKey ; Flush ;
	return( OK );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserWritePrimitives
 *
 * Arguments:	which, which primitive (IN)
 *				bulk, getbulk? (IN)
 *				int, request type (IN)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function writes an available request primitive. 
 *
 * Remarks:	Bulk boolean is superfluous now !
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserWritePrimitive( char *which, int nr,char **argv )
{
	(void) fprintf( stdout, "\tsnmp%sRequest(\n", which );
	(void) fprintf( stdout, "\t\t\t\tNAME host,\n " );
	(void) fprintf( stdout, "\t\t\t\tNAME application,\n" );
 	(void) fprintf( stdout, "\t\t\t\tINTEGER requestId,\n" );
	if V2MANAGER {
		(void) fprintf( stdout, "\t\t\t\tQOS qos, \n" );
		if( strcmp( which, "GetBulk" ) == 0 ) {
			(void) fprintf( stdout, "\t\t\t\tINTEGER maxRepetitions,\n" );
			(void) fprintf( stdout, "\t\t\t\tINTEGER nonRepeaters,\n" );
		}
	}
	(void) fprintf( stdout, "\t\t\t\tVARLIST variableNames );\t[%d]\n\n", nr );
	
	return OK;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserWritePrimitive
 *
 * Arguments:	none.
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function prints the available request primitives. 
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */

	STATUS
MngrUserWritePrimitives( char **argv)
{

    system( "clear" ); StarLine ; NewLine ;
	(void) fprintf( stdout, "\n\tUT-SNMP C API request service primitives:\n" );
	(void) fprintf( stdout, "\t-----------------------------------------\n\n" );

	if V1MANAGER {
		MngrUserWritePrimitive( "Get", 1, argv );
		MngrUserWritePrimitive( "GetNext", 2, argv );
		MngrUserWritePrimitive( "Set", 3, argv );
		(void) fprintf( stdout, "\tsnmpTrapRequest( VARLIST variableValues );\t\t[4]\n" );
		NewLine ; StarLine ; 
		(void) fprintf( stdout, "\n\t\t\t\t\t\t[ 0-4, exit=0 ]: " );
	} else {
		MngrUserWritePrimitive( "Get", 1, argv );
		MngrUserWritePrimitive( "GetNext", 2, argv );
		MngrUserWritePrimitive( "GetBulk", 3, argv );
		MngrUserWritePrimitive( "Set", 4, argv );
		MngrUserWritePrimitive( "Inform", 5, argv );
		(void) fprintf( stdout, "\tsnmpTrapRequest(\tVARLIST variableValues );\t[6]\n" );
		NewLine ; StarLine ; 
		(void) fprintf( stdout, "\n\t\t\t\t\t\t[ 0-6, exit=0 ]: " );
	}
	return( OK );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserReadHostName
 *
 * Arguments:	hostname, pointer to character string (IN/OUT).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function reads a hostname. 
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */

	STATUS
MngrUserReadHostname( char *hostname )
{
	int none ;

	(void) fprintf( stdout, "\t\t\tNAME host = " );
	Flush ;	
	while( MngrUserMonitor( ) );
	(void) MngrUserRead( hostname, &none );
	if( strlen( hostname ) == 0 ) {
		NewLine ; StarLine; NewLine ; HitKey; Flush;
		return FAILED;
	} 
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserReadApplicationname
 *
 * Arguments:	application, pointer to character string (IN/OUT).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function reads a application. 
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserReadApplicationname( char *application, char **argv )
{
	int 	nope ;

	(void) fprintf( stdout, "\t\t\tNAME application = " );
	Flush ;
	while( MngrUserMonitor( ) );
		(void) MngrUserRead( application, &nope );
	return( OK );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserReadNumber
 *
 * Arguments:	text, displaytext (IN)
 *				value, integer value (OUT)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function reads a hostname. 
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */

	STATUS
MngrUserReadNumber( char *text, int *value )
{
	char 	none[100];

	(void) fprintf( stdout, "\t\t\tINTEGER %s = ", text ); Flush ;
	while( MngrUserMonitor( ) );
	(void) MngrUserRead( none, value );
	if( (*value<0) || (*value>1234567890) || (strlen(none) == 0) ) {
		NewLine ; Flush ; StarLine ; NewLine ; HitKey ; Flush ;
		return FAILED ;
	}
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserReadQOS
 *
 * Arguments:	qos, 0, 1 or 2 (IN/OUT)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function reads a hostname. 
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */

	STATUS
MngrUserReadQOS( int *qos )
{
	char	none[100] ;

	*qos = -1 ; 

	(void) fprintf( stdout, "\t\t\tQOS qos = " );
	Flush ;
	while( MngrUserMonitor( ) );
	(void) MngrUserRead( none, qos );
	if( (*qos < 0) || (*qos > 2) || (strlen( none ) == 0) ) {
		NewLine ; StarLine ; NewLine ; HitKey ; Flush ;
		return FAILED;
	}
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserWriteHeading
 *
 * Arguments:	which, which service request type (IN)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function writes an servicesheading.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */

	STATUS
MngrUserWriteHeading( char *which )
{
	system( "clear" ) ;
	StarLine ; NewLine ;
	fprintf( stdout, "\tUT-SNMP C API snmp%sRequest primitive:\n", which );
	fprintf( stdout, "\t---------------------------------------\n\n" );
	fprintf( stdout, "\tsnmp%sRequest(\n", which );
	return OK;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserReadVarList
 *
 * Arguments:	list, ivariable binding list (IN/OUT)
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function writes an servicesheading.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */

	STATUS
MngrUserReadVarList( VARLIST *list, BOOLEAN justNames )
{
	NAME variableNames;
	AsnOid *newOid;
	VarBind *vb;
	int nope;

	do {
		(void) fprintf( stdout, "\t\t\tVARLIST variableNames = " ); Flush;
		while( MngrUserMonitor( ) );
		(void) MngrUserRead( variableNames, &nope );
		if( strlen( variableNames ) == 0 )
			break;
		ASCIItoOID( variableNames, newOid=mOiNew(255) ) ;
		vb=mVbPut(mVbNew(255),newOid,mVcPut(mVcNew(),VARBINDCHOICE_UNSPECIFIED,0));
		if( justNames == TRUE )
			if( MngrReadValue( vb, newOid ) != OK )
				break;
		mVlPut( list, vb );
	} while( (int) strlen( variableNames ) > 0 );
	(void) fprintf( stdout, "\t);\n\n" ); Flush ;
	return OK;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrRequest
 *
 * Arguments:	which, reqquest type (IN).
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     The more generic requests like get, get-next, get-bulk
 * 		and set. This function prints the available request primitives,
 *		reads the given input and generates a request.
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrRequest( char *which, BOOLEAN values, char **argv )
{
	ASCII 		targetHostname, applicationName;
	int 		maxRepetitions, nonRepeaters, qos, eventIndex;
	VARLIST		*variableNames;
	VARLIST 	**list ;
	VarBind 	*vb;
	AsnOid		*newOid=NULL;
	UAsnInt		upTime;
	struct	timeval	now;

/*
 *	trap is a special case, solve that one first! 
 *	btw. should be in the agent !
 */ 
	if( strcmp( which, "Trap" ) == 0 )
		return ( MngrTrapRequest() ) ; 
/*
 *	Inform is also a special case
 *	for reasons of clarity in a special function!
 */ 
	if( strcmp( which, "Inform" ) == 0 )
		return ( MngrInformRequest() ) ;

	MngrUserWriteHeading( which );
	variableNames = mVlNew() ;	

	requestIdentifier++;

	if( MngrUserReadHostname( targetHostname ) != OK ) 
		return FALSE ;
	if( MngrUserReadApplicationname( applicationName, argv ) != OK ) 
		return FALSE ;
	(void) fprintf( stdout, "\t\t\tINTEGER requestIdentifier = %d\n", requestIdentifier); 
	Flush ;

	if V2MANAGER {
		if( MngrUserReadQOS( &qos ) != OK ) return FALSE ;
		if ( strcmp( which, "GetBulk" ) == 0 ) {
		    if ( MngrUserReadNumber( "maxRepetitions", &maxRepetitions ) != OK)
			return FALSE ;
		    if (MngrUserReadNumber( "nonRepeaters", &nonRepeaters ) != OK )
			return FALSE ; 
		}
	} else qos =qosNONE;

	if( MngrUserReadVarList( variableNames, values ) != OK ) return FALSE ;

	if( strcmp( which, "Get" ) == 0 )
		snmpGetRequest( targetHostname, applicationName, 
				requestIdentifier, qos, variableNames );
	else if( strcmp( which, "GetNext" ) == 0 )
		snmpGetNextRequest( targetHostname, applicationName, 
				requestIdentifier, qos, variableNames );
	else if( strcmp( which, "GetBulk" ) == 0 )
	        snmpGetBulkRequest( targetHostname, applicationName, 
			requestIdentifier, qos, maxRepetitions, 
			nonRepeaters, variableNames );
	else if( strcmp( which, "Set" ) == 0 )
		snmpSetRequest( targetHostname, applicationName, 
				requestIdentifier, qos, variableNames );
	return OK ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrTrapRequest
 *
 * Arguments:	nonw (note: no argv is passed!) 
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function reads the input for a trap and generates 
 * 		the appropriate TrapRequest. 
 * 		This function should be in a agent, NOT in a manager! 
 * 
 * Remarks:	First the sysUptime is prepended to the varbindlist, 
 * 		Than a varbind with snmpTrapOID and an example trap "linkUp"
 * 		is appended, followed by a varbind which relates to the 
 * 		objectID who generated the trap.
 * 		Finally some possible additional varbinds.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrTrapRequest()
{
	VARLIST		*variableNames;
	VarBind 	*vb;
	AsnOid		*newOid=NULL;
	UAsnInt		upTime;
	struct	timeval	now;

	MngrUserWriteHeading( "Trap" );
	variableNames = mVlNew() ;
	
	gettimeofday(&now, (struct timezone *)0);
	upTime = (UAsnInt) ((now.tv_sec - startTime.tv_sec) * 100
			 + (now.tv_usec - startTime.tv_usec) / 10000);

	newOid=mOiNew(255);
	(void) memset(newOid->octs, 0x00, (size_t) 255);
	(void) memcpy(newOid->octs, sysUpTime.octs, sysUpTime.octetLen) ; 
	newOid->octetLen = sysUpTime.octetLen + 1 ; /* appends a zero */

	vb=mVbPut(mVbNew(255),newOid,mVcPut(mVcNew(),VARBINDCHOICE_VALUE,
		mObPut(mObNew(), OBJECTSYNTAX_TIMETICKS_VALUE, 0 )));
	vb->varBindChoice->a.value->a.timeticks_value = upTime ;
	mVlPut( variableNames, vb );

	newOid=mOiNew(255);
	(void) memset(newOid->octs, 0x00, (size_t) 255);
	(void) memcpy(newOid->octs, snmpTrapOID.octs, snmpTrapOID.octetLen) ; 
	newOid->octetLen = snmpTrapOID.octetLen+1;	/* appends a zero */

	vb=mVbPut(mVbNew(255),newOid,mVcPut(mVcNew(),VARBINDCHOICE_VALUE,
		mObPut(mObNew(), OBJECTSYNTAX_OBJECTID_VALUE, linkUp.octetLen)));
	vb->varBindChoice->a.value->a.objectID_value = &linkUp ;
	mVlPut( variableNames, vb );

	if( MngrUserReadVarList( variableNames, FALSE ) != OK )
		return FALSE ;

	snmpTrapRequest( variableNames );
	return OK; 
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrInformRequest
 *
 * Arguments:	argv  (IN) 
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function is very similar to the generic case, but
 * 		significant different to put it in a sperate function. 
 * 		It reads the hostname, application name  and QOS and
 * 		possible varbinds. 
 * 
 * Remarks:	First a sysUptime is prepended. The next varbind is  
 * 		snmpEventID.i and as value an example event "linkUp". 
 * 		Than a varbind is appended  containing the OID of the 
 * 		object who generated the event. 
 * 		finally some possible additional varbinds.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrInformRequest( char **argv )
{
	ASCII 		targetHostname, applicationName;
	int 		qos, eventIndex;
	VARLIST		*variableNames;
	VarBind 	*vb;
	AsnOid		*newOid=NULL;
	UAsnInt		upTime;
	struct	timeval	now;

	variableNames = mVlNew() ;
	requestIdentifier++;

	MngrUserWriteHeading( "Inform" );

	if( MngrUserReadHostname( targetHostname ) != OK ) return FALSE ;
	if( MngrUserReadApplicationname( applicationName, argv ) != OK ) return FALSE ;
	(void) fprintf( stdout, "\t\t\tINTEGER requestIdentifier = %d\n", requestIdentifier); 
	Flush ;
	if( MngrUserReadQOS( &qos ) != OK ) return FALSE ;

	gettimeofday(&now, (struct timezone *)0);
	upTime = (UAsnInt) ((now.tv_sec - startTime.tv_sec) * 100
			 + (now.tv_usec - startTime.tv_usec) / 10000);

	newOid=mOiNew(255);
	(void) memset(newOid->octs, 0x00, (size_t) 255);
	(void) memcpy(newOid->octs, sysUpTime.octs, sysUpTime.octetLen) ; 
	newOid->octetLen = sysUpTime.octetLen + 1 ; /* appends a zero */

	vb=mVbPut(mVbNew(255),newOid,mVcPut(mVcNew(),VARBINDCHOICE_VALUE,
	mObPut(mObNew(), OBJECTSYNTAX_TIMETICKS_VALUE, 0 )));
	vb->varBindChoice->a.value->a.timeticks_value = upTime ;
	mVlPut( variableNames, vb );

	if( MngrUserReadNumber( "eventOid", &eventIndex ) != OK ) 
		return FALSE ; 

	newOid=mOiNew(255);
	(void) memset(newOid->octs, 0x00, (size_t) 255);
	(void) memcpy(newOid->octs, snmpEventID.octs, snmpEventID.octetLen) ; 
	newOid->octetLen = snmpEventID.octetLen+1;	/* appends a zero */
	newOid->octs[newOid->octetLen] = eventIndex ;	/*!! maybe buggy for > 127 */

	vb=mVbPut(mVbNew(255),newOid,mVcPut(mVcNew(),VARBINDCHOICE_VALUE,
		mObPut(mObNew(), OBJECTSYNTAX_OBJECTID_VALUE, linkUp.octetLen)));
	vb->varBindChoice->a.value->a.objectID_value = &linkUp ;
	mVlPut( variableNames, vb );

	if( MngrUserReadVarList( variableNames, FALSE ) != OK ) return FALSE ;

	snmpInformRequest( targetHostname, applicationName, 
			requestIdentifier, qos, variableNames );
	return OK; 
}
 

/* ^L
 * -----------------------------------------------------------------------
 *
 * Function:    MngrUserRead
 *
 * Arguments:   
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function fetches the user input.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrHandleUser( char **argv )
{
	ASCII buf;
	int type;

	getchar( );

	(void) MngrUserWritePrimitives( argv );
	while( MngrUserMonitor( ) );
	(void) MngrUserRead( buf, &type );

	if V1MANAGER {
		switch( type ) {
		case 1: return( MngrRequest( "Get", FALSE, argv ) );
		case 2: return( MngrRequest( "GetNext", FALSE, argv ) );
		case 3: return( MngrRequest( "Set", TRUE, argv ) );
		default: return TRUE;
		}
	} else {
		switch( type ) {
		case 1: return( MngrRequest( "Get", FALSE, argv ) );
		case 2: return( MngrRequest( "GetNext", FALSE, argv ) );
		case 3: return( MngrRequest( "GetBulk", FALSE, argv ) );
		case 4: return( MngrRequest( "Set", TRUE, argv ) );
		case 5: return( MngrRequest( "Inform", FALSE, argv ) );
		case 6: return( MngrRequest( "Trap", FALSE, argv ) );
		default: return TRUE;
		} 	/* type */
	}
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserRead
 *
 * Arguments:	
 *
 * Return:	STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     This function fetches the user input.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserRead( char* abuf, int* ibuf )
{
	short i = 0;

    	do {
	    	abuf[i++] = getchar();
	} while( abuf[i-1] != '\n' );
	abuf[i-1] = '\0';
	*ibuf = atoi( abuf );
	return( OK );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	MngrUserMonitor
 *
 * Arguments:	
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose:     Prints the results.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MngrUserMonitor( )
{
	fd_set fds;
	struct timeval tv;

	tv.tv_sec = 0;
	tv.tv_usec = 10;

	FD_ZERO( &fds );
	FD_SET( 0, &fds );

	if( select( getdtablesize(), &fds, 0, 0, &tv ) > 0 )
		return( OK );
	return( FAILED );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	main
 *
 * Arguments:	argc, argument octetLen (IN)
 *				argv, argument vector (IN)
 *				arge, argument environment (IN)
 *
 * Return:		STATUS, OK on success and FAILED otherwise.
 *
 * Purpose: 	This function dispatches user - and net events.    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	int
MngrMain( int argc, char** argv, char ** arge )
{
	SPTYPE type = 0;
	BOOLEAN	dying = FALSE;

    (void) gettimeofday(&startTime, (struct timezone *)0);

	if V1MANAGER {		
		if( snmpOpen( "v1Application", arge ) != OK  ) {
			Error( "couldn't open snmp" );
			exit( -1 );
		}
	} else {
		if( snmpOpen( argv[0], arge ) != OK  ) {
			Error( "couldn't open snmp" );
			exit( -1 );
		}
	} 

	(void) MngrUserWriteIntro( argv );
	
	while( ! dying ) {
		if( MngrUserMonitor( ) == OK ) 
			dying = MngrHandleUser( argv );
		else 	if( MngrNetMonitor( &type ) == OK )
			dying = MngrHandleNet( type, argv );
	}

	if( snmpClose( GRACEFULL ) ) {
		Error( "couldn't close snmp" );
		exit( -1 );
	}
	exit( 0 );
}

/* --------------------- end of file snmpManager.c ----------------------- */

