/* -----------------------------------------------------------------------
 *
 * Project:		UT-SNMP
 *
 * Filename:    SHM.c
 *
 * Date:        June 26, 1994
 *
 * Version:		1.0
 *
 * Purpose:		This layer handles the communication between Snmp
 *				Protocol Machine and the possibly various SNMP appls.
 *
 * Author(s):   Rob Post, Eric van Hengstum, Vincent Berkhout
 *
 * -----------------------------------------------------------------------
 */

#include <simTF.SHM.h>

/*
 * 	local variables
 */
static CONNECTTABLE     connectTable;
static char spmAddress[10]="spm";
static char spmaddress[10]="spm";
IPATH listenfile, connectfile;
IADDRESS myAddress;
int mutexid; /* semaphore on listenfile spm and connectfile api */

	STATUS
shmCheckInputs( int *allHandles, int *activeHandles )
{
	return 1;
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	shmPassiveOpen	  
 *
 * Arguments:	localAddress, reference to local application (IN).
 *
 * Purpose:		This function opens an listen endpoint for the receipt
 *				of pending connection requests.
 *				This function opens one side of an connection to an
 *				application.
 *
 * Remarks:		local connectTable is initialized, just once and
 *				the local listenfile is opened.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
shmPassiveOpen( char *localAddress, int *handle )
{
    ICOMMAND command = commandPrefix; /* "/user/bin/touch /tmp/.ipc." */
    BOOLEAN init = FALSE;
    short i;
    key_t mutexkey;
    FILE *fp;
	
    sprintf( listenfile, "%s%s", pathPrefix, localAddress );
	strcpy(myAddress, localAddress );
    if( (fp = fopen( listenfile, "r" ))  != (FILE *) NULL ) {
		fclose( fp );
		return( ERROR );
    }
    (void) strcat( command, localAddress );
    if( system( command ) < 0 ) { 
		return( ERROR );
    }
    if( init == FALSE ) {
	for( i = 0; i < CONNECTMAX; i++ ) {
	    connectTable[i].handle = connectTable[i].shmid1 = DEFAULTVALUE;
	    connectTable[i].shmid2 = connectTable[i].semid = DEFAULTVALUE;
	    connectTable[i].shmem1 = connectTable[i].shmem2 = (void *) NULL;
	    connectTable[i].address[0] = TERMINATORVALUE;
	}
	if( ( mutexkey = ftok( listenfile, (int) 0 ) ) == ((key_t) -1) ) {
	    return( ERROR );
	}
	if( ( mutexid = semget( mutexkey, 2, IPC_CREAT|0600 ) ) == -1 ) {
	    return( ERROR );
	}
	arg.val = INITMUTEX;
	if( semctl( mutexid, SPMMUTEX, SETVAL, arg ) == -1 ) {
	    return( ERROR );
	} 
	arg.val = INITMUTEX;
	if( semctl( mutexid, APIMUTEX, SETVAL, arg ) == -1 ) {
	    return( ERROR );
	} 
	strcpy( spmaddress, localAddress );
	init = TRUE;
    }
    connectTable[0].handle = 0;
    strcpy( connectTable[0].address, localAddress );
	*handle = 0;
    return( OK );
}


/*  
 * -----------------------------------------------------------------------
 *
 * Function:	shmActiveOpen	  
 *
 * Arguments:	localAddress, reference to local application (IN).
 *		remoteAddres, referenct to remote application (IN).
 *		handle, reference to connection (OUT).
 *
 * Purpose:	This function initiates an connection request to the
 *		(opposite) remote application.
 *
 * Remarks:	- check if local connectfile exists
 *			if exists -> return ERROR
 *		- initiates connectTable ONCE with defaults
 *		- check for the listenfile from the remote application
 *		- if not existent: return error
 *		- open local connectfile
 *		- write applicationname to listenfile
 *		- create key for reserving memory and semaphores
 *		- reserve memory and semaphores
 *		- initialize semaphores
 *		- update local connectTable
 *		- check connectfile (for connect indication) if spm 
 *		  made a connection
 *			timeout -> return ERROR
 *
 * -----------------------------------------------------------------------
 */
	STATUS
shmActiveOpen( char *localAddress, char *remoteAddress, int *handle )
{

    IPATH path = pathPrefix, dummy = "";
    ICOMMAND command;
    key_t shmkey;
    int shmid1, shmid2, semid;
    void *shmem1, *shmem2;	
    FILE *fp;
    union semun arg;
    BOOLEAN init = FALSE;
    short i;
    key_t mutexkey;
    BOOLEAN found = FALSE;
    int timeout; /* is timeout on reading connectfile */

    timeout = 20000; 


    /* check existence listenfile */
    (void) sprintf( listenfile, "%s%s", pathPrefix, remoteAddress );
    if( ( fp = fopen( listenfile, "r" ) ) == (FILE *) NULL ) {
		return( ERROR );
    }

    /* try to read connectfile, if file exists return error */
    (void) sprintf( connectfile, "%s%s", pathPrefix, localAddress );
    if( (fp = fopen( connectfile, "r" ))  != (FILE *) NULL ) {
		fclose( fp );
		return( ERROR );
    }

    /* initialize connectTable */
    if( init == FALSE ) {
	for( i = 0; i < CONNECTMAX; i++ ) {
	    connectTable[i].handle = connectTable[i].shmid1 = DEFAULTVALUE;
	    connectTable[i].shmid2 = connectTable[i].semid = DEFAULTVALUE;
	    connectTable[i].shmem1 = connectTable[i].shmem2 = (void *) NULL;
	    connectTable[i].address[0] = TERMINATORVALUE;
	}
	if( ( mutexkey = ftok( listenfile, (int) 0 ) ) == ((key_t) -1) ) {
	    return( ERROR );
	}
	if( ( mutexid = semget( mutexkey, 2, IPC_CREAT|0600 ) ) == -1 ) {
	    return( ERROR );
	}
	if( (arg.val = semctl( mutexid, SPMMUTEX, GETVAL)) < 0) {
	    return( ERROR );
	}
	if( (arg.val = semctl( mutexid, APIMUTEX, GETVAL)) < 0) {
	    return( ERROR );
	}
	init=TRUE;
	}

    /* lock listenfile */
    PM(mutexid, SPMMUTEX);

    /* check for listenfile */
    if( ( fp = fopen( listenfile, "r" ) ) == (FILE *) NULL ) {
		VM(mutexid, SPMMUTEX);
		return( ERROR );
    }
    (void) fscanf( fp, "%s", dummy );
    if( strcmp( dummy, "" ) != 0 ) {
		fclose(fp);
		VM(mutexid, SPMMUTEX);
		return( ERROR );
    }
    fclose(fp);

    /* open local connectfile */
    PM(mutexid, APIMUTEX);
    (void) sprintf( command, "touch %s", connectfile ); 
    if( system( command ) < 0 ) { 
		VM(mutexid, SPMMUTEX);
		VM(mutexid, APIMUTEX);
		return( ERROR );
    }
    VM(mutexid, APIMUTEX);

    /* write apiname to listenfile */
    (void) sprintf( command, "echo %s > %s", localAddress, listenfile );
    if( system( command ) < 0 ) {
		VM(mutexid, SPMMUTEX);
		return( ERROR );
    }
    
    /* unlock listenfile */
    VM(mutexid, SPMMUTEX);
    
/*
    if( ( fp = fopen( connectfile, "r+" ) ) == (FILE *) NULL ) { 
	    PM(mutexid, SPMMUTEX);
	    (void) sprintf( command, "echo %s > %s", localAddress, listenfile );
	    if( system( command ) < 0 ) {
		VM(mutexid, SPMMUTEX);
		return( ERROR );
	}
	VM(mutexid, SPMMUTEX);
	return( ERROR );
    } 
    while (fscanf( fp, "%s", dummy ) < 0 ) {
		timeout--;
		if (timeout < 0) {
	    PM(mutexid, SPMMUTEX);
	    (void) sprintf( command, "echo > %s", listenfile );
	    if( system( command ) < 0 ) {
			VM(mutexid, SPMMUTEX);
			return( ERROR );
		}
	    VM(mutexid, SPMMUTEX);
	    fclose(fp);
	    return( ERROR );
	}
    }
    if( strcmp( dummy, "" ) == 0 ) {
	    PM(mutexid, SPMMUTEX);
	    (void) sprintf( command, "echo > %s", listenfile );
	    if( system( command ) < 0 ) {
			VM(mutexid, SPMMUTEX);
			return( ERROR );
	    }
		VM(mutexid, SPMMUTEX);
		fclose(fp);
		return( ERROR );
    }
    fclose(fp);

    PM( mutexid, APIMUTEX );
    (void) sprintf( command, "echo > %s", connectfile ); 
    if( system( command ) < 0 ) {
		VM( mutexid, APIMUTEX );
		PM(mutexid, SPMMUTEX);
		(void) sprintf( command, "echo > %s", listenfile );
		if( system( command ) < 0 ) {
	    	VM(mutexid, SPMMUTEX);
	    	return( ERROR );
	}
		VM(mutexid, SPMMUTEX);
		return( ERROR );
    }
    VM( mutexid, APIMUTEX );
*/


    /* create key */

	shmkey = (key_t) ftok( connectfile, (int) 0 );
	if (shmkey == (key_t) -1 )
		return ERROR ;

	shmid1 = (int) shmget( shmkey, SHMMAX, IPC_CREAT|0600 );
	if (shmid1 == -1 ) 
		return ERROR ;

	shmem1 = (void *) shmat( shmid1, (void *) NULL, 0600 );
	if (shmem1 == (void *) -1 )
		return ERROR ;

	semid = (int) semget( shmkey, 2, IPC_CREAT|0600 );
	if ( semid == -1  )
		return ERROR ;
	
	arg.val = semWRITE;
	if ( semctl(semid, channel1, SETVAL, arg) < 0 )
		return ERROR ;

	arg.val = semWRITE;
	if ( semctl(semid, channel2, SETVAL, arg)  < 0 ) 
		return ERROR;

	shmkey = (key_t) ftok( connectfile, (int) 1 );
	if (shmkey == (key_t) -1 )
		return ERROR ;

	shmid2 = (int) shmget( shmkey, SHMMAX, IPC_CREAT|0600 );
	if ( shmid2 == (int) -1 )
		return ERROR ;

	shmem2 = (void *) shmat( shmid2, (void *) 0, 0600 ); 
	if ( shmem2 == (void *) -1 )
		return ERROR ;

	for( i = 0; ((i < CONNECTMAX) && (found == FALSE)); i++ ) {
		if( connectTable[i].handle == DEFAULTVALUE ) {
			connectTable[i].handle = *handle = i;
			connectTable[i].shmid1 = shmid1;
			connectTable[i].shmid2 = shmid2;
			connectTable[i].semid = semid;
			connectTable[i].shmem1 = shmem1;
			connectTable[i].shmem2 = shmem2;
			strcpy( connectTable[i].address, localAddress );
			found = TRUE;
		}
	}	/* for */

	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	shmAcceptOpen	  
 *
 * Arguments:	localAddress, reference to local application (IN).
 *		remoteAddres, referenct to remote application (IN).
 *		handle, reference to connection (OUT).
 *
 * Purpose:	This function accepts an pending connection request.
 *		This function establishes a connection to an other 
 *		application.
 *
 * Remarks:	- look in listenfile for an (connect request)
 *		  application name string
 *		- use string to make a unique key
 *		- reserve memory and semaphores
 *		- inititalize semaphores
 *		- update local connectTable
 *
 * -----------------------------------------------------------------------
 */
	STATUS
shmAcceptOpen( char *localAddress, char *remoteAddress, int *handle )
{
    IPATH path = pathPrefix, dummy, apiname;
    ICOMMAND command;
    key_t shmkey;
    int shmid1, shmid2, semid, i;
    void *shmem1, *shmem2;	
    FILE *fp;
    BOOLEAN found = FALSE;

    PM(mutexid, SPMMUTEX);
    if( ( fp = fopen( listenfile, "r+" ) ) == (FILE *) NULL ) { 
	VM(mutexid, SPMMUTEX);
	return( ERROR );
    }

    if( fscanf( fp, "%s", apiname ) < 0 ) {
		fclose(fp);
		VM(mutexid, SPMMUTEX);
		return( ERROR );
    }
    if( strcmp( apiname, "" ) == 0 ) {
		fclose(fp);
		VM(mutexid, SPMMUTEX);
		return( ERROR );
    }
    (void) sprintf( command, "echo > %s", listenfile ); 
    if( system( command ) < 0 ) {
		fclose(fp);
		VM(mutexid, SPMMUTEX);
		return( ERROR );
    }
    fclose(fp);
    VM(mutexid, SPMMUTEX);

	(void) strcpy( remoteAddress, (const char *) apiname );
	(void) sprintf( connectfile, "%s%s", pathPrefix, remoteAddress );
	shmkey = (key_t) ftok( connectfile, (int) 0 ); 
	if ( shmkey == ((key_t) -1) ) 
		return ERROR ;

	shmid1 = (int) shmget( shmkey, SHMMAX, IPC_CREAT|0600 ) ;
	if ( shmkey == -1 )
		return ERROR ;

	shmem1 = (void *) shmat( shmid1, (void *) 0, 0600 ) ;
	if ( shmem1 == (void *) 0 )
		return ERROR ;

	semid = (int) semget( shmkey, 2, IPC_CREAT|0600 ) ;
	if ( semid == -1 ) 
		return ERROR ;

	shmkey = (key_t) ftok( connectfile, (int) 1 ) ;
	if ( shmkey == (key_t) -1 ) 
		return ERROR ;

	shmid2 = (int) shmget( shmkey, SHMMAX, IPC_CREAT|0600 ) ;
	if ( shmid2 == -1 ) 
		return ERROR ;

	shmem2 = (void *) shmat( shmid2, (void *) 0, 0600 ) ;
	if ( shmem2 == (void *) 0 ) 
		return ERROR ;

	for( i = 0; ((i < CONNECTMAX) && (found == FALSE)); i++ ) {
		if( connectTable[i].handle == DEFAULTVALUE ) {
			connectTable[i].handle = *handle = i;
			connectTable[i].shmid1 = shmid1;
			connectTable[i].shmid2 = shmid2;
			connectTable[i].semid = semid;
			connectTable[i].shmem1 = shmem1;
			connectTable[i].shmem2 = shmem2;
			strcpy( connectTable[i].address, myAddress );
			found = TRUE;
		}
	}

/*
    (void) sprintf( connectfile, "%s%s", pathPrefix, remoteAddress );
    PM(mutexid, APIMUTEX);
    if( ( fp = fopen( connectfile, "r+" ) ) == (FILE *) NULL ) {
	VM(mutexid, APIMUTEX);
	return( ERROR );
    }
    (void) fscanf( fp, "%s", dummy );
    if( strcmp( dummy, "" ) != 0 ) {
	fclose(fp);
	VM(mutexid, APIMUTEX);
	return( ERROR );
    }
    fclose(fp);
    (void) sprintf( command, "echo %s > %s", localAddress, connectfile ); 
    if( system( command ) < 0 ) { 
	VM(mutexid, APIMUTEX);
	return( ERROR );
    }
    VM(mutexid, APIMUTEX);
*/

    return( OK );
}	



/*  
 * ------------------------------------------------------------------------
 *
 * Function:	shmSend	  
 *
 * Arguments:	handle, reference to connection (IN).
 *				message, reference to management communication (IN).
 *
 * Purpose:		This function sends messages over connection to remote 
 *				application.
 *				This function receives the message from an application.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
shmSend( int handle, char *message )
{
    int i, size, channel;
    char *shmem;

    for( i = 0; i < CONNECTMAX; i++ ) {
		if( connectTable[i].handle == handle ) {
	    	if( strstr( listenfile, connectTable[i].address ) == NULL ) {
				channel = channel2;
				shmem = connectTable[i].shmem2;
	    	} else {
				channel = channel1;
				shmem = connectTable[i].shmem1;
	    	}
	    	if( P( i, channel, semWRITE ) != OK )
				return( ERROR );
	    	memset( shmem, TERMINATORVALUE, SHMMAX );
			memcpy( &size, message, 4 );
	    	memcpy( shmem, message, size+4 );
	    	(void) V( i, channel, semREAD );
	    	return( OK );
		}
    }
    return( ERROR );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	shmReceive	  
 *
 * Arguments:	handle, reference to connection (IN).
 *				message, reference to management communication (OUT)
 *
 * Purpose:		This function recieves the management communication over
 *				connection from the remote application.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
shmReceive( int handle, char *message )
{
    int i, channel, size;
    char *shmem;

    for( i = 0; i < CONNECTMAX; i++ ) {
		if( connectTable[i].handle == handle ) {
	    	if( strstr( listenfile, connectTable[i].address ) == NULL ) {
				channel = channel1;
				shmem = connectTable[i].shmem1;
	    	} else {
				channel = channel2;
				shmem = connectTable[i].shmem2;
	    	}
	    	if( P( i, channel, semREAD ) != OK )
				return( ERROR );
			memcpy( &size, shmem, 4 );
	    	memcpy( message, shmem, size+4 );
	    	V( i, channel, semWRITE );
	    	return( OK );
		}
    }
    return( ERROR );
}




/* 
 * -----------------------------------------------------------------------
 *
 * Function:	shmClose
 *
 * Arguments:	handle, reference to connection (IN)
 *
 * Purpose:		This function closes the connection.
 *				- remove semaphores on memory
 *				- if api side -> (api closes connection to spm)
 *				remove mutex semaphore on listen and connectfile
 *		  		else if spm side ->
 *				if handle = 0 (spm closes spm)
 *				remove mutex semaphore on listen and connectfile
 *
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
shmClose( int handle  )
{
    int i;

    for( i = 0; i < CONNECTMAX; i++ ) {
		if( connectTable[i].handle == handle ) {
	    	semctl( connectTable[i].semid, 2, IPC_RMID, 0 ); 
	    	shmctl( connectTable[i].shmid1, IPC_RMID, 0 );
	    	shmctl( connectTable[i].shmid2, IPC_RMID, 0 );
	    	if( ( handle == 0 ) && 
				( strcmp(connectTable[i].address, spmaddress) == 0 )) { 
				semctl( mutexid, 2, IPC_RMID, 0 );
	    	} 
	    	connectTable[i].handle = connectTable[i].shmid1 = DEFAULTVALUE;
	    	connectTable[i].shmid2 = connectTable[i].semid = DEFAULTVALUE;
	    	connectTable[i].shmem1 = connectTable[i].shmem2 = (void *) NULL;
			shmFileClose( connectTable[i].address );
	    	connectTable[i].address[0] = TERMINATORVALUE;
	    	return( OK );
		}
    }
    return( ERROR );
} 


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	shmFileClose
 *
 * Arguments:	localAddress, reference to local application (IN)
 *
 * Purpose:		This function closes the listen endpoint from the local
 *				application.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
shmFileClose( char *localAddress )
{
    ICOMMAND command;

    if( strlen( localAddress ) == 0 )
		return( OK );
    sprintf( command, "rm %s%s", pathPrefix, localAddress );
    if (strcmp( localAddress, spmAddress ) == 0) {
		PM(mutexid, SPMMUTEX);
		if( system( command ) < 0 ) { 
	    	VM(mutexid, SPMMUTEX);
	    	return( ERROR );
		}
		VM(mutexid, SPMMUTEX);
    } else {
		PM(mutexid, APIMUTEX);
		if( system( command ) < 0 ) { 
	   		VM(mutexid, APIMUTEX);
	   		return( ERROR );
		}
		VM(mutexid, APIMUTEX);
    }
    return( OK ); 
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	shmRemoteExist
 *
 * Arguments:	Address, reference to remote side (IN)
 *
 * Purpose:		This function checks if the remote connectfile exists
 *				If Address = "" -> return OK, because remote side is SPM
 *
 * Remarks:		The function is called ONLY by the SPM.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
shmRemoteExist( char *Address )
{
    IPATH path = pathPrefix;	  /* "/tmp/.ipc." */
    FILE *fp;

    if( strlen( Address ) == 0 )
		return( OK );
    (void) strcat( path, Address );
    PM(mutexid, APIMUTEX);
    if( (fp = fopen( path, "r" ))  == (FILE *) NULL ) {
		VM(mutexid, APIMUTEX);
		return( ERROR );
    } else {
		VM(mutexid, APIMUTEX);
		fclose(fp);
		return( OK );
    }
}


/*
 * -----------------------------------------------------------------------
 *
 * Function:	P	(Dijkstra's semaphores)
 *
 * Arguments:	i, 	index to connection (IN)
 *		whichChannel, 	reference to buffer, either to spm or 
 *				from spm (IN)
 *		operation,	specifies either an read action or 
 *				write action (IN)>
 *
 * Purpose:	This function checks for permission of the intended
 *		action/operation.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
P( int i, int whichChannel, int operation )
{
	union semun arg;

	if( (arg.val = semctl( connectTable[i].semid, whichChannel, GETVAL)) < 0 ) {
		return( ERROR );
	}
	if( arg.val == operation )
		return( OK );
	return( ERROR );
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	V	(Dijkstra's semaphores)
 *
 * Arguments:	i, 	index to connection (IN)
 *		whichChannel, 	reference to buffer, either to spm or 
 *				from spm (IN)
 *		operation,	specifies either an read action or 
 *				write action (IN)>
 *
 * Purpose:	This function swaps the focus from one side to the other
 *		side of the channel (buffer). Besides this, the permitted
 *		action is flipped, as well.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
V( int i, int whichChannel, int operation )
{
	union semun arg;

	arg.val = operation;
	if( semctl( connectTable[i].semid, whichChannel, SETVAL, arg ) < 0 ) {
		return( ERROR );
	}
	return( OK );
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	PM	(Dijkstra's semaphores)
 *
 * Arguments:	mutex,		semaphore number (IN)
 *
 * Remarks:	wait until semaphore is greater than INITMUTEX
 *		enter critical section and decrement semaphore
 *		
 *
 * -----------------------------------------------------------------------
 */
	STATUS
PM( int *mutexid, int mutex )
{
	union semun arg;

	if( (arg.val = semctl( mutexid, mutex, GETVAL)) < 0) {
		return( ERROR );
	}
	do {
	    if( (arg.val = semctl( mutexid, mutex, GETVAL)) < 0) {
		    return( ERROR );
	    }
	} while (arg.val < INITMUTEX);
	arg.val--;
	if ( semctl( mutexid, mutex, SETVAL, arg ) < 0) {
		return( ERROR );
	}
	return( OK );
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	VM	(Dijkstra's semaphores)
 *
 * Arguments:	mutex,		semaphore number (IN)
 *
 * Remarks:	release critical section and increment semaphore
 *
 * -----------------------------------------------------------------------
 */
	STATUS
VM( int *mutexid, int mutex )
{
	union semun arg;

	if( (arg.val = semctl( mutexid, mutex, GETVAL)) < 0) {
		return( ERROR );
	}
	arg.val++;
	if ( semctl( mutexid, mutex, SETVAL, arg ) < 0) {
		return( ERROR );
	} 
	return( OK );
}



/* -----------------------------------------------------------------------
 */
