/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    spmList.c 
 *
 * Date:        May 1994
 *
 * Purpose:     This file implements the local - and remoterequest-lists.
 *
 * Author:      Frederic P. H. van Hengstum, Vincent Berkhout.
 *
 * Remarks: 	Asn1malloc and free replaced by "usual" mallocs.
 *		The malloc and free routines are not "shelled" to be full
 *		(fool) proof. Best list to implement for this case would
 *		be a double linked list: front insertion, last deletion
 *		Remark date: 8 May 1994 by Vincent berkhout.
 *
 * -----------------------------------------------------------------------
 */
#include <spmList.h>



/* 
 * -------------------------------------------------------------------
 *
 * Function:    SpmInitAdminMib
 *
 * Arguments:   none 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     assign the value of some global "constants"
 *
 * Remarks:	
 *
 * -------------------------------------------------------------------
 */
	STATUS
SpmInitAdminMib ()
{
	register short i;

	remoteTable=NULL;
	localTable=NULL;
	if( getenv( "SNMP_DEBUG" ) )
		if( strstr( (char *) getenv( "SNMP_DEBUG" ), "admin" ) )
			SNMP_DEBUG_ADMIN = TRUE;
      	return OK ;
}


/*  
 * -----------------------------------------------------------------------
 *
 * Function:    MibInsertLocalEntry
 *
 * Arguments:   AsnInt reqId, 		(in) 
 * 		AsnOcts *address, 	(in) 
 * 		AsnInt domain		(in) 
 *
 * Returns:	STATUS
 *
 * Purpose:    Insert a logical pdu in the local administration.
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MibInsertLocalEntry( AsnInt reqId, UAsnInt src, UAsnInt dst, UAsnInt ctx,
		     AsnOcts *address, AsnInt domain )
{
	struct timeval tp;
	LOCALENTRY *p=NULL, *n=NULL ;

	(void) MibUpdateLocalEntry();

	/* last insertion, so newest are last in list */
	for( p = localTable; p!=NULL; p=p->next )
		if( p->reqId == reqId ) return FAILED;

	if( gettimeofday( &tp, NULL ) == -1 ) return FAILED;

	n = (void *) malloc( sizeof( LOCALENTRY ) );
	if (n == NULL ) 
		return FAILED;

	n->reqId=reqId;
	n->spmId=0; 
	strncpy( n->address, address->octs, 6 );
	n->domain=domain;
	n->src=src;
	n->dst=dst;
	n->ctx=ctx;
	n->timestamp=tp.tv_sec; 
	n->next=localTable ; 
	localTable=n ;

	return OK ;
}


/*  
 * -----------------------------------------------------------------------
 *
 * Function:    MibInsertLocalId
 *
 * Arguments:   
 *
 * Returns:
 *
 * Purpose:    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MibInsertLocalId( AsnInt reqId, AsnInt spmId )
{
	struct timeval tp;
	LOCALENTRY *p=NULL;

	for( p=localTable; p!=NULL; p=p->next )
		if( p->reqId == reqId ) {
			p->spmId=spmId;
			return OK ;
		}
	return FAILED ;
}


/*  
 * -----------------------------------------------------------------------
 *
 * Function:    MibDeleteLocalEntry
 *
 * Arguments:   AsnInt spmId, 		(in) 
 * 		AsnInt *reqId, 		(out) 
 * 		AsnOcts **address, 	(out) 
 * 		AsnInt *d		(out)
 *
 * Returns:	STATUS ok on succes, FAILED on error
 *
 * Purpose:    removce entry from local list
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MibDeleteLocalEntry( AsnInt spmId, AsnInt *reqId, UAsnInt *src, UAsnInt *dst,
		     UAsnInt *ctx, AsnOcts *address, AsnInt *d )
{
	LOCALENTRY *p=NULL, *n=NULL;

	for( p=localTable; p!=NULL; n=p, p=p->next )
		if( p->spmId == spmId ) {
			*reqId=p->reqId; 
			strncpy( address->octs, p->address, 6 ); 
			address->octetLen = 6;
			*d=p->domain;
			*src=p->src; *dst=p->dst; *ctx=p->ctx;
			if( localTable==p )
				localTable = p->next ;
			else
				n->next= p->next;
			free( p );
			return OK ;
		}
	return FAILED ;
}


/*  
 * -----------------------------------------------------------------------
 *
 * Function:    MibUpdateLocalEntry
 *
 * Arguments:   none
 *
 * Returns:	STATUS OK on succes, FAILED otherwise
 *
 * Purpose:	periodically update the outstanding request list and remove
 * 		outdated "pdus"
 *
 * Remarks:	Returns OK, even if no entries were deleted!
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MibUpdateLocalEntry( )
{
	struct timeval tp;
	int	timestamp;
	LOCALENTRY *p=NULL;
	AsnInt	a, b; 
	UAsnInt z, y, w;
	AsnOcts *c=pNewOct(10);


 	if( gettimeofday( &tp, NULL ) == -1 )
		return FAILED;
	timestamp = tp.tv_sec;

	for( p=localTable; p!=NULL; p=p->next )
		if( (p->timestamp+LOCALTIMEOUT) < timestamp ) {
			MibDeleteLocalEntry( p->spmId, &a, &z, &w, &y, c, &b ); 
			FreeOct( c );
			/* fall through */
		}
	return OK;
}


/*  
 * -----------------------------------------------------------------------
 *
 * Function:    MibInsertRemoteEntry
 *
 * Arguments:   
 *
 * Returns:
 *
 * Purpose:    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MibInsertRemoteEntry( AsnInt appId, AsnInt spmId, AsnInt type, IADDRESS address )
{
	struct timeval tp;
	REMOTEENTRY *p=NULL, *n=NULL ;


	for( p = remoteTable; p!=NULL; p=p->next )
		if( p->appId == appId ) return FAILED;

	if( gettimeofday( &tp, NULL ) == -1 ) return FAILED;
	n = (void *) malloc( sizeof( REMOTEENTRY ) );

	n->timestamp=tp.tv_sec; 
	n->appId=appId; 
	n->spmId=spmId;
	n->type=type; 
	strcpy( n->address, address );
	n->next=remoteTable;

	remoteTable=n;

	return OK ;
}


/*  
 * -----------------------------------------------------------------------
 *
 * Function:    MibDeleteRemoteEntry
 *
 * Arguments:   
 *
 * Returns:
 *
 * Purpose:    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MibDeleteRemoteEntry( AsnInt spmId, AsnInt *appId, AsnInt *type, IADDRESS address )
{
	REMOTEENTRY *p=NULL, *n=NULL;

	for( p=remoteTable; p!=NULL; n=p, p=p->next )
		if( p->spmId == spmId ) {
			*appId=p->appId; 
			*type=p->type; 
			strcpy( address, p->address );
			if( remoteTable==p ) 
				remoteTable=p->next;  /* is NULL */
			else
				n->next=p->next; 
			free( p );
			return OK ;
		}
	return FAILED ;
}


/*  
 * -----------------------------------------------------------------------
 *
 * Function:    MibUpdateRemoteEntry
 *
 * Arguments:   
 *
 * Returns:
 *
 * Purpose:    
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
MibUpdateRemoteEntry( AsnInt *appId, IADDRESS address )
{
	struct timeval tp;
	int timestamp;
	REMOTEENTRY *p;
	AsnInt a;


 	if( gettimeofday( &tp, NULL ) == -1 )
		return FAILED;
	timestamp = tp.tv_sec;

	for( p=remoteTable; p!=NULL; p=p->next )
		if( (p->timestamp + LOCALTIMEOUT) < timestamp ) {
			MibDeleteRemoteEntry( p->spmId, appId, &a, address );
			return OK;
		}
	return FAILED ;
}


/* --------------------- end of file spmList.c ------------------------- */

