/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    apiMSG.c
 *
 * Date:        May 1994
 *
 * Purpose:     This file implements the Interface functions, which 
 *		are needed accessability of the incoming-message 
 *		database, such as open, close, send and receive ... :)
 *
 * Author(s):   Vincent Berkhout, Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */

#include <apiMSG.h>

/*
 * 	local variables
 */
static 	MSGTABLE	msgTable ;

/* 
 * -----------------------------------------------------------------------
 *
 * Function: 	msgOpen
 *
 * Arguments:	mHandle, reference to messagebuffer (OUT).
 *		mMsg, reference to messagebuffer (IN/OUT).
 *
 * Purpose:	This function allocates space for a message by following
 *		the next steps:
 *		1) Initialize the message bufferpool, once.
 *		2) Allocate a messagebuffer?
 *			ERROR ->	Function returns ERROR.
 *		3) Function returns OK
 *
 * Remarks:	When mMsg contains the NULL value, no poAsnInter reference to
 *		the allocated messagebuffer is returned.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
msgOpen( AsnInt *mHandle, MSG **mMsg )
{
	static init = FALSE;
	AsnInt i;

	if( init == FALSE ) {
		for( i = 0; i < MSGTABLEMAX; i++ )
			/* msgCache[i].type = */
			msgTable[i].type = DEFAULTVALUE;
		init = TRUE;
	}
	for( i = 0; i < MSGTABLEMAX; i++ )
		if( msgTable[i].type == DEFAULTVALUE ) {
			*mHandle = i;
			if( mMsg != (MSG **) NULL )
				*mMsg = &msgTable[i];
			return( OK );
		}
	return( ERROR );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	msgPut
 *
 * Arguments:	mType, type of message (IN).
 *		...
 *		mHandle, reference to messagebuffer (IN).
 *
 * Purpose:	This function fills the referenced messagebuffer with
 *		the provided parameterinformation and puts it AsnInto the
 *		messageTable.
 *		Steps:
 *		1) Belongs buffer to bufferpool?
 *			ERROR -> 	Function returns ERROR.
 *		2) Is buffer already filled ?
 *			ERROR -> 	Function returns ERROR.
 *		3) Fill buffer.
 *		4) Function returns OK.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
msgPut( SPTYPE mType, char *mSysName, QOS mQos, char *mConName, 
	AsnInt mReqId, VarBindList *mVarBinds, AsnInt mErrStat, 
	AsnInt mErrInd, AsnInt mHandle )
{
	AsnInt i = mHandle;


	if( (mHandle < 0) || (mHandle > MSGTABLEMAX) ) 
		return ERROR ;
	/* if( msgTable[i].type != DEFAULTVALUE )
	 *	return( ERROR );
	 */

	if( mType != (SPTYPE) NULL )
		msgTable[i].type = mType;
	if( mSysName != (char *) NULL )
		(void) strcpy( msgTable[i].systemname, mSysName );
	if( mQos != (QOS) NULL )
		msgTable[i].qos = mQos;
	if( mConName != (char *) NULL )
		(void) strcpy( msgTable[i].contextname, mConName );
	if( mReqId != (AsnInt) NULL )
		msgTable[i].request_id = mReqId;
	if( mErrStat != (AsnInt) NULL )
		msgTable[i].errorstatus = mErrStat;
	if( mErrInd != (AsnInt) NULL )
		msgTable[i].errorindex = mErrInd;
	if( mVarBinds != (VarBindList *) NULL ) 
		DynToStatVbl( mVarBinds, msgTable[i].variablebindings );
	
	return OK ;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	msgGet
 *
 * Arguments: 	mType, type of message (IN).
 *		...
 *		mHandle, reference to messagebuffer (OUT).
 *		mMsg, reference to message (OUT).
 *
 * Purpose:	This fucntion searches for message with type mType. If found,
 *		the parameters will be filled with information contained in
 *		the message.
 *		Steps:
 *		1) Either search for message with specific type?
 *			ERROR ->	Function returns ERROR
 *		2) Or search for first message in bufferpool ?
 *			ERROR ->	Function returns ERROR
 *		3) Fill parameters.
 *		4) Function returns OK.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
msgGet( SPTYPE *mType, char *mSysName, QOS *mQos, char *mConName, 
	AsnInt *mReqId, VarBindList *mVarBinds, AsnInt *mErrStat, 
	AsnInt *mErrInd, AsnInt *mHandle )
{
	AsnInt i;

	if( (mType != (SPTYPE *) NULL) && (*mType != 0) ) {
		for( i = 0; i < MSGTABLEMAX; i++ )
		    	if( *mType == msgTable[i].type )
				break;
		if( i == MSGTABLEMAX )
			return( ERROR );
	}
	if( (mType != (SPTYPE *) NULL) && (*mType == 0) ) {
		for( i = 0; i < MSGTABLEMAX; i++ )
		    	if( DEFAULTVALUE != msgTable[i].type )
				break;
		if( i == MSGTABLEMAX )
			return( ERROR );
	}

	if( mType != (SPTYPE *) NULL )
		*mType = msgTable[i].type;
	if( mSysName != (char *) NULL )
		(void) strcpy( mSysName, msgTable[i].systemname );
	if( mQos != (QOS *) NULL )
		*mQos = msgTable[i].qos;
	if( mConName != (char *) NULL )
		(void) strcpy( mConName, msgTable[i].contextname );
	if( mReqId != (AsnInt *) NULL )
		*mReqId = msgTable[i].request_id;
	if( mErrStat != (AsnInt *) NULL )
		*mErrStat = msgTable[i].errorstatus;
	if( mErrInd != (AsnInt *) NULL )
		*mErrInd = msgTable[i].errorindex;
	if( mHandle != (AsnInt *) NULL )
		*mHandle = i;
	if( mVarBinds != (VarBindList *) NULL ) 
		StatToDynVbl( msgTable[i].variablebindings, mVarBinds );

	return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	msgClose
 *
 * Arguments:	mHandle, reference to messagebuffer.
 *
 * Purpose:	Function clears messagebuffer.
 *		Steps:
 *		1) Messagebuffer belongs to bufferpool.
 *			ERROR ->	Function returns ERROR.
 *		2) Messagebuffer is active/in use ?
 *			ERROR ->	Function returns ERROR.
 *		3) Clear messagebuffer.
 *		4) Function returns OK.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
msgClose( AsnInt mHandle )
{
	if( (mHandle < 0) ||(mHandle > MSGTABLEMAX) )
		return( ERROR );
	if( msgTable[mHandle].type == DEFAULTVALUE )
		return( ERROR );
	msgTable[mHandle].type = DEFAULTVALUE;
	return( OK );
}



/* -----------------------------------------------------------------------
 */
