/* -----------------------------------------------------------------------
 *
 * Project:		UT-SNMP
 *
 * Filename:    spmAWM.c
 *
 * Date:        May 1994
 *
 * Purpose:     This file implements the Application Interface Module.
 *
 * Author(s):   Vincent Berkhout, Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */

#include <spmAWM.h>

static	 struct	timeval	startTime;
static char *spmAddress="spm";

/* 
 * -----------------------------------------------------------------------
 *
 * Function:    AimOpen
 *
 * Arguments: 	localAddress, here "spm". (IN) 	 
 *
 * Return:		STATUS, OK on success and FAILED otherwise
 *
 * Purpose:    	This function opens the Application Interface Module
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AimOpen( )
{
        /*      initialize nibble mem on a per message base
	InitNibbleMem( 64*1024, 512);
	 */

	if( simOpen( spmAddress, (char *) "" ) != OK ) {
		return( ERROR );
	}
	if( CrOpen( ) != OK ) {
		return( ERROR );
	}
    (void) gettimeofday(&startTime, (struct timezone *)0);
	(void) memset( applicationTable, 0x00, MAXAPPLICATIONS*IADDRESSMAX );
	return OK;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    AimClose
 *
 * Arguments:   none.
 *
 * Purpose:     This function close the Application Interface Module.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AimClose( )
{
	if( simClose( spmAddress ) != OK ) 
		return FAILED;
	
	if( CrClose( ) != OK ) 
		return FAILED;
	return OK;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    AimPutAddress
 *
 * Arguments:   
 *
 * Purpose:     
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AimPutAddress( IADDRESS remoteAddress, APPLICATIONTABLE appTable )
{
	register short i;

	for( i = 0; i < MAXAPPLICATIONS; i++ ) {
		if( strcmp( appTable[i], remoteAddress ) == 0 ) {
			(void) memset( appTable[i], 0x00, IADDRESSMAX );
			(void) simClose( remoteAddress );
			applicationsConnected--;
			TRACE( SNMP_DEBUG_AIM, "%s-//-spm", remoteAddress );
			if( ! applicationsConnected ) {
				return( ERROR );
			}
			return FAILED;
		}
		if( strcmp( appTable[i], "" ) == 0 ) {
			applicationsConnected++;
			(void) strcpy( appTable[i], remoteAddress );
			TRACE( SNMP_DEBUG_AIM, "%s<-->spm", remoteAddress );
			return OK;
		}
	}

	return FAILED;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    AimPutIds
 *
 * Arguments:   
 *
 * Purpose:     
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AimPutIds( IADDRESS rAddress, PDUsChoiceId mType, AsnInt *mReqId )
{
	switch( (int) mType ) {

	case PDUS_GET_REQUEST:
	case PDUS_GET_NEXT_REQUEST:
	case PDUS_GET_BULK_REQUEST:
	case PDUS_INFORM_REQUEST:
	case PDUS_SET_REQUEST:
	case PDUS_SNMPV2_TRAP:

		if( MibInsertRemoteEntry( *mReqId, ++spmIds, mType, rAddress )
			!= OK )
			return FAILED ;
		*mReqId = spmIds;
		return OK ;
	}
	return FAILED ;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    AimError
 *
 * Arguments:   
 *
 * Purpose:     
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AimError( IADDRESS rAddress, AsnInt mHandle, MSG *mMsg,  ERRTYPE mErrTyp )
{
	char m[32*1024];
	int size;

	(void) msgPut( ERR_IND,NULL,(QOS)NULL,NULL,(AsnInt)NULL,NULL,mErrTyp,
		       (AsnInt) NULL, mHandle );
	memcpy( m+4, mMsg, size=sizeof( MSG ) );
	memcpy( m, &size, 4 );
	simSend( rAddress, (char *) m );
	return OK;
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:    AimConvert
 *
 * Arguments:   
 *
 * Purpose:     
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AimConvert( IADDRESS mSysName, IADDRESS mConName, 
	    AsnOcts *tAddress, AsnOcts **localEntity )
{
	struct hostent *he ;
	u_long addr;

	*localEntity = NewOct( 32 );

 	he = gethostbyname( mSysName );
	if( he == NULL ) {
		return FAILED;
	}

	addr = inet_addr( inet_ntoa( *((struct in_addr *)(he->h_addr)) ) );

	(tAddress)->octetLen = 4;
	(void) memcpy( (tAddress)->octs, &addr, (tAddress)->octetLen );

	(*localEntity)->octetLen = strlen( mConName );
	memcpy( (*localEntity)->octs, mConName, (*localEntity)->octetLen );

	return OK;
}
	    

/* 
 * -----------------------------------------------------------------------
 *
 * Function:    AimDispatch
 *
 * Arguments:   
 *
 * Purpose:     
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AimDispatch( IADDRESS rAddress, SPTYPE mType, VarBindList *mVarBinds,
	     int mHandle, MSG *mMsg )
{

	AsnInt mErrType=0, mErrInd=0;
	AsnOid *oid = NewOid(255);
	ObjectSyntax	objectval;
	ReqType reqtype;
	VarBind *vb;
	char m[32*1024];
	int size;

	switch( mType ) {

	case META_UNDO_REQ : 
		mType = META_UNDO_CNF; reqtype = P_UNDO; 
		break;

	case META_COMMIT_REQ : 
		mType = META_COMMIT_CNF; reqtype = P_COMMIT; 
		break;
	
	case META_GET_REQ : 
		mType = META_GET_CNF; reqtype = P_GET; 
		break;

	case META_GETNEXT_REQ : 
		mType = META_GETNEXT_CNF; reqtype = P_GET_NEXT; 
		break;

	case META_SET_REQ : 
		mType = META_SET_CNF; reqtype = P_SET;
		break;

	default: 
		return FAILED;
	}
	(void) MibVLRequest( mVarBinds, reqtype, &mErrType);
    (void) msgPut( mType, NULL, (QOS) NULL, NULL, (AsnInt) NULL,
	       mVarBinds, mErrType, mErrInd, mHandle );
    memcpy( m+4, mMsg, size=sizeof( MSG ) );
    memcpy( m, &size, 4 );
    (void) simSend( rAddress, (char *) m );
    return OK;
}

eric(){}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:    AimReceive
 *
 * Arguments:   
 *
 * Purpose:     
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AimReceive( AsnInt *version, UAsnInt *mSrcId, UAsnInt *mDstId, UAsnInt *mCtxId, AsnInt *mType, 
	    AsnInt *mReqId, AsnInt *mErrStat, AsnInt *mErrInd, 
	    VarBindList **mVarBinds, AsnOcts *tAddress, AsnOcts *community )
{
	IADDRESS rAddress, mConName, mSysName;
	AsnOcts *localEntity;
	MSG* mMsg = NULL;
	char m[32*1024];
	AsnInt mHandle = 0;
	QOS mQos = 0;
	struct timeval now; 
	UAsnInt upTime;
	VarBind *vb;
	AsnOid	*tmpOid = NULL;
	int size;

	*mReqId = *mErrStat = *mErrInd = 0 ;
	
	if( msgOpen( &mHandle, &mMsg ) != OK ) {
		return FAILED;
	}

	if( MibUpdateRemoteEntry( mReqId, rAddress ) == OK ) {
		mMsg->request_id = *mReqId;
		(void) AimError( rAddress, mHandle, mMsg, SnmpErrTIMEOUT );
		(void) msgClose( mHandle );
		mHandle = 0;
		(void) msgOpen( &mHandle, &mMsg ) ;
	}

	if( simReceive( rAddress, (char *) m ) != OK ) {
		(void) msgClose( mHandle );
		return FAILED;
	}

	eric();
    memcpy( &size, m, 4 );
	if ( size == 0 )
		return FAILED;
	memcpy( mMsg, m+4, size );

	*mReqId = 0;
	*mType = 0;
	*mErrStat = 0;
	*mErrInd = 0;
	*mVarBinds = NewVbl( );

	(void) msgGet( (SPTYPE *) mType, mSysName, &mQos, mConName, mReqId,
			*mVarBinds, mErrStat, mErrInd, &mHandle );

	switch( *mType ) {
	case TRAP_REQ :
		
/*!!
 *!!		Just to show how it works, this code is NOT CORRECT
 * 		see rfc 1448.txt for more information 
 */ 
		if( *mType == TRAP_REQ ) *mType = PDUS_SNMPV2_TRAP;

		if( CrMapTrap( NULL, mSrcId, mDstId, mCtxId ) != OK )	
			return FAILED ;

		if (MibGetTrapNumber( *mDstId, mReqId ) != OK )
            return FAILED ;

		(void) msgClose( mHandle );
		return OK ;

	case INFORM_REQ :
		/*!! all imform dependencies are resolved in the application !!
		   Here a informMapper should be called 
		 */
	case GET_REQ :
	case GETNEXT_REQ :
	case GETBULK_REQ :
	case SET_REQ :

		if( AimConvert( mSysName, mConName, tAddress, &localEntity ) != OK ) {
			AimError( rAddress, mHandle, mMsg, SnmpErrNOSUCHHOST );
			(void) msgClose( mHandle );
			return FAILED;
		}

		if( *mType == GET_REQ ) *mType = PDUS_GET_REQUEST;
		else if( *mType == GETNEXT_REQ ) *mType = PDUS_GET_NEXT_REQUEST;
		else if( *mType == GETBULK_REQ ) *mType = PDUS_GET_BULK_REQUEST;
		else if( *mType == INFORM_REQ ) *mType = PDUS_INFORM_REQUEST;
		else if( *mType == SET_REQ ) *mType = PDUS_SET_REQUEST;

		if( CrMap( *mType, mQos, currentTime, tAddress, localEntity,
			   mSrcId, mDstId, mCtxId, version, community ) != OK ) {
			(void) AimError( rAddress, mHandle, mMsg, SnmpErrQOSFAILED );
			(void) FreeOct( localEntity );
			(void) msgClose( mHandle );		
			return FAILED;
		}

		if( AimPutIds( rAddress, *mType, mReqId ) != OK ) {
			(void) AimError( rAddress, mHandle, mMsg, SnmpErrUNKNOWN );
			(void) msgClose( mHandle );
			return FAILED;
		}

		(void) msgClose( mHandle );
		return OK;

	case GET_RSP :
	case GETNEXT_RSP :
	case GETBULK_RSP :
	case SET_RSP :
		*mType = PDUS_RESPONSE;
		(void) msgClose( mHandle );
		return OK;

	case AGENT_ADM :
	case MANAGER_ADM :
	case AGENTANDMAN_ADM :
		if( AimPutAddress( rAddress, applicationTable ) == ERROR )
			return ERROR;
		(void) msgClose( mHandle );
		return FAILED;

	case META_GET_REQ :
	case META_GETNEXT_REQ :
	case META_SET_REQ :
		(void) AimDispatch( rAddress, *mType, *mVarBinds, mHandle, mMsg );
		(void) msgClose( mHandle );
		return FAILED ;
	}

	(void) AimError( rAddress, mHandle, mMsg, SnmpErrUNKNOWN );
	(void) msgClose( mHandle );
	return FAILED;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:    AimSend
 *
 * Arguments:   
 *
 * Purpose:     
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
AimSend( AsnInt version,  UAsnInt mSrcId, UAsnInt mDstId, UAsnInt mCtxId, PDUsChoiceId mType, 
	 AsnInt mReqId, AsnInt mErrStat, AsnInt mErrInd, VarBindList *mVarBinds )
{
	AsnOcts	*mSysName, *mConName;
	char m[32*1024];
	int size;
	char	name[256];
	AsnInt	mHandle = 0;
	MSG	*mMsg = NULL;
	AsnInt	sp; 			/*service primitive */
	UAsnInt	dummy;
	

	mSysName = NewOct( 256 ); 
	memset ( mSysName->octs, '\0', 256 );
	mConName = NewOct( 256 ); 
	memset ( mConName->octs, '\0', 256 );

	if( msgOpen( &mHandle, &mMsg ) != OK ) {
		return FAILED;
	}

	if (version == SNMPVERSION1) { /* special case for V1 message */
		switch( mType ) {
    	case PDUS_GET_REQUEST : sp = GET_IND; break;
	    case PDUS_GET_NEXT_REQUEST : sp = GETNEXT_IND ; break;
	    case PDUS_SET_REQUEST : sp = SET_IND;  break;
	    case PDUS_RESPONSE :
			if( MibDeleteRemoteEntry( mReqId, &mReqId, (AsnInt *) &sp,
                      mSysName->octs ) != OK )
				return FAILED;
			switch (sp) {
	        case PDUS_GET_REQUEST : sp = GET_CNF; break;
   			case PDUS_GET_NEXT_REQUEST : sp = GETNEXT_CNF ; break;
			case PDUS_SET_REQUEST : sp = SET_CNF; break;
			default: 
				Error("response type not supported") ;
				return FAILED ;
			}
		default: 
			Error("service primitive not supported");
			return FAILED;
		}

		(void) msgPut( sp, NULL, (QOS) NULL, NULL, ++spmIds,
			mVarBinds, (AsnInt) NULL, (AsnInt) NULL, mHandle);
		(void) MibInsertLocalId( mReqId, spmIds );
        memcpy( m+4, mMsg, size=sizeof( MSG ) );
        memcpy( m, &size, 4 );
		(void) simSend( "v1Application" , (char *) m );
		(void) msgClose( mHandle );
		return OK;
	}


	switch( mType ) {

	case PDUS_GET_REQUEST :

		(void) msgPut( GET_IND, NULL, (QOS) NULL, NULL, ++spmIds, 
				mVarBinds, (AsnInt) NULL, (AsnInt) NULL, mHandle );
		(void) MibInsertLocalId( mReqId, spmIds );
		(void) MibGetContextLocalEntity( mCtxId, mSysName ); 
		memcpy( m+4, mMsg, size=sizeof( MSG ) );
    	memcpy( m, &size, 4 );
		(void) simSend( mSysName->octs, (char *) m );
		(void) msgClose( mHandle );
		return OK;

	case PDUS_GET_NEXT_REQUEST :
		(void) msgPut( GETNEXT_IND, NULL, (QOS) NULL, NULL, 
			       ++spmIds, mVarBinds,
			       (AsnInt) NULL, (AsnInt) NULL, mHandle );
		(void) MibInsertLocalId( mReqId, spmIds );
		(void) MibGetContextLocalEntity( mCtxId, mSysName );
		memcpy( m+4, mMsg, size=sizeof( MSG ) );
    	memcpy( m, &size, 4 );
		(void) simSend( mSysName->octs, (char *) m );
		(void) msgClose( mHandle );
		return OK;

	case PDUS_GET_BULK_REQUEST :
		(void) msgPut( GETBULK_IND, NULL, (QOS) NULL, NULL, 
				++spmIds, mVarBinds,
			       mErrStat, mErrInd, mHandle );
		(void) MibInsertLocalId( mReqId, spmIds );
		(void) MibGetContextLocalEntity( mCtxId, mSysName );
		memcpy( m+4, mMsg, size=sizeof( MSG ) );
    	memcpy( m, &size, 4 );
		(void) simSend( mSysName->octs, (char *) m );
		(void) msgClose( mHandle );
		return OK;

	case PDUS_SET_REQUEST : 
		(void) msgPut( SET_IND, NULL, (QOS) NULL, NULL, 
				++spmIds, mVarBinds,
			       (AsnInt) NULL, (AsnInt) NULL, mHandle );
		(void) MibInsertLocalId( mReqId, spmIds );
		(void) MibGetContextLocalEntity( mCtxId, mSysName );
        memcpy( m+4, mMsg, size=sizeof( MSG ) );
        memcpy( m, &size, 4 );
		(void) simSend( mSysName->octs, (char *) m );
		(void) msgClose( mHandle );
		return OK;

	case PDUS_INFORM_REQUEST :
		(void) MibGetContextLocalEntity( mCtxId, mConName );	
		(void) MibGetPartyTAddress( mSrcId, mSysName );	
		sprintf( name, "%u.%u.%u.%u", mSysName->octs[0], 
			 mSysName->octs[1], mSysName->octs[2], mSysName->octs[3] );
		(void) msgPut( INFORM_IND, name, (QOS) NULL, (char *) mConName->octs, ++spmIds, 
			       mVarBinds, (AsnInt) NULL, (AsnInt) NULL, mHandle );
        memcpy( m+4, mMsg, size=sizeof( MSG ) );
        memcpy( m, &size, 4 );
		(void) simSend( "snmpAgent" /* :( */, (char *) m );
		(void) msgClose( mHandle );
		return OK;

	case PDUS_SNMPV2_TRAP :
		
		(void) MibGetContextLocalEntity( mCtxId, mConName );	
		(void) msgPut( TRAP_IND, name, (QOS) NULL, (char *) mConName->octs,
			 ++spmIds, mVarBinds, (AsnInt) NULL, (AsnInt) NULL, mHandle );
        memcpy( m+4, mMsg, size=sizeof( MSG ) );
        memcpy( m, &size, 4 );
		(void) simSend( "snmpAgent" /* :( */, (char *) m );

		if (MibGetTrapNumber( mSrcId, &dummy ) != OK )
            return FAILED ;

		(void) msgClose( mHandle );
		return OK;

	case PDUS_RESPONSE :
		if( MibDeleteRemoteEntry( mReqId, &mReqId, (AsnInt *) &mType, 
					  mSysName->octs ) != OK )
			return FAILED;

		switch( mType )
		{

		case PDUS_GET_REQUEST :
			(void) msgPut( GET_CNF, NULL, (QOS) NULL, NULL, mReqId, mVarBinds,
			               mErrStat, mErrInd, mHandle );
        	memcpy( m+4, mMsg, size=sizeof( MSG ) );
        	memcpy( m, &size, 4 );
			(void) simSend( mSysName->octs, (char *) m );
			(void) msgClose( mHandle );
			return OK;

		case PDUS_GET_NEXT_REQUEST :
			(void) msgPut( GETNEXT_CNF, NULL,(QOS)  NULL, NULL, mReqId, mVarBinds,
			               mErrStat, mErrInd, mHandle );
            memcpy( m+4, mMsg, size=sizeof( MSG ) );
            memcpy( m, &size, 4 );
			(void) simSend( mSysName->octs, (char *) m );
			(void) msgClose( mHandle );
			return OK;

		case PDUS_GET_BULK_REQUEST :
			(void) msgPut( GETBULK_CNF, NULL, (QOS) NULL, NULL, mReqId, mVarBinds,
			               mErrStat, mErrInd, mHandle );
            memcpy( m+4, mMsg, size=sizeof( MSG ) );
            memcpy( m, &size, 4 );
			(void) simSend( mSysName->octs, (char *) m );
			(void) msgClose( mHandle );
			return OK;

		case PDUS_SET_REQUEST :
			(void) msgPut( SET_CNF, NULL, (QOS) NULL, NULL, mReqId, mVarBinds,
			       	       mErrStat, mErrInd, mHandle );
            memcpy( m+4, mMsg, size=sizeof( MSG ) );
            memcpy( m, &size, 4 );
			(void) simSend( mSysName->octs, (char *) m );
			(void) msgClose( mHandle );
			return OK;

		case PDUS_INFORM_REQUEST :
			(void) msgPut( INFORM_CNF, NULL,(QOS)  NULL, NULL, mReqId, mVarBinds,
			       (AsnInt)NULL, (AsnInt)NULL, mHandle );
            memcpy( m+4, mMsg, size=sizeof( MSG ) );
            memcpy( m, &size, 4 );
			(void) simSend( mSysName->octs, (char *) m );
			(void) msgClose( mHandle );
			return OK ;
		default:
			break ;
		}
	default:
		break ;
	}

	return FAILED ;
}



/* -----------------------------------------------------------------------
 */
  
