/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    per_bits.c
 *
 * Date:        May 1994.
 *
 * Purpose:     Interface to PER modules.
 *
 * Author(s):   Rene Oude Vrielink
 *
 * -----------------------------------------------------------------------
 */

/*
 * The BIT STRING in SNMPv2 is to be found in SimpleSyntax
 * There are NO lenght constraints to this BIT STRING this
 * means that the bit-strings are octet-aligned (see PER-CD 14)
 */

#include <snacc/asn_config.h>
#include <snacc/asn_incl.h>
#include <snmpTypes.h>
#include <perv2/per_buf.h>
#include <perv2/per_bits.h>


/* -------------------------------------------------------
 * Function:	PerEncBits
 * Arguments:	b, ...... (IN/OUT)
 *		bits, ....... (IN)
 * Returns:	The lenght of the octet-algned bit field in bits 
 * Purpose:	Encode the bit-string 'bits' into the buffer
 * Remarks:	This function is only for the ALIGNED variant of PER
 * ---------------------------------------------------------
 */

AsnLen
PerEncBits(BUF_TYPE b, AsnBits *bits)
{
 unsigned long int	byteLen;

 if (bits->bitLen == 0)
	byteLen = 0;
 else 
 {
	byteLen = ((bits->bitLen-1) / 8) + 1;
 }
 PerPutSeg( b, bits->bits, byteLen );
 return(bits->bitLen);
}

/* -------------------------------------------------------
 * Function:	PerDecBits
 * Arguments:	b, ...... (IN)
 *		bits, ....... (OUT)
 *		bitLen, ..... (IN)
 * Returns:	STATUS	
 * Purpose:	Decode the bit-string 'bits' from the buffer
 * Remarks:	This function is only for the ALIGNED variant
 *		of PER. The space for bits must be allocated
 *		before the function call!!!!!!!!!!
 * ---------------------------------------------------------
 */

	STATUS
PerDecBits(BUF_TYPE b, AsnBits *bits, AsnLen bitLen) 
{
 unsigned long int	byteLen;

 byteLen = ((bitLen-1) / 8) + 1;

 bits->bitLen = bitLen;
 bits->bits = Asn1Alloc(byteLen);
 PerCopy( bits->bits, b, byteLen);
 return( OK );
}

 
