/* -----------------------------------------------------------------------
 *
 * Project:	UT-SNMP
 *
 * Filename:    spmPartyMIB.c
 *
 * Date:        May 1994
 *
 * Purpose:     This file contains the primairy partymib functions.
 *
 * Author( s):  Koert W. G. Flapper
 *
 * -----------------------------------------------------------------------
 */

#include <spmPartyMIB.h>

/* For testing purposes; the path is not correct outside testing dir 
 * Those settings used as default, but can be overruled by user specific setting. */
static char PARTY_MIB_FILE[256]   = "../../../../etc/party.conf";
static char CONTEXT_MIB_FILE[256] = "../../../../etc/context.conf";
static char ACL_MIB_FILE[256] = "../../../../etc/acl.conf";
static char VIEW_MIB_FILE[256]    = "../../../../etc/view.conf";

static AsnInt NextPIndex=1, NextCIndex=1, NextVIndex=1, findex2lindex[256];

int	psize, pradix, csize, cradix;

/* ^L
 * -----------------------------------------------------------------------
 * Function:	SpmInitPartyMIB
 * Arguments:	none.
 * Purpose:     Initializes the Party MIB.		
 * Remarks:
 * -----------------------------------------------------------------------
 */
	STATUS
SpmInitPartyMIB () 
{
	if (MibInit( partymibtab, partymsize, partytabtab, partytsize) == OK)
		return(PartyMibOpen());
	return(ERROR);
}



;

/* 
 * -----------------------------------------------------------------------
 *
 * Function:    OpenPartyFile
 *
 * Arguments:   filname, filename .
 *
 * Purpose:     check if filename exists, and if it an regular file.
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
OpenPartyFile( char *filname, char *type, FILE **fp )
{
	struct stat buf;

	if( stat( filname, &buf ) == 0 ) {
		if( S_ISREG( buf.st_mode ) ) {
			*fp = fopen( filname, type );
			return(OK);
		}
	}
	return( FAILED );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	PartyMibOpen
 *
 * Arguments:	none.
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     Initializes the Party MIB.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
PartyMibOpen()
{
	char	*cp;

	cp = getenv("SNMP_PARTYMIB_PATH");

	if( cp != (char *) NULL ) {
		strcpy(PARTY_MIB_FILE, cp);
		strcat(PARTY_MIB_FILE, "/party.conf" );
                strcpy(CONTEXT_MIB_FILE, cp);
                strcat(CONTEXT_MIB_FILE, "/context.conf");
                strcpy(ACL_MIB_FILE, cp);
		strcat(ACL_MIB_FILE, "/acl.conf" );
                strcpy(VIEW_MIB_FILE, cp);
                strcat(VIEW_MIB_FILE, "/view.conf" );
	}
	
	/*	Initializing PartyTable.
	 */
	if( LoadPartyTable( PARTY_MIB_FILE ) != OK ) {
		Error( "Failed to load partytable" );
		return(ERROR);
	}


	/*	Initializing ContextTable 
	*/
	if( LoadContextTable( CONTEXT_MIB_FILE ) != OK ) {
		Error( "Failed to load contexttable" );
		return( ERROR );
	}


	/*	Initializing AclTable 
	 */
	if( LoadAclTable( ACL_MIB_FILE ) != OK ) { 
		Error( "Failed to load acltable" );
		return(ERROR);
	}


	/*	Initializing Viewtable.
	 */
	if( LoadViewTable( VIEW_MIB_FILE ) != OK ) { 
		Error( "Failed to load viewtable" );
		return( ERROR );
	}


	return( OK );
}	



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	PrintPartyMIB
 *
 * Arguments:	none.
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     PrAsnInts the Party MIB.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
PrintPartyMIB()
{
	PartyEntry	*pp;
	ContextEntry	*cp;
	AclEntry	*ap;
	ViewEntry	*vp;

	fprintf(stderr, "The partytable looks like this\n");
	fprintf(stderr, "Ind\t Name\t\t\t   Local TAddress\t\tID\n");
	pp = PartyMIB.parties;
	while (pp) {
		PrintPartyEntry( pp);
		pp = pp->next;
		}
	fprintf(stderr, "\n");

	fprintf(stderr, "The contexttable looks like this\n");
	fprintf(stderr, "Ind\t Name\t\t\t  Local ID\n");
	cp = PartyMIB.contexts;
	while (cp) { 
		PrintContextEntry( cp);
		cp = cp->next;
		}
	fprintf(stderr, "\n");

	fprintf(stderr, "The acltable looks like this\n");
	fprintf(stderr, "TInd\t SInd\t RInd\t Privileges\n");
	ap = PartyMIB.acls;
	while( ap) {
		PrintAclEntry( ap);
	      	ap = ap->next;
	}
	fprintf(stderr, "\n");


	fprintf(stderr, "The viewtable looks like this\n");
	fprintf(stderr, "Ind\t Subtree \t\t     Type Mask\n");
	vp = PartyMIB.views;
	while( vp) { 
		PrintViewEntry( vp);
	      	vp = vp->next;
	}
	fprintf(stderr, "\n");
	return(OK);
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	LoadPartyTable
 *
 * Arguments:	filename, filename for party file (IN).
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     This function loads the party table.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS 
LoadPartyTable( char *filename )
{ 
	FILE	*fp=NULL;
	char	*cp=NULL, buf[BUFSIZE], buf1[BUFSIZE], buf2[BUFSIZE], buf3[BUFSIZE], 
		*ucp=NULL, hostname[MAXHOSTNAMELEN];
	AsnInt 	blank, nonhex, state=IDENTITY_STATE;
	UAsnInt	tmpint;
	PartyEntry *party=NULL;
	AsnOid 	*partyid=NULL;
	u_long 	addr, byte, myaddr;
	u_short port;
	struct hostent	*he=NULL;


	if( OpenPartyFile( filename,"r", &fp ) != OK ) {
		Error( "Error reading file %s", PARTY_MIB_FILE );
		return ERROR ;
	}
	psize = 101;
	pradix = 21;
	phashtab = (struct Hashnode **)malloc(psize *sizeof(struct Hashnode *));
	hashclear(phashtab, psize);

    	while( fgets( buf, BUFSIZE, fp ) ) 
	{
		if( strlen(buf) > 250 ) {
		    	Error( "Line longer than 250 bytes" );
			return ERROR;
		}
		if( buf[0] == '#' ) continue;
		blank = TRUE;
		for( cp = buf; *cp; cp++ )
		    	if( !isspace( *cp ) ) {
				blank = FALSE;
				break;
		    	}
		if( blank ) continue;
	
		switch( state ) 
		{
		case IDENTITY_STATE:
			if( sscanf( buf, "%s %s", buf1, buf2) != 2) {
				Error( "Bad parse" );
				return ERROR;
			}
		   	partyid = pNewOid(strlen(buf1)) ; 
			if( ASCIItoOID(buf2, partyid) != OK ) {
				Error( "Bad party object ID" );
				FreeOid( partyid);
				return ERROR ;
		   	}
			if( GetPartyIndex(partyid, &tmpint) == OK) {
				Error( "partyid already in use");
				FreeOid( partyid);
				return ERROR;
			}
			if( CreatePartyEntry( partyid, &party ) != OK ) {
				Error( "Failed to create new party" );
				FreeOid( partyid);
				return ERROR ;
			}
			partyid = NULL;
			party->partyName = (char *)malloc(sizeof(buf1)+1);
			(void) memcpy( party->partyName, buf1, strlen( buf1 )+1 );
		    	(void) memset( buf1, 0, (size_t) BUFSIZE );
		    	state = TRANSPORT_STATE;
		    	break;

		case TRANSPORT_STATE:
		    	if( sscanf( buf, "%s %s %s", buf1, buf2, buf3 ) != 3 ) {
				Error( "Bad parse" );
				FreeParty(party);
				return ERROR ;
			}
			if( ! (int) strcasecmp( buf1, "snmpUdpDomain") ) {
				party->partyTDomain = pNewOid( snmpUDPDomain.octetLen);
				party->partyTDomain->octetLen = snmpUDPDomain.octetLen;
				memcpy( party->partyTDomain->octs, snmpUDPDomain.octs,
				party->partyTDomain->octetLen);
			} else if (!strcasecmp( buf1, "snmpTcpDomain") ) {
				party->partyTDomain = pNewOid( snmpTCPDomain.octetLen );
				party->partyTDomain->octetLen = snmpTCPDomain.octetLen;
				memcpy( party->partyTDomain->octs, snmpTCPDomain.octs,
				party->partyTDomain->octetLen);
			} else if (!strcasecmp( buf1, "rfc1157Domain") ) {
				party->partyTDomain = pNewOid( rfc1157Domain.octetLen);
				party->partyTDomain->octetLen = rfc1157Domain.octetLen;
				memcpy( party->partyTDomain->octs, rfc1157Domain.octs,
				party->partyTDomain->octetLen);
			} else	{
				Error( "Bad domain type" );
				FreeParty(party);
				return ERROR ;
			}
			party->partyTAddress = pNewOct(6);
			party->partyTAddress->octetLen = 6;
		    	if( (addr = inet_addr( buf2 )) == -1 ) {
				Error( "Bad IP address" );
				FreeParty(party);
				return ERROR ;
			}
			addr = htonl(addr);
			memcpy( party->partyTAddress->octs, &addr, 4 );
		    	for( cp = buf3; *cp; cp++ )
				if( !isdigit( *cp ) ) {
					Error( "Not a port number" );
					FreeParty(party);
					return ERROR ;
				}
		    	port = htons(atoi(buf3));
			memcpy( party->partyTAddress->octs + 4, &port, 2 );
		    	memset( buf1, 0, (size_t) BUFSIZE );
		    	memset( buf2, 0, (size_t) BUFSIZE );
		    	memset( buf3, 0, (size_t) BUFSIZE );
		    	state = PROTOCOL_STATE;
		    	break;

		case PROTOCOL_STATE:
		    	if (sscanf(buf, "%s %s", buf1, buf2) != 2) {
				Error( "Bad parse" );
				FreeParty(party);
				return ERROR ;
			}
		    	if( !strcasecmp( buf1, "noAuth") ) {
				party->partyAuthProtocol = pNewOid( noAuth.octetLen);
				party->partyAuthProtocol->octetLen = noAuth.octetLen;
				memcpy( party->partyAuthProtocol->octs, noAuth.octs,
party->partyAuthProtocol->octetLen );
		    	} else if( !strcasecmp(buf1, "snmpv2MD5Auth")) {
				party->partyAuthProtocol = pNewOid( v2md5AuthProtocol.octetLen);
				party->partyAuthProtocol->octetLen = v2md5AuthProtocol.octetLen;
				memcpy( party->partyAuthProtocol->octs, v2md5AuthProtocol.octs,
party->partyAuthProtocol->octetLen );
		    	} else if( !strcasecmp( buf1, "rfc1157noAuth") ) {
				party->partyAuthProtocol = pNewOid( rfc1157noAuth.octetLen);
				party->partyAuthProtocol->octetLen = rfc1157noAuth.octetLen;
				memcpy( party->partyAuthProtocol->octs, rfc1157noAuth.octs,
party->partyAuthProtocol->octetLen );
			} else	{
				Error( "Bad authentication protocol type" );
				FreeParty(party);
				return ERROR ;
			}
		    	if( !strcasecmp( buf2, "noPriv" ) ) {
				party->partyPrivProtocol = pNewOid( noPriv.octetLen);
				party->partyPrivProtocol->octetLen = noPriv.octetLen;
				memcpy( party->partyPrivProtocol->octs, noPriv.octs,
party->partyPrivProtocol->octetLen );
		    	} else 	if( !strcasecmp( buf2, "desPriv" ) ) {
				party->partyPrivProtocol = pNewOid( desPrivProtocol.octetLen);
				party->partyPrivProtocol->octetLen = desPrivProtocol.octetLen;
				memcpy( party->partyPrivProtocol->octs, desPrivProtocol.octs,
party->partyPrivProtocol->octetLen );
		    	} else {
				Error( "Bad privacy protocol type" );
				FreeParty(party);
				return ERROR ;
			}
		    	memset( buf1, 0, (size_t) BUFSIZE);
		    	memset( buf2, 0, (size_t) BUFSIZE);
		    	state = LIFETIME_STATE;
		    	break;


	 	case LIFETIME_STATE:
		    	if( sscanf( buf, "%s %s", buf1, buf2 ) != 2 ) {
				Error( "Bad parse" );
				FreeParty(party);
				return ERROR ;
			}
		    	for( cp = buf1; *cp; cp++ )
				if( !isdigit( *cp ) ) {
			    		Error( "Bad lifetime value" );
					FreeParty(party);
					return ERROR ;
				}
		    	party->partyAuthLifetime = atoi( buf1 );
		    	if( ( party->partyAuthLifetime < 0 ) || ( party->partyAuthLifetime > MAXLIFETIME) ) {
				Error("Bad auth lifetime value" );
				FreeParty(party);
				return ERROR ;
			}
		    	for( cp = buf2; *cp; cp++ )
				if( !isdigit(*cp) ) {
					Error( "Bad Max Message Size value" );
					FreeParty(party);
					return ERROR ;
				}
		    	party->partyMaxMessageSize = atoi( buf2 );
		    	if( ( party->partyMaxMessageSize < MINMAXMS ) || ( party->partyMaxMessageSize > MAXMAXMS ) ) {
				Error( "Bad Max Message Size value" );
				FreeParty(party);
				return ERROR ;
			}
		    	memset( buf1, 0, (size_t) BUFSIZE );
		    	memset( buf2, 0, (size_t) BUFSIZE );
		    	state = CLOCK_STATE;
		    	break;	    

	  	case CLOCK_STATE:
		    	if( sscanf( buf, "%s", buf1 ) != 1 ) {
				Error( "Bad parse" );
				FreeParty(party);
				return ERROR ;
			}
		    	if( strlen(buf1) != 8 ) {
				Error( "Bad clock value" );
				FreeParty(party);
				return ERROR ;
			}
		    	for( cp = buf1; *cp; cp++ ) 
				if( !isxdigit(*cp) ) {
			    		Error( "Bad clock value" );
					FreeParty(party);
					return ERROR ;
				}
		    	if( sscanf( buf1, "%x", &party->partyAuthClock ) != 1 ) {
				Error( "Bad clock value" );
				FreeParty(party);
				return ERROR ;
			}
		    	memset( buf1, 0, (size_t) BUFSIZE );
		    	state = AUTH_STATE;
		    	break;

	  	case AUTH_STATE:
		    	if( sscanf( buf, "%s %s", buf1, buf2 ) != 2 ) {
				Error( "Bad parse" );
				FreeParty(party);
				return ERROR ;
			}
			if (CMPOID( party->partyAuthProtocol, &rfc1157noAuth) == EQUAL) {
				party->partyAuthPrivate= pNewOct(strlen(buf1));
				party->partyAuthPrivate->octetLen = strlen(buf1);
				memcpy(party->partyAuthPrivate->octs, buf1, party->partyAuthPrivate->octetLen);
			} else {
		    		if( strlen(buf1) != 32 ) {
					Error( "Bad private authentication key value" );
					FreeParty(party);
					return ERROR ;
				}
			    	for(cp = buf1; *cp; cp++)
					if (!isxdigit(*cp)) {
			    			Error( "Bad private authentication key value" );
						FreeParty(party);
						return ERROR ;
					}
		    		ucp = ( char *) ((party->partyAuthPrivate=(AsnOcts *)pNewOct(KEYLEN))->octs);
		    		for( cp = buf1; *cp; cp += 2, ucp++ ) {
					if( sscanf( cp, "%2x", &byte ) != 1 ) {
			    			Error( "Bad private authentication key value" );
						FreeParty(party);
						return ERROR ;
					}
					*ucp = byte;
			     	}
				party->partyAuthPrivate->octetLen = (AsnInt) ucp - (AsnInt)
party->partyAuthPrivate->octs;

				nonhex = 0;
				for( cp = buf2; ((*cp) && (!(nonhex))); cp++ ) {
					if( !isxdigit( *cp ) )
				    		nonhex = 1;
			    	}
		    		if( nonhex ) {
					if( strcasecmp(buf2, "Null") ) {
			    			Error( "Bad public authentication key value" );
						FreeParty(party);
						return ERROR ;
					}
			    	} else {
					if( (strlen(buf2) % 2) || ( strlen(buf2) > 32) ) {
			    			Error( "Bad public authentication key value" );
						FreeParty(party);
						return ERROR ;
					}
					ucp = (char *) ((party->partyAuthPublic=(AsnOcts *)pNewOct(KEYLEN))->octs);
					for( cp = buf2; *cp; cp += 2, ucp++ ) {
			    			if( sscanf( cp, "%2x", &byte) != 1 ) {
							Error( "Bad public authentication key value" );
							FreeParty(party);
							return ERROR ;
						}
			    			*ucp = byte;
					}
					party->partyAuthPublic->octetLen = (AsnInt) ucp - (AsnInt)
party->partyAuthPublic->octs;
				}
			}
		    	memset( buf1, 0, (size_t) BUFSIZE );
		    	memset( buf2, 0, (size_t) BUFSIZE );
		   	state = PRIV_STATE;
		    	break;

	  	case PRIV_STATE:
		    	if (sscanf(buf, "%s %s", buf1, buf2) != 2) {
				Error( "Bad parse" );
				FreeParty(party);
				return ERROR ;
			}
		    	if( strlen(buf1) != 32 ) {
				Error( "Bad private privacy key value" );
				FreeParty(party);
				return ERROR ;
			}
			for( cp = buf1; *cp; cp++ ) {
				if( !isxdigit(*cp) ) {
			    		Error( "Bad private privacy key value" );
					FreeParty(party);
					return ERROR ;
				}
		    	}
		    	ucp = ( char * ) (party->partyPrivPrivate=pNewOct(KEYLEN) )->octs;
		    	for( cp = buf1; *cp; cp += 2, ucp++) {
				if( sscanf(cp, "%2x", &byte ) != 1 ) {
			    		Error( "Bad private privacy key value" );
					FreeParty(party);
					return ERROR ;
				}
				*ucp = byte;
		    	}
			party->partyPrivPrivate->octetLen = (AsnInt) ucp -
(AsnInt)party->partyPrivPrivate->octs;

		    	nonhex = 0;	
		    	for(cp = buf2; ((*cp) && (!(nonhex))); cp++) {
				if( !isxdigit( *cp ) )
			    		nonhex = 1;
		    	}
		    	if( nonhex ) {
				if( strcasecmp( buf2, "Null" ) ) {
					Error( "Bad public privacy key value" );
					FreeParty(party);
					return ERROR ;
				}
			} else {
		        	if( (strlen(buf2) % 2) || ( strlen(buf2) > 32) ) {
			    		Error( "Bad public privacy key value" );
					FreeParty(party);
					return ERROR ;
				}
				ucp = (char *) ((party->partyPrivPublic=(AsnOcts *)pNewOct(KEYLEN))->octs) ;
				for( cp = buf2; *cp; cp += 2, ucp++ ) {
			    		if (sscanf(cp, "%2x", &byte) != 1) {
						Error( "Bad public privacy key value" );
						FreeParty(party);	
					return ERROR ;
					}
				    	*ucp = byte;
				}
       		         	party->partyPrivPublic->octetLen = (AsnInt)ucp -
(AsnInt)party->partyPrivPublic->octs;
	    		}
		    	memset( buf1, 0, (size_t) BUFSIZE);
		    	memset( buf2, 0, (size_t) BUFSIZE);
			state = IDENTITY_STATE;

		    	if ( gethostname(hostname, MAXHOSTNAMELEN) == -1) {
				Error( "Could not get hostname" );
				FreeParty(party);
			  	return ERROR ;
			}
		    	he = gethostbyname(hostname);
		    	if (he != (struct hostent *)0)
	myaddr = inet_addr( inet_ntoa( *((struct in_addr *)(he->h_addr)) ) );
		    	if( ( ( memcmp( party->partyTDomain->octs, snmpUDPDomain.octs, 
				      snmpUDPDomain.octetLen ) == OK ) || 
		    	( memcmp( party->partyTDomain->octs, snmpTCPDomain.octs, 
				      snmpUDPDomain.octetLen ) == OK ) )  && 
			    ( !memcmp( &myaddr, party->partyTAddress->octs, 4 ) ) )
				party->partyLocal = true;
		    	else
				party->partyLocal = false;
		    	party->partyStorageType = Volatile;
		    	party->partyStatus = active;
		    	break;

		default:
			Error( "unknown state" );
			return ERROR ;
		}
		memset( buf, 0, (size_t) BUFSIZE);
    	}

    	if( state != IDENTITY_STATE ) {
		Error( "Unfinished entry at EOF" );
		return ERROR;
	}

    	fclose( fp );
    	return( OK );
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	LoadContextTable
 *
 * Arguments:	filename, reference to contextfile (IN).
 *
 * Returns:	STATUS, which is OK on success
 *
 * Purpose:     This function loads the context table.		
 *
 * Remarks:	Problems on deciding upon value of contextLocal
 *
 * -----------------------------------------------------------------------
 */
	STATUS
LoadContextTable( char *filename )
{
        FILE    *fp;
	ContextEntry	*context;
	char 	buf[BUFSIZE], buf1[BUFSIZE], buf2[BUFSIZE], buf3[BUFSIZE], *cp,
	     	hostname[MAXHOSTNAMELEN];
        u_long 	myaddr, addr;
        AsnInt 	blank, state = IDENTITY_STATE;
	UAsnInt	tmpint;
	AsnOid	*contextid;
    struct hostent  *he;



	if( OpenPartyFile( filename,"r", &fp ) != OK ) {
		Error( "Error reading file %s",CONTEXT_MIB_FILE);
		return FAILED;
	}
	csize = 101;
	cradix = 13;
	chashtab = (struct Hashnode **)malloc( csize* sizeof(struct Hashnode *));
	hashclear(chashtab, csize);
	memset( buf, 0, (size_t) BUFSIZE);
	memset( buf1, 0, (size_t) BUFSIZE);
	memset( buf2, 0, (size_t) BUFSIZE);
	memset( buf3, 0, (size_t) BUFSIZE);


    	while( fgets(buf, BUFSIZE, fp) ) 
	{
	        if( strlen(buf) > 250 ) {
       	     		Error( "Line longer than 250 bytes" );
			return ERROR ;
	     	}
       	 	if (buf[0] == '#')
            		continue;
        	blank = TRUE;
        	for(cp = buf; *cp; cp++)
            		if (!isspace(*cp)){
                		blank = FALSE;
                	break;
            	}
        	if (blank) continue;

        	switch( state ) {

         	case IDENTITY_STATE:
            		if( sscanf( buf, "%s %s", buf1, buf2 ) != 2 ) {
                		Error( "Bad parse" );
				return ERROR ;
			}
            		if( ASCIItoOID( buf2, contextid=pNewOid(strlen(buf2)) ) != 0) {
                		Error( "Bad context object ID" );
				FreeOid(contextid);
				return ERROR ;
			}
			if ( GetContextIndex( contextid, &tmpint) == OK) {
				Error( "ContextIdentity already in use");
				FreeOid(contextid);
				return ERROR;
				}
            		if (CreateContextEntry(contextid, &context) != OK) {
				Error( "Failed to create new context" );
				FreeOid(contextid);
				return ERROR ;
				}
			contextid = NULL;
			context->contextName = (char *)malloc(sizeof(buf1)+1);
			memcpy(context->contextName, buf1, strlen(buf1)+1);
	    		memset( buf1, 0, (size_t) BUFSIZE);
			memset( buf2, 0, (size_t) BUFSIZE);
            		state = VIEW_STATE;
            		break;

          	case VIEW_STATE:
            		if( sscanf( buf, "%s %s %s", buf1, buf2, buf3 ) != 3) {
                		Error("Bad parse");
				FreeContext(context);
				return ERROR ;
			}
            		for( cp = buf1; *cp; cp++ )
                		if (!isdigit(*cp)) {
                    			Error( "View index not an integer" );
					FreeContext(context);
					return ERROR ;
				}
            		context->contextViewIndex = atoi(buf1);

            		if( !strcasecmp( buf2, "Null") )
				context->contextLocalEntity = NULL;
			else {
				context->contextLocalEntity=pNewOct(strlen(buf2));
	    			context->contextLocalEntity->octetLen = strlen(buf2);
				memcpy(context->contextLocalEntity->octs, buf2, 
					context->contextLocalEntity->octetLen);
			}

            		if( !strcasecmp( buf3, "currentTime") ) {
				context->contextLocalTime = pNewOid( currentTime.octetLen);
                		context->contextLocalTime->octetLen = currentTime.octetLen;
				memcpy( context->contextLocalTime->octs, currentTime.octs,
context->contextLocalTime->octetLen);
            		} else if( !strcasecmp(buf3, "restartTime") ) {
				context->contextLocalTime = pNewOid( restartTime.octetLen);
                		context->contextLocalTime->octetLen = restartTime.octetLen;
				memcpy( context->contextLocalTime->octs, restartTime.octs,
context->contextLocalTime->octetLen);
            		} else if( !strcasecmp(buf3, "cacheTime")) {
				context->contextLocalTime = pNewOid( cacheTime.octetLen);
                		context->contextLocalTime->octetLen = cacheTime.octetLen;
				memcpy( context->contextLocalTime->octs, cacheTime.octs,
context->contextLocalTime->octetLen);
	    		} else {
                		Error( "Bad local time value" );
				FreeContext(context);
				return ERROR ;
			}
	    		memset( buf1, 0, (size_t) BUFSIZE );
			memset( buf2, 0, (size_t) BUFSIZE );
	    		memset( buf3, 0, (size_t) BUFSIZE );
            		state = PROXY_STATE;
            		break;

          	case PROXY_STATE:
            		if( sscanf(buf, "%s %s %s", buf1, buf2, buf3) != 3) {
                		Error( "Bad parse" );
				FreeContext(context);
				return ERROR ;
			}
			if( !strcasecmp(buf1, "Null") || (!memcmp(buf1, "0", 1)))
				context->contextProxyDstParty = NULL;
            		else 	if (GetPartyByName( buf1, &context->contextProxyDstParty) != OK)
					if( ASCIItoOID(buf1, context->contextProxyDstParty = pNewOid(strlen(buf1))) != OK ) {
                				Error( "Bad Proxy Destination object ID" );
						FreeContext(context);
						return ERROR ;
					}
			if( !strcasecmp(buf2, "Null") || (!memcmp(buf2, "0", 1)) )
				context->contextProxySrcParty = NULL;
            		else 	if (GetPartyByName( buf2, &context->contextProxySrcParty) != OK)
					if( ASCIItoOID(buf2, context->contextProxySrcParty = pNewOid(strlen(buf2))) != OK ) {
        		        		Error( "Bad Proxy Source object ID" );
						FreeContext(context);
						return ERROR ;
					}
			if( !strcasecmp(buf3, "Null") || (!memcmp(buf3, ".0.0", 4)) )
				context->contextProxyContext = NULL;
            		else 	if (GetContextByName( buf3, &context->contextProxyContext) != OK)
            				if( ASCIItoOID(buf3, context->contextProxyContext = pNewOid(strlen(buf3))) != OK ) {
                				Error( "Bad Proxy Context object ID" );
						FreeContext(context);
						return ERROR ;
					}
	    		memset( buf1, 0, (size_t) BUFSIZE);
	    		memset( buf2, 0, (size_t) BUFSIZE);
	    		memset( buf3, 0, (size_t) BUFSIZE);
            		state = IDENTITY_STATE;

	    		if ( context->contextViewIndex == 0)
				context->contextLocal = false;
	    		else {
				if ( gethostname(hostname, MAXHOSTNAMELEN) == -1) {
					Error( "Couldn't get hostname");
					return ERROR ;
				}
				(void) OIDtoASCII( context->contextIdentity, buf1);
				(void) sscanf( buf1, "1.3.6.1.6.3.3.1.4.%s", buf2);
				buf2[strlen( buf2) - 2] = '\0';
				addr = inet_addr( buf2);
				he = gethostbyname(hostname);
				if ( he != (struct hostent *) 0)
					myaddr = inet_addr( 
						inet_ntoa(*((struct in_addr *)(he->h_addr))));
				if( !memcmp( &myaddr, &addr, (size_t) 4) )
		    			context->contextLocal = true;
				else context->contextLocal = false;
		    	}
            		if (context->contextViewIndex > 0 )
				context->contextStatus = notReady;
	    		else
				context->contextStatus = active;
			context->contextStorageType = Volatile;
			memset( buf1, 0, (size_t) BUFSIZE);
			memset( buf2, 0, (size_t) BUFSIZE);
            		break;

 		default:
            		Error("unknown state" );
			return ERROR ;
        	}
		memset( buf, 0, (size_t) BUFSIZE);
    	}
    	if( state != IDENTITY_STATE ) {
		Error( "Unfinished entry at EOF" );
		return ERROR ;
	}
    	fclose(fp);
    	return(OK);
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	LoadAclTable
 *
 * Arguments:	filename, refference to acl file (IN).
 *
 * Returns:	STATUS, which is OK on success
 *
 * Purpose:     This function loads the acl table.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS	
LoadAclTable( char *filename )
{
	FILE	*fp;
	char	buf[BUFSIZE], buf1[BUFSIZE], 
		buf2[BUFSIZE], buf3[BUFSIZE], buf4[BUFSIZE],*cp;
	AsnInt	blank, privileges;
	AsnOid	*targetid, *subjectid, *resourcesid;
	UAsnInt	target, subject, resources;
	AclEntry *acl;

        if( OpenPartyFile( filename,"r", &fp ) != OK ) {
                Error( "Error reading file %s", ACL_MIB_FILE );
                return FAILED ;
        }
	
    	while( fgets( buf, BUFSIZE, fp ) ) 
	{
        	if( strlen(buf) > 250 ) {
            		Error( "Line longer than 250 bytes" );
			return ERROR ;
		}
        	if (buf[0] == '#') continue;
        	blank = TRUE;
        	for(cp = buf; *cp; cp++)
            	if (!isspace(*cp)){
                	blank = FALSE;
                	break;
            	}
        	if (blank) continue;
        	if (sscanf(buf, "%s %s %s %s", buf1, buf2, buf3, buf4) != 4){
            		Error("Bad parse");
			return ERROR ;
		}
		memset( buf, 0, (size_t) BUFSIZE);
		if (GetPartyByName( buf1, &target) != OK) {
	    		if( ASCIItoOID( buf1, targetid = pNewOid(32)) != OK) {
				Error( "Bad target party identification " );
				FreeOid(targetid);
				return ERROR ;
			}
			if ( GetPartyIndex( targetid, &target) != OK) {
				Error( "Bad target party identification " );
				FreeOid(targetid);
				return ERROR ;
			}
			else 	FreeOid(targetid);
		}
		memset( buf1, 0, (size_t) BUFSIZE);

		if (GetPartyByName( buf2, &subject ) != OK) {
	    		if ( ASCIItoOID( buf2, subjectid = pNewOid(32)) != OK) {
				Error( "Bad subject party identification" );
				FreeOid(subjectid);
				return ERROR ;
			}
			if ( GetPartyIndex( subjectid, &subject) != OK) {
				Error( "Bad subject party identification " );
				FreeOid(targetid);
				return ERROR ;
			}
			else	FreeOid(subjectid);
		}
		memset( buf2, 0, (size_t) BUFSIZE);

		if (GetContextByName( buf3, &resources) != OK) {
	    		if( ASCIItoOID( buf3, resourcesid = pNewOid(32)) != OK ) { 
				Error( "Bad resources context identification" );
				FreeOid(resourcesid);
				return ERROR ;
			}
			if ( GetContextIndex( resourcesid, &resources) != OK) {
				Error( "Bad resources context identification" );
				FreeOid(resourcesid);
				return ERROR ;
			}
			else	FreeOid(resourcesid);
		}
		memset( buf3, 0, (size_t) BUFSIZE);
		privileges = 0;
        	for(cp = buf4; *cp; cp++){
            		switch(*cp){
              		case 'g': case 'G':
                		privileges |= ACLPRIVILEGESGET; break;
              		case 'n': case 'N':
                		privileges |= ACLPRIVILEGESGETNEXT; break;
              		case 'r': case 'R':
                		privileges |= ACLPRIVILEGESGETRESPONSE; break;
              		case 's': case 'S':
                		privileges |= ACLPRIVILEGESSET; break;
              		case 't': case 'T':
                		privileges |= ACLPRIVILEGESTRAP; break;
              		case 'b': case 'B':
                		privileges |= ACLPRIVILEGESBULK; break;
              		case 'i': case 'I':
                		privileges |= ACLPRIVILEGESINFORM; break;
              		case 'u': case 'U':
                		privileges |= ACLPRIVILEGESTRAP2; break;
              		default:
                		Error( "Bad privileges code" ); 
				return ERROR ;
            		}
        	}
		if( GetAclEntry( target, subject, resources, &acl) == OK ) {
			Error( "An entry with this target, subject and resources already exists");
			return(ERROR);
			}
	    	if( CreateAclEntry( target, subject, resources, &acl) != OK ) {
			Error( "Failed to create new ACL-entry\n");
		  	return(FAILED);
		}
        	acl->aclPrivileges = privileges;
       		acl->aclStorageType = Volatile;
        	acl->aclStatus = active;
    	}
    	fclose(fp);
    	return(OK);
}




/* 
 * -----------------------------------------------------------------------
 *
 * Function:	LoadViewTable
 *
 * Arguments:	filename, reference to view file (IN).
 *
 * Returns:	STATUS, which is OK on success
 *
 * Purpose:     This function  loads the view table.		
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS	
LoadViewTable( char *filename )
{
        FILE    *fp;
	ViewEntry	*view;
	char    buf[BUFSIZE], buf1[BUFSIZE], buf2[BUFSIZE],
		buf3[BUFSIZE], buf4[BUFSIZE], *cp, *ucp; 
	u_long	byte;
	AsnOid	*subtree;
    AsnInt 	blank, nonhex, index, newindex;

	
	if( OpenPartyFile( filename,"r", &fp ) != OK ) {
		Error( "Error reading file %s",VIEW_MIB_FILE );
		return(FAILED);
	} 

	memset( findex2lindex, 0, (size_t) 256);
	memset( buf, 0, (size_t) BUFSIZE);
	memset( buf1, 0, (size_t) BUFSIZE);
	memset( buf2, 0, (size_t) BUFSIZE);
	memset( buf3, 0, (size_t) BUFSIZE);
	memset( buf4, 0, (size_t) BUFSIZE);

    	while (fgets(buf, BUFSIZE, fp))
	{
        	if (strlen(buf) > 250) {
            		Error( "Line longer than 250 bytes" );
			return ERROR ;
		}
        	if (buf[0] == '#') continue;
        	blank = TRUE;
        	for(cp = buf; *cp; cp++)
            	if (!isspace(*cp)){
               	 	blank = FALSE; break;
            	}
        	if (blank) continue;
        	if (sscanf(buf, "%s %s %s %s", buf1, buf2, buf3, buf4) != 4) {
            		Error( "Bad view parse" );
			return ERROR ;
		}
		memset( buf, 0, (size_t) BUFSIZE);
        	for(cp = buf1; *cp; cp++)
            		if (!isdigit(*cp)) {
                		Error( "View index not an Integer" );
				return ERROR ;
			}
        	index = atoi(buf1);
		memset( buf1, 0, (size_t) BUFSIZE);

        	if (ASCIItoOID(buf2, subtree = pNewOid(strlen(buf2))) != 0) {
            		Error("Bad subtree object ID" );
			FreeOid(subtree);
			return ERROR ;
		}
		memset( buf2, 0, (size_t) BUFSIZE);

		if (ViewEntryExists( index, subtree) == OK) {
			Error( "A view-entry with these parameters already exists");
			FreeOid(subtree);
			return ERROR;
			}
		if (CreateViewEntry( index, subtree, &newindex, &view) != OK) {
			Error( "Failed to create new view-entry" );
			FreeOid(subtree);
	      		return(FAILED);
	    	}
		subtree = NULL;

        	if (!strcasecmp(buf3, "included"))
            		view->viewType = VIEWINCLUDED;
        	else if (!strcasecmp(buf3, "excluded"))
		 	view->viewType = VIEWEXCLUDED;
             	else {
                 	Error("Bad ViewType (should be included or excluded)");
			FreeView(view);
			return ERROR ;
		}
		memset( buf3, 0, (size_t) BUFSIZE);
	    	nonhex = 0;	
	    	for(cp = buf4; ((*cp) && (!(nonhex))); cp++){
			if (!isxdigit(*cp))
		    		nonhex = 1;
	    	}
	    	if (nonhex){
			if (strcasecmp(buf4, "Null")) {
		    		Error("Bad view mask value" );
				FreeView(view);
				return ERROR ;
			}
			else view->viewMask = NULL;
	    	} else {
	        	if ( strlen(buf4) > 32) {
		    		Error( "Bad view mask value" );
				FreeView(view);
				return ERROR ;
			}
			ucp = ( char *)((view->viewMask = pNewOct(4))->octs);
			for(cp = buf4; *cp; cp += 2, ucp++){
		    		if (sscanf(cp, "%2x", &byte) != 1) {
					Error( "Bad view mask value" );
					FreeView(view);
					return ERROR ;
				}
		    		*ucp = byte;
			}
                	view->viewMask->octetLen = (AsnInt) ucp - (AsnInt) view->viewMask;
	    	}
		memset( buf4, 0, (size_t) BUFSIZE);
        	view->viewStorageType = Volatile;
        	view->viewStatus = active;
		UpdateContextTable( index, newindex);
    	}
    	fclose(fp);
    	return(OK);
}





/* 
 * -----------------------------------------------------------------------
 *
 * Function:	CreatePartyEntry
 *
 * Arguments:	partyidentity,	name of the party to be created (IN).
 *		newparty, to new created party-entry (OUT).
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     This funtcion creates a new party entry of partyIdentity.		
 *
 * Remarks:	none.
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CreatePartyEntry( AsnOid *partyidentity, PartyEntry **newparty )
{ 
	PartyEntry	*party, *ppp, *pp;
	struct Hashnode	*hashnode;
	int		loc;

	if ((hashnode = ( struct Hashnode *)malloc(sizeof(struct Hashnode))) == (struct Hashnode *)NULL)
		{ Error("No memory available for new party hashnode");
		  return(FAILED);
		}
	if ((party = ( PartyEntry *)malloc(sizeof(PartyEntry))) == (PartyEntry *)NULL)
		{ Error("No memory available for new party");
		  free( hashnode);
		  return(FAILED);
		}
	memset((char *)party, 0, (size_t) sizeof(PartyEntry));
	party->partyIdentity = partyidentity;
	party->partyIndex = NextPIndex++;

	/* Insertion in linked list indexed by partyidentity */
        ppp = pp = PartyMIB.parties;
        while (pp) {
                if (CMPOID( partyidentity, pp->partyIdentity) == LESS)
			break;
		ppp = pp;
                pp = pp->next;
        }
        if (pp == PartyMIB.parties) {
                party->next = PartyMIB.parties;
                PartyMIB.parties = party;
        } else {
                party->next = ppp->next;
                ppp->next = party;
        }

	/* Insertion in hash-table */
	hashnode->pentry = ( PartyEntry *)party;
	loc = hashoid( party->partyIdentity, psize, pradix);
	hashnode->pnext = phashtab[loc];
	phashtab[loc] = hashnode;
	*newparty = party;
	return(OK);
 }

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	FreeParty
 *
 * Arguments:	PartyEntry	*pe, pointer to the party to be removed (IN).
 *
 * Returns:	STATUS, which is OK on success
 *
 * Purpose:     This function frees the memory allocated to the party entry.
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
void FreeParty( PartyEntry *pe)
{
	free(pe->partyName);
	FreeOid( pe->partyIdentity);
	FreeOid( pe->partyTDomain);
	FreeOct( pe->partyTAddress);
	FreeOid( pe->partyAuthProtocol);
	if (pe->partyAuthPrivate)
		FreeOct( pe->partyAuthPrivate);
	if (pe->partyAuthPublic)
		FreeOct( pe->partyAuthPublic);
	FreeOid( pe->partyPrivProtocol);
	if (pe->partyPrivPrivate)
		FreeOct( pe->partyPrivPrivate);
	if (pe->partyPrivPublic)
		FreeOct( pe->partyPrivPublic);
	if ( pe->partyCloneFrom)
		FreeOid( pe->partyCloneFrom);
	free(pe);
	pe = NULL;
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	RemovePartyEntry
 *
 * Arguments:	partyidentity, name of the party to be removed (IN).
 *
 * Returns:	STATUS, which is OK on success
 *
 * Purpose:     This function removes the party entry with partyIdentity.
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
RemovePartyEntry( AsnOid *partyid )
{
	int	loc;
	COMPARETYPE	res;
	struct Hashnode	*phn, *lphn;
	PartyEntry	*pp, *ppp;

	loc = hashoid(partyid, psize, pradix);
	lphn = NULL;
	phn = phashtab[loc];
	while (phn != NULL)
		if ( CMPOID( partyid, ((PartyEntry *)phn->pentry)->partyIdentity) == EQUAL)
			break;
		else {
			lphn = phn;
			phn = phn->pnext;
			}
	if ( phn == NULL )
		return(FAILED);
	if ( lphn == NULL )
		phashtab[loc] = NULL;
	else 
		lphn->pnext = phn->pnext;

	ppp = pp = PartyMIB.parties;
	while (pp) {
		if ( (res = CMPOID( partyid, pp->partyIdentity)) == GREATER)
			return(FAILED);
		else	if ( res == EQUAL)
				break;
		ppp = pp;
		pp = pp->next;
	}

	FreeParty(phn->pentry);
	free(phn);
	return(OK);
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	PrintPartyEntry
 *
 * Arguments:	none.
 *
 * Returns:	STATUS, which is OK on success
 *
 * Purpose:     This function prints a PartyEntry.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS	
PrintPartyEntry( PartyEntry *pe )
{
	char	oid[256];
	u_char	inetaddr[4];
	short	portnr;

	fprintf(stderr, "%-5u", pe->partyIndex);
	fprintf(stderr, "%-30s ",pe->partyName);
	if (pe->partyLocal == true)
	    fprintf(stderr, "True  ");
	else if (pe->partyLocal == false)
		fprintf(stderr, "False ");
	    else fprintf(stderr, "?     ");
	memcpy( inetaddr, pe->partyTAddress->octs, (size_t) 4);
	memcpy( &portnr, pe->partyTAddress->octs+4, (size_t) 2);
	(void) fprintf(stderr, "%u.%u.%u.%u:%u\t", inetaddr[0], inetaddr[1], inetaddr[2], inetaddr[3], portnr);
	OIDtoASCII( pe->partyIdentity, oid);
	fprintf(stderr, "%s\n",oid);
	return(OK);
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:    GetPartyByName
 *
 * Arguments:   partyName	*char		(in)
 *		partyId		*UAsnInt	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves partyid using the partyname
 *
 * -------------------------------------------------------------------
 */
	STATUS	
GetPartyByName( char *name, UAsnInt *index )
{ 
	PartyEntry	*pp;

	pp = PartyMIB.parties;
	while (pp) {
		if ( strcasecmp( pp->partyName, name) == 0 ) {
			*index = (UAsnInt) pp;
			return(OK);
		}
		pp = pp->next;
	}
	return(FAILED);
}	

/* 
 * -------------------------------------------------------------------
 *
 * Function:    GetPartyIndex
 *
 * Arguments:   partyId		OID	(in)
 *		partyIndex	AsnInt	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the party's index identified 
 * 		by  party identifier 
 *
 * Remarks:	none
 *
 *-------------------------------------------------------------------
 */
	STATUS	
GetPartyIndex( AsnOid *partyid, AsnInt *index )
{ 
	int	loc;
	struct Hashnode	*phn;

	loc = hashoid( partyid, psize, pradix);
	phn = phashtab[loc];
	while (phn != NULL) {
		if ( CMPOID( partyid, (((PartyEntry *)phn->pentry)->partyIdentity)) == EQUAL) {
		    *index = (int) phn->pentry;
		    return(OK);
		    }
		else phn = phn->pnext;
	}
	return(FAILED);
}		

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	CreateContextEntry
 *
 * Arguments:	contextidentity, reference to context (IN)
 *		newcontext, created context (OUT).
 *
 * Returns:	STATUS
 *
 * Purpose:     This function creates a new context entry with 
 *		referenced by contextidentity;
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CreateContextEntry( AsnOid *contextidentity, ContextEntry **newcontext )
{ 
	ContextEntry	*context, *pcp, *cp;
	struct Hashnode	*hashnode;
	int		loc;
	
	if ((hashnode = ( struct Hashnode *)malloc(sizeof(struct Hashnode))) == (struct Hashnode *)NULL)
		{ fprintf(stderr, "No memory available for new context hashnode\n");
		  return(FAILED);
		}
	if ((context = ( ContextEntry *)malloc(sizeof(ContextEntry))) == (ContextEntry *)NULL)
		{ fprintf(stderr, "No memory available for new context\n");
		  free( hashnode);
		  return(FAILED);
		}
	memset((char *)context, 0, (size_t) sizeof(ContextEntry));
	context->contextIdentity = contextidentity;
	context->contextIndex = NextCIndex++;

	/* First insertion in linked list */
        pcp = cp = PartyMIB.contexts;
        while (cp) {
                if (CMPOID( contextidentity, cp->contextIdentity) == LESS)
                        break;
                pcp = cp;
                cp = cp->next;
        }
        if (cp == PartyMIB.contexts) {
                context->next = PartyMIB.contexts;
                PartyMIB.contexts = context;
        } else {
                context->next = pcp->next;
                pcp->next = context;
        }

	/* Insertion in hash-table */
	hashnode->pentry = ( ContextEntry *)context;
	loc = hashoid( context->contextIdentity, csize, cradix);
	hashnode->pnext = chashtab[loc];
	chashtab[loc] = hashnode;
	*newcontext = context;
	return(OK);
}

/* 
 * -----------------------------------------------------------------------
 *
 * Function:	FreeContext
 *
 * Arguments:	ContextEntry	*ce, pointer to the context to be removed (IN).
 *
 * Returns:	STATUS, which is OK on success
 *
 * Purpose:     This function frees the memory allocated to the context entry.
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
void FreeContext( ContextEntry *ce)
{
	free(ce->contextName);
	FreeOid( ce->contextIdentity);
	if (ce->contextLocalEntity)
		FreeOct( ce->contextLocalEntity);
	if (ce->contextProxyDstParty)
		FreeOid( ce->contextProxyDstParty);
	if (ce->contextProxySrcParty)
		FreeOid( ce->contextProxySrcParty);
	if (ce->contextProxyContext)
		FreeOid( ce->contextProxyContext);
	free(ce);
	ce = NULL;
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	RemoveContextEntry
 *
 * Arguments:	contextidentity, reference to context (IN).
 *
 * Returns:	STATUS 
 *
 * Purpose:     This function removes the referenced context entry.
 *
 * Remarks:	none
 *
 * -----------------------------------------------------------------------
 */
	STATUS
RemoveContextEntry( AsnOid *contextid )
{ 
	AsnInt	loc;
	struct Hashnode	*phn, *lphn;
	ContextEntry	*cp, *pcp;
	COMPARETYPE		res;

	loc = hashoid(contextid, csize, cradix);
	lphn = NULL;
	phn = chashtab[loc];
	while (phn != NULL) {
		if ( CMPOID( contextid, (((ContextEntry *)phn->pentry)->contextIdentity)) ==
EQUAL)
			break;
		lphn = phn;
		phn = phn->pnext;
	}
	if ( phn == NULL )
		return(FAILED);
	if ( lphn == NULL )
		chashtab[loc] = NULL;
	else 
		lphn->pnext = phn->pnext;

	pcp = cp = PartyMIB.contexts;
	while (cp) {
		if ((res = CMPOID( contextid, cp->contextIdentity)) == GREATER)
                        return(FAILED);
                if ( res == EQUAL)
                        break;
		pcp = cp;
		cp = cp->next;
	}
	if (!cp)
		return(FAILED);
	if ( pcp == PartyMIB.contexts)
		PartyMIB.contexts = cp;
	else
		pcp->next = cp->next;
	FreeContext(phn->pentry);
	free(phn);
	return(OK);
}



/* 
 *-------------------------------------------------------------------
 *
 * Function:    UpdateContextTable
 *
 * Arguments:   index, reference for view entry (IN)
 *		newindex, new reference to view entry (IN).
 *
 * Returns:     STATUS
 *
 * Purpose:     This function updates the contexttable with new 
 *		viewindices
 * 
 * Remarks:	none
 *
 *-------------------------------------------------------------------
 */
	STATUS
UpdateContextTable( AsnInt index, AsnInt newindex )
{ 
	int     i;
	struct Hashnode *htp;

	for (i = 0; i < csize; i++) {
	    htp = chashtab[i];
	    while (htp  != NULL) {
		if ( ((ContextEntry *)htp->pentry)->contextViewIndex == index )
		    ((ContextEntry *)htp->pentry)->contextViewIndex = newindex;
		htp = htp->pnext;
		}
	    }
	return(OK);
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	PrintContextEntry
 *
 * Arguments:	ctable, reference to CONTEXTTABLE (IN)
 *
 * Returns:	STATUS 
 *
 * Purpose:     This function prints a ContextEntry.		
 *
 * Remarks:	none
 *
 * -----------------------------------------------------------------------
 */
	STATUS	
PrintContextEntry( ContextEntry *ce )
{ 
	char	oid[256];

	fprintf(stderr, "%-5u", ce->contextIndex);
	fprintf(stderr, "%-30s",ce->contextName);
	if (ce->contextLocal == true)
	    fprintf(stderr, "True  ");
	else if (ce->contextLocal == false)
		fprintf(stderr, "False ");
	    else fprintf(stderr, "?     ");
	OIDtoASCII( ce->contextIdentity, oid);
	fprintf(stderr, "%s\n",oid); 
	return(OK);
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:    GetContextByName
 *
 * Arguments:   contextName	*char		(in)
 *		contextId	AsnOid	(out) 
 *
 * Returns:     STATUS
 *
 * Purpose:     This function maps the contextname upon contexid.
 *
 * Remarks:	none.
 *
 * -------------------------------------------------------------------
 */
	STATUS
GetContextByName( char *name, UAsnInt *index )
{ 
	ContextEntry	*cp;

	cp = PartyMIB.contexts;
	while (cp) {
		if (strcasecmp(cp->contextName, name) == 0) { 
			*index = (UAsnInt)cp;
			return(OK);
		}
		cp = cp->next;
	}
	return(FAILED);
}

/* 
 * -------------------------------------------------------------------
 *
 * Function:    GetContextIndex
 *
 * Arguments:   contextId		UAsnInt	(in)
 *		contextIndex	AsnInt	(out) 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     Retrieves the context's index identified 
 *              by  context identifier 
 *
 * -------------------------------------------------------------------
 */
	STATUS
GetContextIndex( AsnInt contextid, UAsnInt *index )
{ 
	int	loc;
	struct Hashnode	*phn;

	loc = hashoid( contextid, csize, cradix);
	phn = chashtab[loc];
	while (phn != NULL) {
		if ( CMPOID( (AsnOid *)contextid, (((ContextEntry *)phn->pentry)->contextIdentity))
== EQUAL) {
		    *index = (int)phn->pentry;
		    return(OK);
		    }
		else phn = phn->pnext;
	}
	return(FAILED);
}		


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	GetAclEntry
 *
 * Arguments:	target, subject, name	AsnInteger
 *			indices of the acl to be found (IN).
 *		acl	*AclEntry, poAsnInter to acl-entry if found, 
 *			NULL otherwise (OUT)
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     Searches for a specified acl in the acl database.		
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
GetAclEntry( UAsnInt target, UAsnInt subject, UAsnInt resources,
	     AclEntry **acl )
{ 
	AclEntry	*ap;

	ap = PartyMIB.acls;
	while (ap) {
		if ( ap->aclTarget < target ) {
			ap = ap->next;
			continue;
		} else if ( ap->aclTarget != target )
				return(FAILED);
		if ( ap->aclSubject < subject ) {
			ap = ap->next;
			continue;
		} else if ( ap->aclSubject != subject )
				return(FAILED);
			
		if ( ap->aclResources < resources) {
			ap = ap->next;
			continue;
		} else if ( ap->aclResources != resources)
				return(FAILED);
	}
	if (!ap)
		return(FAILED);
	*acl = ap;
	return(OK);
}		



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	CreateAclEntry
 *
 * Arguments:	target, subject, name	AsnInteger
 *			indices of the acl to be found (IN).
 *		newacl	poAsnInter to new created acl-entry.
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     Creates a new acl entry with aclIdentity = aclidentity.		
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CreateAclEntry( UAsnInt target, UAsnInt subject, UAsnInt resources,
		AclEntry **newacl )
{
	AclEntry	*acl, *ap, *pap;
	
	if ((acl = ( AclEntry *)malloc(sizeof(AclEntry))) == (AclEntry *)NULL)
		{ fprintf(stderr, "No memory available for new acl\n");
		  return(FAILED);
		}
	memset((char *)acl, 0, (size_t)sizeof(AclEntry));
	acl->aclTarget = target;
	acl->aclSubject = subject;
	acl->aclResources = resources;

	/* First insertion of new acl entry */
	pap = ap = PartyMIB.acls;
	while (ap) {
		if ( ((PartyEntry *)ap->aclTarget)->partyIndex > ((PartyEntry *)target)->partyIndex)
			break;
		else	if ( ((PartyEntry *)ap->aclTarget)->partyIndex ==  ((PartyEntry *)target)->partyIndex)
				if ( ((PartyEntry *)ap->aclSubject)->partyIndex >  ((PartyEntry *)subject)->partyIndex)
					break;
				else	if ( ((PartyEntry *)ap->aclSubject)->partyIndex ==  ((PartyEntry *)subject)->partyIndex)
						if ( ((ContextEntry *)ap->aclResources)->contextIndex > ((ContextEntry *)resources)->contextIndex )
							break;
		pap = ap;
		ap = ap->next;
	}
	if (ap == PartyMIB.acls) { 
		acl->next = PartyMIB.acls;
		PartyMIB.acls = acl;
	} else {
		acl->next = pap->next;
		pap->next = acl;
	}
	*newacl = acl;
	return(OK);
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	RemoveAclEntry
 *
 * Arguments:	target, subject, name	AsnInteger
 *			indices of the acl to be found (IN).
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     Removes the acl entry with aclIdentity = aclidentity.		
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
RemoveAclEntry( UAsnInt target, UAsnInt subject, UAsnInt resources )
{
	AclEntry	*ap, *pap;

	pap = ap = PartyMIB.acls;
	while (ap) {
		if ( ap->aclTarget < target ) {
			pap = ap;
			ap = ap->next;
			continue;
		} else if ( ap->aclTarget != target )
				return(FAILED);
		if ( ap->aclSubject < subject ) {
			pap = ap;
			ap = ap->next;
			continue;
		} else if ( ap->aclSubject != subject )
				return(FAILED);
			
		if ( ap->aclResources < resources) {
			pap = ap;
			ap = ap->next;
			continue;
		} else if ( ap->aclResources != resources)
				return(FAILED);
	}
	if (ap == PartyMIB.acls)
		PartyMIB.acls = ap->next;
	else
		pap->next = ap->next;
	free(ap);
	return(OK);
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	PrintAclEntry
 *
 * Arguments:	acle	*AclEntry	(IN)
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     PrAsnInts an AclEntry.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS	
PrintAclEntry( AclEntry	*acle )
{
	fprintf(stderr, "%-12u", ((PartyEntry *)acle->aclTarget)->partyIndex);
	fprintf(stderr, "%-12u", ((PartyEntry *)acle->aclSubject)->partyIndex);
	fprintf(stderr, "%-12u", ((ContextEntry *)acle->aclResources)->contextIndex); 
	fprintf(stderr, "%5d\n", acle->aclPrivileges);
	return(OK);
}

/* 
 * -----------------------------------------------------------------------
 * 
 * Function:	ViewEntryExists
 *
 * Arguments:	index,		AsnInt, index used in conf-file. 
 *		*subtree,	poAsnInter to poAsnInter of new created view-entry.
 *
 * Returns:	STATUS which is OK if a view-entry with these parameters exists
 *
 * Purpose:     Check to see if a view-entry with these parameters already exists
 *		in the partyMIB.		
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
ViewEntryExists( UAsnInt index, AsnOid *subtree)
{
	ViewEntry	*vp;
	UAsnInt		lindex;

	if ( FindNewIndex( index, &lindex) != OK)
	    return(FAILED);

	vp = PartyMIB.views;
	while (vp) { 
		if ((vp->viewIndex == lindex) && (CMPOID( vp->viewSubtree, subtree) == EQUAL))
			return(OK);
		else	vp = vp->next;
	}
	return(FAILED);
}


/* 
 * -----------------------------------------------------------------------
 * 
 * Function:	CreateViewEntry
 *
 * Arguments:	index,		UAsnInt, index used in conf-file. 
 *		*newindex,	UAsnInt to index of the view to be created (IN).
 *		*subtree	AsnOid identifying the subtree (IN)
 *		**newview,	pointer to pointer of new created view-entry.
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     Creates a new view entry with the specified subtree and the new index
 *		and places it at the correct position in the viewtable.		
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
CreateViewEntry( AsnInt	index, AsnOid *subtree, AsnInt *newindex, ViewEntry **newview )
{
	ViewEntry	*view, *pvp, *pv;

	if ( FindNewIndex( index, newindex) != OK)
	    return(FAILED);

	if ((view = ( ViewEntry *)malloc(sizeof(ViewEntry))) == (ViewEntry *)NULL)
		  return(FAILED);
	memset((char *)view, 0, (size_t)sizeof(ViewEntry));

	pvp = pv = PartyMIB.views;
	while (pv) { 
		if (pv->viewIndex > *newindex)
			break;
		else	if (pv->viewIndex == *newindex)
				if (CMPOID( subtree, pv->viewSubtree) == LESS)
					break;
		pvp = pv;
		pv = pv->next;
	}

	view->viewIndex = *newindex;
	view->viewSubtree = subtree;
	if (pv == PartyMIB.views) {
		view->next = PartyMIB.views;
		PartyMIB.views = view;
	} else {
		view->next = pvp->next;
		pvp->next = view;
	}
	*newview = view;
	return(OK);
}


/* 
 * -----------------------------------------------------------------------
 *
 * Function:	FreeView
 *
 * Arguments:	ViewEntry	*ve, pointer to the view to be removed (IN).
 *
 * Returns:	STATUS, which is OK on success
 *
 * Purpose:     This function frees the memory allocated to the view entry.
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
void FreeView( ViewEntry *ve)
{
	FreeOid( ve->viewSubtree);
	if (ve->viewMask)
		FreeOct( ve->viewMask);
	free(ve);
	ve = NULL;
}


/*  
 * -----------------------------------------------------------------------
 *
 * Function:	RemoveViewEntry
 *
 * Arguments:	index,		poAsnInter to index of the view to be created (IN).
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     Removes the view entries with viewIndex = index.		
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
RemoveViewEntry( AsnInt index )
{
	ViewEntry	*vp, *pvp;
	AsnInt		found = 0;

	vp = PartyMIB.views;
	pvp = vp;
	while (vp)
	    { if (vp->viewIndex == index)
		{ if ( vp == PartyMIB.views)
		      PartyMIB.views = vp->next;
		  else pvp->next = vp->next;
		  FreeView( vp);
		  found = 1;
		}
	      else pvp = vp;
	      vp = vp->next;
	    }
	if (found)
	    { findex2lindex[index] = 0; 
	      return(OK);
	    }
	else return(FAILED);
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	FindNewIndex
 *
 * Arguments:	index	short, index of the view in the conf file (IN).
 *		newindex	*short, new index in mib
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     Finds a local index for the indices in the conf file		
 *
 * Remarks:	
 *
 * -----------------------------------------------------------------------
 */
	STATUS
FindNewIndex( AsnInt index, AsnInt *newindex )
{
	if ( (*newindex = findex2lindex[index]) == 0)
		{ *newindex = NextVIndex++;
		  findex2lindex[index] = *newindex;
		}
	return(OK);
}



/* 
 * -----------------------------------------------------------------------
 *
 * Function:	PrintViewEntry
 *
 * Arguments:	vtable	*VIEWTABLE	(IN)
 *
 * Returns:	STATUS which is OK on success
 *
 * Purpose:     PrAsnInts a ViewEntry.		
 *
 * Remarks:
 *
 * -----------------------------------------------------------------------
 */
	STATUS
PrintViewEntry( ViewEntry *vp )
{
	char	oid[256];

	fprintf(stderr, "%-5u", vp->viewIndex);
	OIDtoASCII( vp->viewSubtree, oid);
	fprintf(stderr, "%-25s", oid);
	if ( vp->viewType == VIEWINCLUDED)
	    fprintf(stderr, "incl \n");
	else if ( vp->viewType == VIEWEXCLUDED)
	    fprintf(stderr, "excl \n");
	    else fprintf(stderr, "?    \n");
	return(OK);
}
