/* -----------------------------------------------------------------------
 *
 * Project:     UT-SNMP
 *
 * Filename:    spmSend.c
 *
 * Date:        May 1994
 *
 * Purpose:     Receivinging application message, processing it and then
 * 		sending it to transport address. 
 *
 * Author(s):   Vincent Berkhout, Frederic P. H. van Hengstum
 *
 * -----------------------------------------------------------------------
 */

#include <spmSend.h>



/* 
 * -------------------------------------------------------------------
 *
 * Function:    SpmInitSend
 *
 * Arguments:   none 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     This function initializes the SEND module.
 *
 * Remark:
 *
 * -------------------------------------------------------------------
 */
        STATUS
SpmInitSend ()
{
        return OK ;
}



/* 
 * -------------------------------------------------------------------
 *
 * Function:    SpmInitSend
 *
 * Arguments:   none 
 *
 * Returns:     STATUS  which is OK on succes
 *
 * Purpose:     This function processes an COMMUNICATIUON MESSAGE to
 *		be send.
 *
 * Remark:
 *
 * -------------------------------------------------------------------
 */
	STATUS
SpmSend() 
{
	UAsnInt		src=0, dst=0, ctx=0;
	AsnOcts		*msg=NULL, *remote, *local, *community;
	AsnInt 		version=0, reqId=0, errStat=0, errInd=0, tag=0 ;
	VarBindList	*varBinds=NULL ;
	AsnOid		*dom=NULL ;
	TDOMAIN		domain=0 ;
	STATUS		s;
	struct		hostent *he ;
	long   		hostNr ;    /* internet address  in byte order, 4 bytes */
	short		portNr = 8162;
	char 		*hostname ;

	/*      initialize nibble mem on a per message base
  	 * 		From this moment SNACC memory management will be used!
	 */
 	
	InitNibbleMem( 64*1024, 512);

	remote = NewOct(6) ; memset(remote->octs, 0, 6) ;
	local = NewOct(6) ; memset(local->octs, 0, 6) ;
	community = NewOct(0) ; community->octs = NULL ;
	remote->octetLen = local->octetLen = community->octetLen = 0 ;
	varBinds = NewVbl () ;


	/*	RECEIVE COMMUNICATION MESSAGE FROM APPLICATION.
	 */
	if( (s = AimReceive( &version, &src, &dst, &ctx, &tag, &reqId, &errStat,
		&errInd, &varBinds, remote, community)) != OK ) { 
		ShutdownNibbleMem( );
		return s ;
	}

	if ( version == SNMPVERSION1 ) { /* it's an SNMP version 1 message */ 

		if( SpmWrapV1Message( community, remote, tag, reqId, errStat, 
			errInd, varBinds, &msg ) != OK ) {
			ShutdownNibbleMem();
			return FAILED ;
		}
		hostname = getenv ("HOST") ;	
		he = gethostbyname( hostname );
		if (he != (struct hostent *)0) 
			hostNr = ((struct in_addr *) (he->h_addr ))->s_addr ; 
		memcpy( local->octs, (const void *) &hostNr, 4 ) ;
		memcpy( local->octs+4 , (const void *) &portNr, 2 ) ;
		local->octetLen = 6;
		if (tag == PDUS_RESPONSE)
			if( MibDeleteLocalEntry( reqId, &reqId, &dst, &src, &ctx, remote, 
						 (AsnInt *) &domain ) != OK ) {
				ShutdownNibbleMem( );
				return FAILED ;
			}
	
	} else { 		/* message is an SNMP version 2 message */

		/*	ADMINISTER COMMUNICATION MESSAGE.
		 */
		switch( tag ) 
		{
		case PDUS_GET_REQUEST :
		case PDUS_SET_REQUEST :
		case PDUS_GET_NEXT_REQUEST :
		case PDUS_GET_BULK_REQUEST :
		case PDUS_INFORM_REQUEST :
		case PDUS_SNMPV2_TRAP :
			dom = NewOid(10) ;
			if( (MibGetPartyTAddress( src, local ) != OK) ||
			    (MibGetPartyTAddress( dst, remote ) != OK) || 
				(MibGetPartyTDomain( src, dom ) != OK) ||
				(DomainConvert( dom, &domain ) != OK) ) {
				ShutdownNibbleMem( );
				return FAILED ;
			}

			break ;
		case PDUS_RESPONSE :
			if( MibDeleteLocalEntry( reqId, &reqId, &dst, &src, &ctx, remote, 
						 (AsnInt *) &domain ) != OK ) {
				ShutdownNibbleMem( );
				return FAILED ;
			}
			if( MibGetPartyTAddress( src, local ) != OK ) {
				ShutdownNibbleMem( );
				return FAILED ;
			}
			break ;
		default :
			ShutdownNibbleMem( );
			return FAILED ;
		} 	


		/*	ENCODE COMMUNICATION MESSAGE.
	 	*/
		if( SpmWrapV2Message( src, dst, ctx, tag, reqId, errStat, errInd, varBinds,
			     &msg ) != OK ) {
			ShutdownNibbleMem( );
			return FAILED ;
		}
	}	/* SNMP version 2 message */ 


	/*	SEND COMMUNICATION MESSAGE OVER NETWORK.
	 */
	if( TransportSend( domain, local->octs, remote->octs, msg ) != OK ) {
		ShutdownNibbleMem( );
		return FAILED ;
	}

	ShutdownNibbleMem( );
	return OK;

}	/* end function SpmSend */

/*---------------------- end of file SpmSend.c -----------------------*/


